@@ -64,7 +64,8 @@ def Delay(opts, args):
   op = opcodes.OpTestDelay(duration=delay,
                            on_master=opts.on_master,
                            on_nodes=opts.on_nodes,
-                           repeat=opts.repeat)
+                           repeat=opts.repeat,
+                           no_locks=opts.no_locks)
   SubmitOrSend(op, opts)
 
   return 0
@@ -188,9 +189,8 @@ def _TestJobDependency(opts):
   """
   ToStdout("Testing job dependencies")
 
-  cl = cli.GetClient()
-
   try:
+    cl = cli.GetClient()
     SubmitOpCode(opcodes.OpTestDelay(duration=0, depends=[(-1, None)]), cl=cl)
   except errors.GenericError, err:
     if opts.debug:
@@ -218,6 +218,7 @@ def _TestJobDependency(opts):
                                            ht.TOr(ht.TNonEmptyString,
                                                   ht.TJobId)])))
 
+  cl = cli.GetClient()
   result = cl.SubmitManyJobs(jobs)
   if not check_fn(result):
     raise errors.OpExecError("Job submission doesn't match %s: %s" %
@@ -287,8 +288,6 @@ def _TestJobSubmission(opts):
         (4, 2, priority + offset),
         ])
 
-  cl = cli.GetClient()
-
   for before, after, failpriority in testdata:
     ops = []
     ops.extend([opcodes.OpTestDelay(duration=0) for _ in range(before)])
@@ -296,6 +295,7 @@ def _TestJobSubmission(opts):
     ops.extend([opcodes.OpTestDelay(duration=0) for _ in range(after)])
 
     try:
+      cl = cli.GetClient()
       cl.SubmitJob(ops)
     except errors.GenericError, err:
       if opts.debug:
@@ -311,16 +311,15 @@ def _TestJobSubmission(opts):
        opcodes.OpTestDelay(duration=0, dry_run=True)],
       ops,
       ]
-    result = cl.SubmitManyJobs(jobs)
-    if not (len(result) == 2 and
-            compat.all(len(i) == 2 for i in result) and
-            isinstance(result[0][1], int) and
-            isinstance(result[1][1], basestring) and
-            result[0][0] and not result[1][0]):
-      raise errors.OpExecError("Submitting multiple jobs did not work as"
-                               " expected, result %s" % result)
-    assert len(result) == 2
-
+    try:
+      cl = cli.GetClient()
+      cl.SubmitManyJobs(jobs)
+    except errors.GenericError, err:
+      if opts.debug:
+        ToStdout("Ignoring error for 'wrong priority' test: %s", err)
+    else:
+      raise errors.OpExecError("Submitting manyjobs with an incorrect one"
+                               " did not fail when it should.")
   ToStdout("Job submission tests were successful")
 
 
@@ -595,7 +594,7 @@ def ListLocks(opts, args): # pylint: disable=W0613
     """Format pending acquires.
 
     """
-    return utils.CommaJoin("%s:%s" % (mode, ",".join(threads))
+    return utils.CommaJoin("%s:%s" % (mode, ",".join(map(str, threads)))
                            for mode, threads in value)
 
   # Format raw values
@@ -631,6 +630,9 @@ commands = {
                 action="append", help="Select nodes to sleep on"),
      cli_option("-r", "--repeat", type="int", default="0", dest="repeat",
                 help="Number of times to repeat the sleep"),
+     cli_option("-l", "--no-locks", default=False, dest="no_locks",
+                action="store_true",
+                help="Don't take locks while performing the delay"),
      DRY_RUN_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
     "[opts...] <duration>", "Executes a TestDelay OpCode"),
   "submit-job": (
@@ -673,10 +675,10 @@ commands = {
                 help="Select number of VCPUs for the instance"),
      cli_option("--tags", default=None,
                 help="Comma separated list of tags"),
-     cli_option("--evac-mode", default=constants.IALLOCATOR_NEVAC_ALL,
-                choices=list(constants.IALLOCATOR_NEVAC_MODES),
+     cli_option("--evac-mode", default=constants.NODE_EVAC_ALL,
+                choices=list(constants.NODE_EVAC_MODES),
                 help=("Node evacuation mode (one of %s)" %
-                      utils.CommaJoin(constants.IALLOCATOR_NEVAC_MODES))),
+                      utils.CommaJoin(constants.NODE_EVAC_MODES))),
      cli_option("--target-groups", help="Target groups for relocation",
                 default=[], action="append"),
      cli_option("--spindle-use", help="How many spindles to use",
