@@ -95,7 +95,7 @@ def _ExpandMultiNames(mode, names, client=None):
   # pylint: disable=W0142
 
   if client is None:
-    client = GetClient()
+    client = GetClient(query=True)
   if mode == _EXPAND_CLUSTER:
     if names:
       raise errors.OpPrereqError("Cluster filter mode takes no arguments",
@@ -182,7 +182,8 @@ def GenericManyOps(operation, fn):
     if opts.multi_mode is None:
       opts.multi_mode = _EXPAND_INSTANCES
     cl = GetClient()
-    inames = _ExpandMultiNames(opts.multi_mode, args, client=cl)
+    qcl = GetClient(query=True)
+    inames = _ExpandMultiNames(opts.multi_mode, args, client=qcl)
     if not inames:
       if opts.multi_mode == _EXPAND_CLUSTER:
         ToStdout("Cluster is empty, no instances to shutdown")
@@ -223,10 +224,12 @@ def ListInstances(opts, args):
                                                       for item in value),
                                False))
 
+  cl = GetClient(query=True)
+
   return GenericList(constants.QR_INSTANCE, selected_fields, args, opts.units,
                      opts.separator, not opts.no_headers,
                      format_override=fmtoverride, verbose=opts.verbose,
-                     force_filter=opts.force_filter)
+                     force_filter=opts.force_filter, cl=cl)
 
 
 def ListInstanceFields(opts, args):
@@ -417,9 +420,10 @@ def RemoveInstance(opts, args):
   instance_name = args[0]
   force = opts.force
   cl = GetClient()
+  qcl = GetClient(query=True)
 
   if not force:
-    _EnsureInstancesExist(cl, [instance_name])
+    _EnsureInstancesExist(qcl, [instance_name])
 
     usertext = ("This will remove the volumes of the instance %s"
                 " (including mirrors), thus removing all the data"
@@ -537,6 +541,14 @@ def RecreateDisks(opts, args):
           raise errors.OpPrereqError("Invalid disk size for disk %d: %s" %
                                      (didx, err), errors.ECODE_INVAL)
 
+      if constants.IDISK_SPINDLES in ddict:
+        try:
+          ddict[constants.IDISK_SPINDLES] = \
+              int(ddict[constants.IDISK_SPINDLES])
+        except ValueError, err:
+          raise errors.OpPrereqError("Invalid spindles for disk %d: %s" %
+                                     (didx, err), errors.ECODE_INVAL)
+
       disks.append((didx, ddict))
 
     # TODO: Verify modifyable parameters (already done in
@@ -820,6 +832,7 @@ def MoveInstance(opts, args):
 
   op = opcodes.OpInstanceMove(instance_name=instance_name,
                               target_node=opts.node,
+                              compress=opts.compress,
                               shutdown_timeout=opts.shutdown_timeout,
                               ignore_consistency=opts.ignore_consistency,
                               ignore_ipolicy=opts.ignore_ipolicy)
@@ -840,15 +853,18 @@ def ConnectToInstanceConsole(opts, args):
   instance_name = args[0]
 
   cl = GetClient()
+  qcl = GetClient(query=True)
   try:
     cluster_name = cl.QueryConfigValues(["cluster_name"])[0]
     ((console_data, oper_state), ) = \
-      cl.QueryInstances([instance_name], ["console", "oper_state"], False)
+      qcl.QueryInstances([instance_name], ["console", "oper_state"], False)
   finally:
     # Ensure client connection is closed while external commands are run
     cl.Close()
+    qcl.Close()
 
   del cl
+  del qcl
 
   if not console_data:
     if oper_state:
@@ -875,7 +891,7 @@ def _DoConsole(console, show_command, cluster_name, feedback_fn=ToStdout,
   @param cluster_name: Cluster name as retrieved from master daemon
 
   """
-  assert console.Validate()
+  console.Validate()
 
   if console.kind == constants.CONS_MESSAGE:
     feedback_fn(console.message)
@@ -896,6 +912,7 @@ def _DoConsole(console, show_command, cluster_name, feedback_fn=ToStdout,
 
     srun = ssh.SshRunner(cluster_name=cluster_name)
     ssh_cmd = srun.BuildCmd(console.host, console.user, cmd,
+                            port=console.port,
                             batch=True, quiet=False, tty=True)
 
     if show_command:
@@ -942,10 +959,6 @@ def _FormatDiskDetails(dev_type, dev, roman):
   return data
 
 
-def _FormatListInfo(data):
-  return list(str(i) for i in data)
-
-
 def _FormatBlockDevInfo(idx, top_level, dev, roman):
   """Show block device information.
 
@@ -1043,8 +1056,6 @@ def _FormatBlockDevInfo(idx, top_level, dev, roman):
       data.append(("logical_id", l_id[0]))
     else:
       data.extend(l_id)
-  elif dev["physical_id"] is not None:
-    data.append(("physical_id:", _FormatListInfo(dev["physical_id"])))
 
   if dev["pstatus"]:
     data.append(("on primary", helper(dev["dev_type"], dev["pstatus"])))
@@ -1065,7 +1076,7 @@ def _FormatBlockDevInfo(idx, top_level, dev, roman):
 
 def _FormatInstanceNicInfo(idx, nic):
   """Helper function for L{_FormatInstanceInfo()}"""
-  (name, uuid, ip, mac, mode, link, _, netinfo) = nic
+  (name, uuid, ip, mac, mode, link, vlan, _, netinfo) = nic
   network_name = None
   if netinfo:
     network_name = netinfo["name"]
@@ -1075,6 +1086,7 @@ def _FormatInstanceNicInfo(idx, nic):
     ("IP", str(ip)),
     ("mode", str(mode)),
     ("link", str(link)),
+    ("vlan", str(vlan)),
     ("network", str(network_name)),
     ("UUID", str(uuid)),
     ("name", str(name)),
@@ -1273,6 +1285,9 @@ def _ParseDiskSizes(mods):
 
   """
   for (action, _, params) in mods:
+    if params and constants.IDISK_SPINDLES in params:
+      params[constants.IDISK_SPINDLES] = \
+          int(params[constants.IDISK_SPINDLES])
     if params and constants.IDISK_SIZE in params:
       params[constants.IDISK_SIZE] = \
         utils.ParseUnit(params[constants.IDISK_SIZE])
@@ -1320,6 +1335,14 @@ def SetInstanceParams(opts, args):
   FixHvParams(opts.hvparams)
 
   nics = _ConvertNicDiskModifications(opts.nics)
+  for action, _, __ in nics:
+    if action == constants.DDM_MODIFY and opts.hotplug and not opts.force:
+      usertext = ("You are about to hot-modify a NIC. This will be done"
+                  " by removing the existing NIC and then adding a new one."
+                  " Network connection might be lost. Continue?")
+      if not AskUser(usertext):
+        return 1
+
   disks = _ParseDiskSizes(_ConvertNicDiskModifications(opts.disks))
 
   if (opts.disk_template and
@@ -1339,6 +1362,8 @@ def SetInstanceParams(opts, args):
   op = opcodes.OpInstanceSetParams(instance_name=args[0],
                                    nics=nics,
                                    disks=disks,
+                                   hotplug=opts.hotplug,
+                                   hotplug_if_possible=opts.hotplug_if_possible,
                                    disk_template=opts.disk_template,
                                    remote_node=opts.node,
                                    pnode=opts.new_primary_node,
@@ -1490,7 +1515,7 @@ commands = {
   "move": (
     MoveInstance, ARGS_ONE_INSTANCE,
     [FORCE_OPT] + SUBMIT_OPTS +
-    [SINGLE_NODE_OPT,
+    [SINGLE_NODE_OPT, COMPRESS_OPT,
      SHUTDOWN_TIMEOUT_OPT, DRY_RUN_OPT, PRIORITY_OPT, IGNORE_CONSIST_OPT,
      IGNORE_IPOLICY_OPT],
     "[-f] <instance>", "Move instance to an arbitrary node"
@@ -1546,7 +1571,8 @@ commands = {
     [DISK_TEMPLATE_OPT, SINGLE_NODE_OPT, OS_OPT, FORCE_VARIANT_OPT,
      OSPARAMS_OPT, DRY_RUN_OPT, PRIORITY_OPT, NWSYNC_OPT, OFFLINE_INST_OPT,
      ONLINE_INST_OPT, IGNORE_IPOLICY_OPT, RUNTIME_MEM_OPT,
-     NOCONFLICTSCHECK_OPT, NEW_PRIMARY_OPT],
+     NOCONFLICTSCHECK_OPT, NEW_PRIMARY_OPT, HOTPLUG_OPT,
+     HOTPLUG_IF_POSSIBLE_OPT],
     "<instance>", "Alters the parameters of an instance"),
   "shutdown": (
     GenericManyOps("shutdown", _ShutdownInstance), [ArgInstance()],
