@@ -31,7 +31,7 @@ from ganeti import utils
 from ganeti.cmdlib.common import ExpandInstanceUuidAndName
 
 
-class ResultWithJobs:
+class ResultWithJobs(object):
   """Data container for LU results with jobs.
 
   Instances of this class returned from L{LogicalUnit.Exec} will be recognized
@@ -269,13 +269,10 @@ class LogicalUnit(object):
   def BuildHooksNodes(self):
     """Build list of nodes to run LU's hooks.
 
-    @rtype: tuple; (list, list) or (list, list, list)
+    @rtype: tuple; (list, list)
     @return: Tuple containing a list of node UUIDs on which the hook
       should run before the execution and a list of node UUIDs on which the
-      hook should run after the execution. As it might be possible that the
-      node UUID is not known at the time this method is invoked, an optional
-      third list can be added which contains node names on which the hook
-      should run after the execution (in case of node add, for instance).
+      hook should run after the execution.
       No nodes should be returned as an empty list (and not None).
     @note: If the C{HPATH} attribute of the LU class is C{None}, this function
       will not be called.
@@ -283,6 +280,27 @@ class LogicalUnit(object):
     """
     raise NotImplementedError
 
+  def PreparePostHookNodes(self, post_hook_node_uuids):
+    """Extend list of nodes to run the post LU hook.
+
+    This method allows LUs to change the list of node UUIDs on which the
+    post hook should run after the LU has been executed but before the post
+    hook is run.
+
+    @type post_hook_node_uuids: list
+    @param post_hook_node_uuids: The initial list of node UUIDs to run the
+      post hook on, as returned by L{BuildHooksNodes}.
+    @rtype: list
+    @return: list of node UUIDs on which the post hook should run. The default
+      implementation returns the passed in C{post_hook_node_uuids}, but
+      custom implementations can choose to alter the list.
+
+    """
+    # For consistency with HooksCallBack we ignore the "could be a function"
+    # warning
+    # pylint: disable=R0201
+    return post_hook_node_uuids
+
   def HooksCallBack(self, phase, hook_results, feedback_fn, lu_result):
     """Notify the LU about the results of its hooks.
 
@@ -401,8 +419,14 @@ class NoHooksLU(LogicalUnit): # pylint: disable=W0223
     """
     raise AssertionError("BuildHooksNodes called for NoHooksLU")
 
+  def PreparePostHookNodes(self, post_hook_node_uuids):
+    """Empty PreparePostHookNodes for NoHooksLU.
 
-class Tasklet:
+    """
+    raise AssertionError("PreparePostHookNodes called for NoHooksLU")
+
+
+class Tasklet(object):
   """Tasklet base class.
 
   Tasklets are subcomponents for LUs. LUs can consist entirely of tasklets or
@@ -448,7 +472,7 @@ class Tasklet:
     raise NotImplementedError
 
 
-class QueryBase:
+class QueryBase(object):
   """Base for query utility classes.
 
   """
