@@ -32,7 +32,7 @@ from ganeti import locking
 from ganeti import objects
 from ganeti import opcodes
 from ganeti import pathutils
-from ganeti import rpc
+import ganeti.rpc.node as rpc
 from ganeti import ssconf
 from ganeti import utils
 
@@ -194,7 +194,7 @@ def RedistributeAncillaryFiles(lu):
   """
   # Gather target nodes
   cluster = lu.cfg.GetClusterInfo()
-  master_info = lu.cfg.GetNodeInfo(lu.cfg.GetMasterNode())
+  master_info = lu.cfg.GetMasterNodeInfo()
 
   online_node_uuids = lu.cfg.GetOnlineNodeList()
   online_node_uuid_set = frozenset(online_node_uuids)
@@ -398,7 +398,7 @@ def CheckOSParams(lu, required, node_uuids, osname, osparams):
 def CheckHVParams(lu, node_uuids, hvname, hvparams):
   """Hypervisor parameter validation.
 
-  This function abstract the hypervisor parameter validation to be
+  This function abstracts the hypervisor parameter validation to be
   used in both instance create and instance modify.
 
   @type lu: L{LogicalUnit}
@@ -426,7 +426,7 @@ def CheckHVParams(lu, node_uuids, hvname, hvparams):
                lu.cfg.GetNodeName(node_uuid))
 
 
-def AdjustCandidatePool(lu, exceptions):
+def AdjustCandidatePool(lu, exceptions, feedback_fn):
   """Adjust the candidate pool after node operations.
 
   """
@@ -436,6 +436,9 @@ def AdjustCandidatePool(lu, exceptions):
                utils.CommaJoin(node.name for node in mod_list))
     for node in mod_list:
       lu.context.ReaddNode(node)
+      cluster = lu.cfg.GetClusterInfo()
+      AddNodeCertToCandidateCerts(lu, node.uuid, cluster)
+      lu.cfg.Update(cluster, feedback_fn)
   mc_now, mc_max, _ = lu.cfg.GetMasterCandidateStats(exceptions)
   if mc_now > mc_max:
     lu.LogInfo("Note: more nodes are candidates (%d) than desired (%d)" %
@@ -717,11 +720,10 @@ def AnnotateDiskParams(instance, devs, cfg):
   @param devs: The root devices (not any of its children!)
   @param cfg: The config object
   @returns The annotated disk copies
-  @see L{rpc.AnnotateDiskParams}
+  @see L{rpc.node.AnnotateDiskParams}
 
   """
-  return rpc.AnnotateDiskParams(instance.disk_template, devs,
-                                cfg.GetInstanceDiskParams(instance))
+  return rpc.AnnotateDiskParams(devs, cfg.GetInstanceDiskParams(instance))
 
 
 def SupportsOob(cfg, node):
@@ -1037,9 +1039,6 @@ def CheckIAllocatorOrNode(lu, iallocator_slot, node_slot):
 def FindFaultyInstanceDisks(cfg, rpc_runner, instance, node_uuid, prereq):
   faulty = []
 
-  for dev in instance.disks:
-    cfg.SetDiskID(dev, node_uuid)
-
   result = rpc_runner.call_blockdev_getmirrorstatus(
              node_uuid, (instance.disks, instance))
   result.Raise("Failed to get disk status from node %s" %
@@ -1108,7 +1107,7 @@ def CheckStorageTypeEnabled(cluster, storage_type):
     CheckStorageTypeEnabled(cluster, constants.ST_LVM_VG)
   else:
     possible_disk_templates = \
-        utils.storage.GetDiskTemplatesOfStorageType(storage_type)
+        utils.storage.GetDiskTemplatesOfStorageTypes(storage_type)
     for disk_template in possible_disk_templates:
       if disk_template in cluster.enabled_disk_templates:
         return
@@ -1137,3 +1136,208 @@ def CheckIpolicyVsDiskTemplates(ipolicy, enabled_disk_templates):
     raise errors.OpPrereqError("The following disk template are allowed"
                                " by the ipolicy, but not enabled on the"
                                " cluster: %s" % utils.CommaJoin(not_enabled))
+
+
+def CheckDiskAccessModeValidity(parameters):
+  """Checks if the access parameter is legal.
+
+  @see: L{CheckDiskAccessModeConsistency} for cluster consistency checks.
+  @raise errors.OpPrereqError: if the check fails.
+
+  """
+  for disk_template in parameters:
+    access = parameters[disk_template].get(constants.LDP_ACCESS,
+                                           constants.DISK_KERNELSPACE)
+    if access not in constants.DISK_VALID_ACCESS_MODES:
+      valid_vals_str = utils.CommaJoin(constants.DISK_VALID_ACCESS_MODES)
+      raise errors.OpPrereqError("Invalid value of '{d}:{a}': '{v}' (expected"
+                                 " one of {o})".format(d=disk_template,
+                                                       a=constants.LDP_ACCESS,
+                                                       v=access,
+                                                       o=valid_vals_str))
+
+
+def CheckDiskAccessModeConsistency(parameters, cfg, group=None):
+  """Checks if the access param is consistent with the cluster configuration.
+
+  @note: requires a configuration lock to run.
+  @param parameters: the parameters to validate
+  @param cfg: the cfg object of the cluster
+  @param group: if set, only check for consistency within this group.
+  @raise errors.OpPrereqError: if the LU attempts to change the access parameter
+                               to an invalid value, such as "pink bunny".
+  @raise errors.OpPrereqError: if the LU attempts to change the access parameter
+                               to an inconsistent value, such as asking for RBD
+                               userspace access to the chroot hypervisor.
+
+  """
+  CheckDiskAccessModeValidity(parameters)
+
+  for disk_template in parameters:
+    access = parameters[disk_template].get(constants.LDP_ACCESS,
+                                           constants.DISK_KERNELSPACE)
+
+    if disk_template not in constants.DTS_HAVE_ACCESS:
+      continue
+
+    #Check the combination of instance hypervisor, disk template and access
+    #protocol is sane.
+    inst_uuids = cfg.GetNodeGroupInstances(group) if group else \
+                 cfg.GetInstanceList()
+
+    for entry in inst_uuids:
+      inst = cfg.GetInstanceInfo(entry)
+      hv = inst.hypervisor
+      dt = inst.disk_template
+
+      if not IsValidDiskAccessModeCombination(hv, dt, access):
+        raise errors.OpPrereqError("Instance {i}: cannot use '{a}' access"
+                                   " setting with {h} hypervisor and {d} disk"
+                                   " type.".format(i=inst.name,
+                                                   a=access,
+                                                   h=hv,
+                                                   d=dt))
+
+
+def IsValidDiskAccessModeCombination(hv, disk_template, mode):
+  """Checks if an hypervisor can read a disk template with given mode.
+
+  @param hv: the hypervisor that will access the data
+  @param disk_template: the disk template the data is stored as
+  @param mode: how the hypervisor should access the data
+  @return: True if the hypervisor can read a given read disk_template
+           in the specified mode.
+
+  """
+  if mode == constants.DISK_KERNELSPACE:
+    return True
+
+  if (hv == constants.HT_KVM and
+      disk_template in (constants.DT_RBD, constants.DT_GLUSTER) and
+      mode == constants.DISK_USERSPACE):
+    return True
+
+  # Everything else:
+  return False
+
+
+def AddNodeCertToCandidateCerts(lu, node_uuid, cluster):
+  """Add the node's client SSL certificate digest to the candidate certs.
+
+  @type node_uuid: string
+  @param node_uuid: the node's UUID
+  @type cluster: C{object.Cluster}
+  @param cluster: the cluster's configuration
+
+  """
+  result = lu.rpc.call_node_crypto_tokens(
+             node_uuid,
+             [(constants.CRYPTO_TYPE_SSL_DIGEST, constants.CRYPTO_ACTION_GET,
+               None)])
+  result.Raise("Could not retrieve the node's (uuid %s) SSL digest."
+               % node_uuid)
+  ((crypto_type, digest), ) = result.payload
+  assert crypto_type == constants.CRYPTO_TYPE_SSL_DIGEST
+
+  utils.AddNodeToCandidateCerts(node_uuid, digest, cluster.candidate_certs)
+
+
+def RemoveNodeCertFromCandidateCerts(node_uuid, cluster):
+  """Removes the node's certificate from the candidate certificates list.
+
+  @type node_uuid: string
+  @param node_uuid: the node's UUID
+  @type cluster: C{objects.Cluster}
+  @param cluster: the cluster's configuration
+
+  """
+  utils.RemoveNodeFromCandidateCerts(node_uuid, cluster.candidate_certs)
+
+
+def CreateNewClientCert(lu, node_uuid, filename=None):
+  """Creates a new client SSL certificate for the node.
+
+  @type node_uuid: string
+  @param node_uuid: the node's UUID
+  @type filename: string
+  @param filename: the certificate's filename
+  @rtype: string
+  @return: the digest of the newly created certificate
+
+  """
+  options = {}
+  if filename:
+    options[constants.CRYPTO_OPTION_CERT_FILE] = filename
+  options[constants.CRYPTO_OPTION_SERIAL_NO] = utils.UuidToInt(node_uuid)
+  result = lu.rpc.call_node_crypto_tokens(
+             node_uuid,
+             [(constants.CRYPTO_TYPE_SSL_DIGEST,
+               constants.CRYPTO_ACTION_CREATE,
+               options)])
+  result.Raise("Could not create the node's (uuid %s) SSL client"
+               " certificate." % node_uuid)
+  ((crypto_type, new_digest), ) = result.payload
+  assert crypto_type == constants.CRYPTO_TYPE_SSL_DIGEST
+  return new_digest
+
+
+def EnsureKvmdOnNodes(lu, feedback_fn, nodes=None):
+  """Ensure KVM daemon is running on nodes with KVM instances.
+
+  If user shutdown is enabled in the cluster:
+    - The KVM daemon will be started on VM capable nodes containing
+      KVM instances.
+    - The KVM daemon will be stopped on non VM capable nodes.
+
+  If user shutdown is disabled in the cluster:
+    - The KVM daemon will be stopped on all nodes
+
+  Issues a warning for each failed RPC call.
+
+  @type lu: L{LogicalUnit}
+  @param lu: logical unit on whose behalf we execute
+
+  @type feedback_fn: callable
+  @param feedback_fn: feedback function
+
+  @type nodes: list of string
+  @param nodes: if supplied, it overrides the node uuids to start/stop;
+                this is used mainly for optimization
+
+  """
+  cluster = lu.cfg.GetClusterInfo()
+
+  # Either use the passed nodes or consider all cluster nodes
+  if nodes is not None:
+    node_uuids = set(nodes)
+  else:
+    node_uuids = lu.cfg.GetNodeList()
+
+  # Determine in which nodes should the KVM daemon be started/stopped
+  if constants.HT_KVM in cluster.enabled_hypervisors and \
+        cluster.enabled_user_shutdown:
+    start_nodes = []
+    stop_nodes = []
+
+    for node_uuid in node_uuids:
+      if lu.cfg.GetNodeInfo(node_uuid).vm_capable:
+        start_nodes.append(node_uuid)
+      else:
+        stop_nodes.append(node_uuid)
+  else:
+    start_nodes = []
+    stop_nodes = node_uuids
+
+  # Start KVM where necessary
+  if start_nodes:
+    results = lu.rpc.call_node_ensure_daemon(start_nodes, constants.KVMD, True)
+    for node_uuid in start_nodes:
+      results[node_uuid].Warn("Failed to start KVM daemon in node '%s'" %
+                              node_uuid, feedback_fn)
+
+  # Stop KVM where necessary
+  if stop_nodes:
+    results = lu.rpc.call_node_ensure_daemon(stop_nodes, constants.KVMD, False)
+    for node_uuid in stop_nodes:
+      results[node_uuid].Warn("Failed to stop KVM daemon in node '%s'" %
+                              node_uuid, feedback_fn)
