@@ -28,18 +28,17 @@ from ganeti import constants
 from ganeti import errors
 from ganeti import locking
 from ganeti import objects
-from ganeti import qlang
-from ganeti import query
+from ganeti import opcodes
 from ganeti import utils
 from ganeti.masterd import iallocator
-from ganeti.cmdlib.base import LogicalUnit, NoHooksLU, QueryBase, \
-  ResultWithJobs
+from ganeti.cmdlib.base import LogicalUnit, NoHooksLU, ResultWithJobs
 from ganeti.cmdlib.common import MergeAndVerifyHvState, \
   MergeAndVerifyDiskState, GetWantedNodes, GetUpdatedParams, \
   CheckNodeGroupInstances, GetUpdatedIPolicy, \
   ComputeNewInstanceViolations, GetDefaultIAllocator, ShareAll, \
   CheckInstancesNodeGroups, LoadNodeEvacResult, MapInstanceLvsToNodes, \
-  CheckIpolicyVsDiskTemplates
+  CheckIpolicyVsDiskTemplates, CheckDiskAccessModeValidity, \
+  CheckDiskAccessModeConsistency
 
 import ganeti.masterd.instance
 
@@ -284,106 +283,6 @@ class LUGroupAssignNodes(NoHooksLU):
             list(previously_split_instances & all_split_instances))
 
 
-class GroupQuery(QueryBase):
-  FIELDS = query.GROUP_FIELDS
-
-  def ExpandNames(self, lu):
-    lu.needed_locks = {}
-
-    self._all_groups = lu.cfg.GetAllNodeGroupsInfo()
-    self._cluster = lu.cfg.GetClusterInfo()
-    name_to_uuid = dict((g.name, g.uuid) for g in self._all_groups.values())
-
-    if not self.names:
-      self.wanted = [name_to_uuid[name]
-                     for name in utils.NiceSort(name_to_uuid.keys())]
-    else:
-      # Accept names to be either names or UUIDs.
-      missing = []
-      self.wanted = []
-      all_uuid = frozenset(self._all_groups.keys())
-
-      for name in self.names:
-        if name in all_uuid:
-          self.wanted.append(name)
-        elif name in name_to_uuid:
-          self.wanted.append(name_to_uuid[name])
-        else:
-          missing.append(name)
-
-      if missing:
-        raise errors.OpPrereqError("Some groups do not exist: %s" %
-                                   utils.CommaJoin(missing),
-                                   errors.ECODE_NOENT)
-
-  def DeclareLocks(self, lu, level):
-    pass
-
-  def _GetQueryData(self, lu):
-    """Computes the list of node groups and their attributes.
-
-    """
-    do_nodes = query.GQ_NODE in self.requested_data
-    do_instances = query.GQ_INST in self.requested_data
-
-    group_to_nodes = None
-    group_to_instances = None
-
-    # For GQ_NODE, we need to map group->[nodes], and group->[instances] for
-    # GQ_INST. The former is attainable with just GetAllNodesInfo(), but for the
-    # latter GetAllInstancesInfo() is not enough, for we have to go through
-    # instance->node. Hence, we will need to process nodes even if we only need
-    # instance information.
-    if do_nodes or do_instances:
-      all_nodes = lu.cfg.GetAllNodesInfo()
-      group_to_nodes = dict((uuid, []) for uuid in self.wanted)
-      node_to_group = {}
-
-      for node in all_nodes.values():
-        if node.group in group_to_nodes:
-          group_to_nodes[node.group].append(node.uuid)
-          node_to_group[node.uuid] = node.group
-
-      if do_instances:
-        all_instances = lu.cfg.GetAllInstancesInfo()
-        group_to_instances = dict((uuid, []) for uuid in self.wanted)
-
-        for instance in all_instances.values():
-          node = instance.primary_node
-          if node in node_to_group:
-            group_to_instances[node_to_group[node]].append(instance.uuid)
-
-        if not do_nodes:
-          # Do not pass on node information if it was not requested.
-          group_to_nodes = None
-
-    return query.GroupQueryData(self._cluster,
-                                [self._all_groups[uuid]
-                                 for uuid in self.wanted],
-                                group_to_nodes, group_to_instances,
-                                query.GQ_DISKPARAMS in self.requested_data)
-
-
-class LUGroupQuery(NoHooksLU):
-  """Logical unit for querying node groups.
-
-  """
-  REQ_BGL = False
-
-  def CheckArguments(self):
-    self.gq = GroupQuery(qlang.MakeSimpleFilter("name", self.op.names),
-                          self.op.output_fields, False)
-
-  def ExpandNames(self):
-    self.gq.ExpandNames(self)
-
-  def DeclareLocks(self, level):
-    self.gq.DeclareLocks(self, level)
-
-  def Exec(self, feedback_fn):
-    return self.gq.OldStyleQuery(self)
-
-
 class LUGroupSetParams(LogicalUnit):
   """Modifies the parameters of a node group.
 
@@ -406,6 +305,9 @@ class LUGroupSetParams(LogicalUnit):
       raise errors.OpPrereqError("Please pass at least one modification",
                                  errors.ECODE_INVAL)
 
+    if self.op.diskparams:
+      CheckDiskAccessModeValidity(self.op.diskparams)
+
   def ExpandNames(self):
     # This raises errors.OpPrereqError on its own:
     self.group_uuid = self.cfg.LookupNodeGroup(self.op.group_name)
@@ -500,8 +402,11 @@ class LUGroupSetParams(LogicalUnit):
       # As we've all subdicts of diskparams ready, lets merge the actual
       # dict with all updated subdicts
       self.new_diskparams = objects.FillDict(diskparams, new_diskparams)
+
       try:
         utils.VerifyDictOptions(self.new_diskparams, constants.DISK_DT_DEFAULTS)
+        CheckDiskAccessModeConsistency(self.new_diskparams, self.cfg,
+                                       group=self.group)
       except errors.OpPrereqError, err:
         raise errors.OpPrereqError("While verify diskparams options: %s" % err,
                                    errors.ECODE_INVAL)
@@ -827,6 +732,24 @@ class LUGroupEvacuate(LogicalUnit):
 
     return (run_nodes, run_nodes)
 
+  @staticmethod
+  def _MigrateToFailover(op):
+    """Return an equivalent failover opcode for a migrate one.
+
+    If the argument is not a failover opcode, return it unchanged.
+
+    """
+    if not isinstance(op, opcodes.OpInstanceMigrate):
+      return op
+    else:
+      return opcodes.OpInstanceFailover(
+        instance_name=op.instance_name,
+        instance_uuid=getattr(op, "instance_uuid", None),
+        target_node=getattr(op, "target_node", None),
+        target_node_uuid=getattr(op, "target_node_uuid", None),
+        ignore_ipolicy=op.ignore_ipolicy,
+        cleanup=op.cleanup)
+
   def Exec(self, feedback_fn):
     inst_names = list(self.owned_locks(locking.LEVEL_INSTANCE))
 
@@ -849,6 +772,16 @@ class LUGroupEvacuate(LogicalUnit):
     self.LogInfo("Iallocator returned %s job(s) for evacuating node group %s",
                  len(jobs), self.op.group_name)
 
+    if self.op.force_failover:
+      self.LogInfo("Will insist on failovers")
+      jobs = [[self._MigrateToFailover(op) for op in job] for job in jobs]
+
+    if self.op.sequential:
+      self.LogInfo("Jobs will be submitted to run sequentially")
+      for job in jobs[1:]:
+        for op in job:
+          op.depends = [(-1, ["error", "success"])]
+
     return ResultWithJobs(jobs)
 
 
@@ -968,12 +901,9 @@ class LUGroupVerifyDisks(NoHooksLU):
                                        inst.secondary_nodes):
         node_to_inst.setdefault(node_uuid, []).append(inst)
 
-    nodes_ip = dict((uuid, node.secondary_ip) for (uuid, node)
-                    in self.cfg.GetMultiNodeInfo(node_to_inst.keys()))
     for (node_uuid, insts) in node_to_inst.items():
       node_disks = [(inst.disks, inst) for inst in insts]
-      node_res = self.rpc.call_drbd_needs_activation(node_uuid, nodes_ip,
-                                                     node_disks)
+      node_res = self.rpc.call_drbd_needs_activation(node_uuid, node_disks)
       msg = node_res.fail_msg
       if msg:
         logging.warning("Error getting DRBD status on node %s: %s",
