@@ -42,6 +42,16 @@ from ganeti.cmdlib.instance_storage import StartInstanceDisks, \
   ShutdownInstanceDisks
 from ganeti.cmdlib.instance_utils import BuildInstanceHookEnvByObject, \
   CheckInstanceBridgesExist, CheckNodeFreeMemory, CheckNodeHasOS
+from ganeti.hypervisor import hv_base
+
+
+def _IsInstanceUserDown(cluster, instance, instance_info):
+  hvparams = cluster.FillHV(instance, skip_globals=True)
+  return instance_info and \
+      "state" in instance_info and \
+      hv_base.HvInstanceState.IsShutdown(instance_info["state"]) and \
+      (instance.hypervisor != constants.HT_KVM or
+       hvparams[constants.HV_KVM_USER_SHUTDOWN])
 
 
 class LUInstanceStartup(LogicalUnit):
@@ -135,7 +145,15 @@ class LUInstanceStartup(LogicalUnit):
       remote_info.Raise("Error checking node %s" %
                         self.cfg.GetNodeName(self.instance.primary_node),
                         prereq=True, ecode=errors.ECODE_ENVIRON)
-      if not remote_info.payload: # not running already
+
+      self.requires_cleanup = False
+
+      if remote_info.payload:
+        if _IsInstanceUserDown(self.cfg.GetClusterInfo(),
+                               self.instance,
+                               remote_info.payload):
+          self.requires_cleanup = True
+      else: # not running already
         CheckNodeFreeMemory(
             self, self.instance.primary_node,
             "starting instance %s" % self.instance.name,
@@ -153,6 +171,15 @@ class LUInstanceStartup(LogicalUnit):
       assert self.op.ignore_offline_nodes
       self.LogInfo("Primary node offline, marked instance as started")
     else:
+      if self.requires_cleanup:
+        result = self.rpc.call_instance_shutdown(
+          self.instance.primary_node,
+          self.instance,
+          self.op.shutdown_timeout, self.op.reason)
+        result.Raise("Could not shutdown instance '%s'", self.instance.name)
+
+        ShutdownInstanceDisks(self, self.instance)
+
       StartInstanceDisks(self, self.instance, self.op.force)
 
       result = \
@@ -160,10 +187,9 @@ class LUInstanceStartup(LogicalUnit):
                                      (self.instance, self.op.hvparams,
                                       self.op.beparams),
                                      self.op.startup_paused, self.op.reason)
-      msg = result.fail_msg
-      if msg:
+      if result.fail_msg:
         ShutdownInstanceDisks(self, self.instance)
-        raise errors.OpExecError("Could not start instance: %s" % msg)
+        result.Raise("Could not start instance '%s'", self.instance.name)
 
 
 class LUInstanceShutdown(LogicalUnit):
@@ -177,6 +203,17 @@ class LUInstanceShutdown(LogicalUnit):
   def ExpandNames(self):
     self._ExpandAndLockInstance()
 
+  def CheckArguments(self):
+    """Check arguments.
+
+    """
+    if self.op.no_remember and self.op.admin_state_source is not None:
+      self.LogWarning("Parameter 'admin_state_source' has no effect if used"
+                      " with parameter 'no_remember'")
+
+    if self.op.admin_state_source is None:
+      self.op.admin_state_source = constants.ADMIN_SOURCE
+
   def BuildHooksEnv(self):
     """Build hooks env.
 
@@ -204,10 +241,10 @@ class LUInstanceShutdown(LogicalUnit):
     assert self.instance is not None, \
       "Cannot retrieve locked instance %s" % self.op.instance_name
 
-    if not self.op.force:
-      CheckInstanceState(self, self.instance, INSTANCE_ONLINE)
-    else:
+    if self.op.force:
       self.LogWarning("Ignoring offline instance check")
+    else:
+      CheckInstanceState(self, self.instance, INSTANCE_ONLINE)
 
     self.primary_offline = \
       self.cfg.GetNodeInfo(self.instance.primary_node).offline
@@ -217,6 +254,23 @@ class LUInstanceShutdown(LogicalUnit):
     else:
       CheckNodeOnline(self, self.instance.primary_node)
 
+    if self.op.admin_state_source == constants.USER_SOURCE:
+      cluster = self.cfg.GetClusterInfo()
+
+      result = self.rpc.call_instance_info(
+        self.instance.primary_node,
+        self.instance.name,
+        self.instance.hypervisor,
+        cluster.hvparams[self.instance.hypervisor])
+      result.Raise("Error checking instance '%s'" % self.instance.name,
+                   prereq=True)
+
+      if not _IsInstanceUserDown(cluster,
+                                 self.instance,
+                                 result.payload):
+        raise errors.OpPrereqError("Instance '%s' was not shutdown by the user"
+                                   % self.instance.name)
+
   def Exec(self, feedback_fn):
     """Shutdown the instance.
 
@@ -224,18 +278,20 @@ class LUInstanceShutdown(LogicalUnit):
     # If the instance is offline we shouldn't mark it as down, as that
     # resets the offline flag.
     if not self.op.no_remember and self.instance.admin_state in INSTANCE_ONLINE:
-      self.cfg.MarkInstanceDown(self.instance.uuid)
+      if self.op.admin_state_source == constants.ADMIN_SOURCE:
+        self.cfg.MarkInstanceDown(self.instance.uuid)
+      elif self.op.admin_state_source == constants.USER_SOURCE:
+        self.cfg.MarkInstanceUserDown(self.instance.uuid)
 
     if self.primary_offline:
       assert self.op.ignore_offline_nodes
       self.LogInfo("Primary node offline, marked instance as stopped")
     else:
-      result = self.rpc.call_instance_shutdown(self.instance.primary_node,
-                                               self.instance,
-                                               self.op.timeout, self.op.reason)
-      msg = result.fail_msg
-      if msg:
-        self.LogWarning("Could not shutdown instance: %s", msg)
+      result = self.rpc.call_instance_shutdown(
+        self.instance.primary_node,
+        self.instance,
+        self.op.timeout, self.op.reason)
+      result.Raise("Could not shutdown instance '%s'", self.instance.name)
 
       ShutdownInstanceDisks(self, self.instance)
 
@@ -393,8 +449,6 @@ class LUInstanceReboot(LogicalUnit):
     if instance_running and \
         self.op.reboot_type in [constants.INSTANCE_REBOOT_SOFT,
                                 constants.INSTANCE_REBOOT_HARD]:
-      for disk in self.instance.disks:
-        self.cfg.SetDiskID(disk, current_node_uuid)
       result = self.rpc.call_instance_reboot(current_node_uuid, self.instance,
                                              self.op.reboot_type,
                                              self.op.shutdown_timeout,
@@ -424,12 +478,13 @@ class LUInstanceReboot(LogicalUnit):
     self.cfg.MarkInstanceUp(self.instance.uuid)
 
 
-def GetInstanceConsole(cluster, instance, primary_node):
+def GetInstanceConsole(cluster, instance, primary_node, node_group):
   """Returns console information for an instance.
 
   @type cluster: L{objects.Cluster}
   @type instance: L{objects.Instance}
   @type primary_node: L{objects.Node}
+  @type node_group: L{objects.NodeGroup}
   @rtype: dict
 
   """
@@ -438,10 +493,11 @@ def GetInstanceConsole(cluster, instance, primary_node):
   # instance and then saving the defaults in the instance itself.
   hvparams = cluster.FillHV(instance)
   beparams = cluster.FillBE(instance)
-  console = hyper.GetInstanceConsole(instance, primary_node, hvparams, beparams)
+  console = hyper.GetInstanceConsole(instance, primary_node, node_group,
+                                     hvparams, beparams)
 
   assert console.instance == instance.name
-  assert console.Validate()
+  console.Validate()
 
   return console.ToDict()
 
@@ -497,5 +553,7 @@ class LUInstanceConsole(NoHooksLU):
     logging.debug("Connecting to console of %s on %s", self.instance.name,
                   self.cfg.GetNodeName(node_uuid))
 
-    return GetInstanceConsole(self.cfg.GetClusterInfo(), self.instance,
-                              self.cfg.GetNodeInfo(self.instance.primary_node))
+    node = self.cfg.GetNodeInfo(self.instance.primary_node)
+    group = self.cfg.GetNodeGroup(node.group)
+    return GetInstanceConsole(self.cfg.GetClusterInfo(),
+                              self.instance, node, group)
