@@ -30,14 +30,11 @@ from ganeti import locking
 from ganeti import netutils
 from ganeti import objects
 from ganeti import opcodes
-from ganeti import qlang
-from ganeti import query
-from ganeti import rpc
+import ganeti.rpc.node as rpc
 from ganeti import utils
 from ganeti.masterd import iallocator
 
-from ganeti.cmdlib.base import LogicalUnit, NoHooksLU, QueryBase, \
-  ResultWithJobs
+from ganeti.cmdlib.base import LogicalUnit, NoHooksLU, ResultWithJobs
 from ganeti.cmdlib.common import CheckParamsNotGlobal, \
   MergeAndVerifyHvState, MergeAndVerifyDiskState, \
   IsExclusiveStorageEnabledNode, CheckNodePVs, \
@@ -45,7 +42,9 @@ from ganeti.cmdlib.common import CheckParamsNotGlobal, \
   CheckInstanceState, INSTANCE_DOWN, GetUpdatedParams, \
   AdjustCandidatePool, CheckIAllocatorOrNode, LoadNodeEvacResult, \
   GetWantedNodes, MapInstanceLvsToNodes, RunPostHook, \
-  FindFaultyInstanceDisks, CheckStorageTypeEnabled
+  FindFaultyInstanceDisks, CheckStorageTypeEnabled, CreateNewClientCert, \
+  AddNodeCertToCandidateCerts, RemoveNodeCertFromCandidateCerts, \
+  EnsureKvmdOnNodes
 
 
 def _DecideSelfPromotion(lu, exceptions=None):
@@ -138,7 +137,10 @@ class LUNodeAdd(LogicalUnit):
       hook_nodes = list(set(hook_nodes) - set([new_node_info.uuid]))
 
     # add the new node as post hook node by name; it does not have an UUID yet
-    return (hook_nodes, hook_nodes, [self.op.node_name, ])
+    return (hook_nodes, hook_nodes)
+
+  def PreparePostHookNodes(self, post_hook_node_uuids):
+    return post_hook_node_uuids + [self.new_node.uuid]
 
   def CheckPrereq(self):
     """Check prerequisites.
@@ -217,7 +219,7 @@ class LUNodeAdd(LogicalUnit):
 
     # check that the type of the node (single versus dual homed) is the
     # same as for the master
-    myself = self.cfg.GetNodeInfo(self.cfg.GetMasterNode())
+    myself = self.cfg.GetMasterNodeInfo()
     master_singlehomed = myself.secondary_ip == myself.primary_ip
     newbie_singlehomed = secondary_ip == self.op.primary_ip
     if master_singlehomed != newbie_singlehomed:
@@ -253,16 +255,18 @@ class LUNodeAdd(LogicalUnit):
     else:
       self.master_candidate = False
 
+    self.node_group = None
     if self.op.readd:
       self.new_node = existing_node_info
+      self.node_group = existing_node_info.group
     else:
-      node_group = self.cfg.LookupNodeGroup(self.op.group)
+      self.node_group = self.cfg.LookupNodeGroup(self.op.group)
       self.new_node = objects.Node(name=node_name,
                                    primary_ip=self.op.primary_ip,
                                    secondary_ip=secondary_ip,
                                    master_candidate=self.master_candidate,
                                    offline=False, drained=False,
-                                   group=node_group, ndparams={})
+                                   group=self.node_group, ndparams={})
 
     if self.op.ndparams:
       utils.ForceDictType(self.op.ndparams, constants.NDS_PARAMETER_TYPES)
@@ -279,7 +283,8 @@ class LUNodeAdd(LogicalUnit):
     #       it a property on the base class.
     rpcrunner = rpc.DnsOnlyRunner()
     result = rpcrunner.call_version([node_name])[node_name]
-    result.Raise("Can't get version information from node %s" % node_name)
+    result.Raise("Can't get version information from node %s" % node_name,
+                 prereq=True)
     if constants.PROTOCOL_VERSION == result.payload:
       logging.info("Communication to node %s fine, sw version %s match",
                    node_name, result.payload)
@@ -296,12 +301,33 @@ class LUNodeAdd(LogicalUnit):
       cname = self.cfg.GetClusterName()
       result = rpcrunner.call_node_verify_light(
           [node_name], vparams, cname,
-          self.cfg.GetClusterInfo().hvparams)[node_name]
+          self.cfg.GetClusterInfo().hvparams,
+          {node_name: self.node_group},
+          self.cfg.GetAllNodeGroupsInfoDict()
+        )[node_name]
       (errmsgs, _) = CheckNodePVs(result.payload, excl_stor)
       if errmsgs:
         raise errors.OpPrereqError("Checks on node PVs failed: %s" %
                                    "; ".join(errmsgs), errors.ECODE_ENVIRON)
 
+  def _InitOpenVSwitch(self):
+    filled_ndparams = self.cfg.GetClusterInfo().FillND(
+      self.new_node, self.cfg.GetNodeGroup(self.new_node.group))
+
+    ovs = filled_ndparams.get(constants.ND_OVS, None)
+    ovs_name = filled_ndparams.get(constants.ND_OVS_NAME, None)
+    ovs_link = filled_ndparams.get(constants.ND_OVS_LINK, None)
+
+    if ovs:
+      if not ovs_link:
+        self.LogInfo("No physical interface for OpenvSwitch was given."
+                     " OpenvSwitch will not have an outside connection. This"
+                     " might not be what you want.")
+
+      result = self.rpc.call_node_configure_ovs(
+                 self.new_node.name, ovs_name, ovs_link)
+      result.Raise("Failed to initialize OpenVSwitch on new node")
+
   def Exec(self, feedback_fn):
     """Adds the new node to the cluster.
 
@@ -365,7 +391,10 @@ class LUNodeAdd(LogicalUnit):
     result = self.rpc.call_node_verify(
                node_verifier_uuids, node_verify_param,
                self.cfg.GetClusterName(),
-               self.cfg.GetClusterInfo().hvparams)
+               self.cfg.GetClusterInfo().hvparams,
+               {self.new_node.name: self.cfg.LookupNodeGroup(self.node_group)},
+               self.cfg.GetAllNodeGroupsInfoDict()
+               )
     for verifier in node_verifier_uuids:
       result[verifier].Raise("Cannot communicate with node %s" % verifier)
       nl_payload = result[verifier].payload[constants.NV_NODELIST]
@@ -376,6 +405,8 @@ class LUNodeAdd(LogicalUnit):
                       (verifier, nl_payload[failed]))
         raise errors.OpExecError("ssh/hostname verification failed")
 
+    self._InitOpenVSwitch()
+
     if self.op.readd:
       self.context.ReaddNode(self.new_node)
       RedistributeAncillaryFiles(self)
@@ -390,6 +421,21 @@ class LUNodeAdd(LogicalUnit):
       self.context.AddNode(self.new_node, self.proc.GetECId())
       RedistributeAncillaryFiles(self)
 
+    cluster = self.cfg.GetClusterInfo()
+    # We create a new certificate even if the node is readded
+    digest = CreateNewClientCert(self, self.new_node.uuid)
+    if self.new_node.master_candidate:
+      utils.AddNodeToCandidateCerts(self.new_node.uuid, digest,
+                                    cluster.candidate_certs)
+      self.cfg.Update(cluster, feedback_fn)
+    else:
+      if self.new_node.uuid in cluster.candidate_certs:
+        utils.RemoveNodeFromCandidateCerts(self.new_node.uuid,
+                                           cluster.candidate_certs)
+        self.cfg.Update(cluster, feedback_fn)
+
+    EnsureKvmdOnNodes(self, feedback_fn, nodes=[self.new_node.uuid])
+
 
 class LUNodeSetParams(LogicalUnit):
   """Modifies the parameters of a node.
@@ -636,7 +682,7 @@ class LUNodeSetParams(LogicalUnit):
     # restrictions.
     if self.op.secondary_ip:
       # Ok even without locking, because this can't be changed by any LU
-      master = self.cfg.GetNodeInfo(self.cfg.GetMasterNode())
+      master = self.cfg.GetMasterNodeInfo()
       master_singlehomed = master.secondary_ip == master.primary_ip
       if master_singlehomed and self.op.secondary_ip != node.primary_ip:
         if self.op.force and node.uuid == master.uuid:
@@ -744,7 +790,15 @@ class LUNodeSetParams(LogicalUnit):
 
       # we locked all nodes, we adjust the CP before updating this node
       if self.lock_all:
-        AdjustCandidatePool(self, [node.uuid])
+        AdjustCandidatePool(self, [node.uuid], feedback_fn)
+
+      cluster = self.cfg.GetClusterInfo()
+      # if node gets promoted, grant RPC priviledges
+      if self.new_role == self._ROLE_CANDIDATE:
+        AddNodeCertToCandidateCerts(self, node.uuid, cluster)
+      # if node is demoted, revoke RPC priviledges
+      if self.old_role == self._ROLE_CANDIDATE:
+        RemoveNodeCertFromCandidateCerts(node.uuid, cluster)
 
     if self.op.secondary_ip:
       node.secondary_ip = self.op.secondary_ip
@@ -758,6 +812,8 @@ class LUNodeSetParams(LogicalUnit):
     if [self.old_role, self.new_role].count(self._ROLE_CANDIDATE) == 1:
       self.context.ReaddNode(node)
 
+    EnsureKvmdOnNodes(self, feedback_fn, nodes=[node.uuid])
+
     return result
 
 
@@ -832,15 +888,6 @@ class LUNodeEvacuate(NoHooksLU):
   """
   REQ_BGL = False
 
-  _MODE2IALLOCATOR = {
-    constants.NODE_EVAC_PRI: constants.IALLOCATOR_NEVAC_PRI,
-    constants.NODE_EVAC_SEC: constants.IALLOCATOR_NEVAC_SEC,
-    constants.NODE_EVAC_ALL: constants.IALLOCATOR_NEVAC_ALL,
-    }
-  assert frozenset(_MODE2IALLOCATOR.keys()) == constants.NODE_EVAC_MODES
-  assert (frozenset(_MODE2IALLOCATOR.values()) ==
-          constants.IALLOCATOR_NEVAC_MODES)
-
   def CheckArguments(self):
     CheckIAllocatorOrNode(self, "iallocator", "remote_node")
 
@@ -997,8 +1044,7 @@ class LUNodeEvacuate(NoHooksLU):
 
     elif self.op.iallocator is not None:
       # TODO: Implement relocation to other group
-      evac_mode = self._MODE2IALLOCATOR[self.op.mode]
-      req = iallocator.IAReqNodeEvac(evac_mode=evac_mode,
+      req = iallocator.IAReqNodeEvac(evac_mode=self.op.mode,
                                      instances=list(self.instance_names))
       ial = iallocator.IAllocator(self.cfg, self.rpc, req)
 
@@ -1098,11 +1144,19 @@ def _GetStorageTypeArgs(cfg, storage_type):
 
   """
   # Special case for file storage
-  if storage_type == constants.ST_FILE:
-    # storage.FileStorage wants a list of storage directories
-    return [[cfg.GetFileStorageDir(), cfg.GetSharedFileStorageDir()]]
 
-  return []
+  if storage_type == constants.ST_FILE:
+    return [[cfg.GetFileStorageDir()]]
+  elif storage_type == constants.ST_SHARED_FILE:
+    dts = cfg.GetClusterInfo().enabled_disk_templates
+    paths = []
+    if constants.DT_SHARED_FILE in dts:
+      paths.append(cfg.GetSharedFileStorageDir())
+    if constants.DT_GLUSTER in dts:
+      paths.append(cfg.GetGlusterStorageDir())
+    return [paths]
+  else:
+    return []
 
 
 class LUNodeModifyStorage(NoHooksLU):
@@ -1154,136 +1208,16 @@ class LUNodeModifyStorage(NoHooksLU):
                  (self.op.name, self.op.node_name))
 
 
-class NodeQuery(QueryBase):
-  FIELDS = query.NODE_FIELDS
+def _CheckOutputFields(fields, selected):
+  """Checks whether all selected fields are valid according to fields.
 
-  def ExpandNames(self, lu):
-    lu.needed_locks = {}
-    lu.share_locks = ShareAll()
-
-    if self.names:
-      (self.wanted, _) = GetWantedNodes(lu, self.names)
-    else:
-      self.wanted = locking.ALL_SET
-
-    self.do_locking = (self.use_locking and
-                       query.NQ_LIVE in self.requested_data)
-
-    if self.do_locking:
-      # If any non-static field is requested we need to lock the nodes
-      lu.needed_locks[locking.LEVEL_NODE] = self.wanted
-      lu.needed_locks[locking.LEVEL_NODE_ALLOC] = locking.ALL_SET
-
-  def DeclareLocks(self, lu, level):
-    pass
-
-  def _GetQueryData(self, lu):
-    """Computes the list of nodes and their attributes.
-
-    """
-    all_info = lu.cfg.GetAllNodesInfo()
-
-    node_uuids = self._GetNames(lu, all_info.keys(), locking.LEVEL_NODE)
-
-    # Gather data as requested
-    if query.NQ_LIVE in self.requested_data:
-      # filter out non-vm_capable nodes
-      toquery_node_uuids = [node.uuid for node in all_info.values()
-                            if node.vm_capable and node.uuid in node_uuids]
-      lvm_enabled = utils.storage.IsLvmEnabled(
-          lu.cfg.GetClusterInfo().enabled_disk_templates)
-      # FIXME: this per default asks for storage space information for all
-      # enabled disk templates. Fix this by making it possible to specify
-      # space report fields for specific disk templates.
-      raw_storage_units = utils.storage.GetStorageUnitsOfCluster(
-          lu.cfg, include_spindles=lvm_enabled)
-      storage_units = rpc.PrepareStorageUnitsForNodes(
-          lu.cfg, raw_storage_units, toquery_node_uuids)
-      default_hypervisor = lu.cfg.GetHypervisorType()
-      hvparams = lu.cfg.GetClusterInfo().hvparams[default_hypervisor]
-      hvspecs = [(default_hypervisor, hvparams)]
-      node_data = lu.rpc.call_node_info(toquery_node_uuids, storage_units,
-                                        hvspecs)
-      live_data = dict(
-          (uuid, rpc.MakeLegacyNodeInfo(nresult.payload,
-                                        require_spindles=lvm_enabled))
-          for (uuid, nresult) in node_data.items()
-          if not nresult.fail_msg and nresult.payload)
-    else:
-      live_data = None
-
-    if query.NQ_INST in self.requested_data:
-      node_to_primary = dict([(uuid, set()) for uuid in node_uuids])
-      node_to_secondary = dict([(uuid, set()) for uuid in node_uuids])
-
-      inst_data = lu.cfg.GetAllInstancesInfo()
-      inst_uuid_to_inst_name = {}
-
-      for inst in inst_data.values():
-        inst_uuid_to_inst_name[inst.uuid] = inst.name
-        if inst.primary_node in node_to_primary:
-          node_to_primary[inst.primary_node].add(inst.uuid)
-        for secnode in inst.secondary_nodes:
-          if secnode in node_to_secondary:
-            node_to_secondary[secnode].add(inst.uuid)
-    else:
-      node_to_primary = None
-      node_to_secondary = None
-      inst_uuid_to_inst_name = None
-
-    if query.NQ_OOB in self.requested_data:
-      oob_support = dict((uuid, bool(SupportsOob(lu.cfg, node)))
-                         for uuid, node in all_info.iteritems())
-    else:
-      oob_support = None
-
-    if query.NQ_GROUP in self.requested_data:
-      groups = lu.cfg.GetAllNodeGroupsInfo()
-    else:
-      groups = {}
-
-    return query.NodeQueryData([all_info[uuid] for uuid in node_uuids],
-                               live_data, lu.cfg.GetMasterNode(),
-                               node_to_primary, node_to_secondary,
-                               inst_uuid_to_inst_name, groups, oob_support,
-                               lu.cfg.GetClusterInfo())
-
-
-class LUNodeQuery(NoHooksLU):
-  """Logical unit for querying nodes.
+  @type fields: L{utils.FieldSet}
+  @param fields: fields set
+  @type selected: L{utils.FieldSet}
+  @param selected: fields set
 
   """
-  # pylint: disable=W0142
-  REQ_BGL = False
-
-  def CheckArguments(self):
-    self.nq = NodeQuery(qlang.MakeSimpleFilter("name", self.op.names),
-                         self.op.output_fields, self.op.use_locking)
-
-  def ExpandNames(self):
-    self.nq.ExpandNames(self)
-
-  def DeclareLocks(self, level):
-    self.nq.DeclareLocks(self, level)
-
-  def Exec(self, feedback_fn):
-    return self.nq.OldStyleQuery(self)
-
-
-def _CheckOutputFields(static, dynamic, selected):
-  """Checks whether all selected fields are valid.
-
-  @type static: L{utils.FieldSet}
-  @param static: static fields set
-  @type dynamic: L{utils.FieldSet}
-  @param dynamic: dynamic fields set
-
-  """
-  f = utils.FieldSet()
-  f.Extend(static)
-  f.Extend(dynamic)
-
-  delta = f.NonMatching(selected)
+  delta = fields.NonMatching(selected)
   if delta:
     raise errors.OpPrereqError("Unknown output fields selected: %s"
                                % ",".join(delta), errors.ECODE_INVAL)
@@ -1294,13 +1228,12 @@ class LUNodeQueryvols(NoHooksLU):
 
   """
   REQ_BGL = False
-  _FIELDS_DYNAMIC = utils.FieldSet("phys", "vg", "name", "size", "instance")
-  _FIELDS_STATIC = utils.FieldSet("node")
 
   def CheckArguments(self):
-    _CheckOutputFields(static=self._FIELDS_STATIC,
-                       dynamic=self._FIELDS_DYNAMIC,
-                       selected=self.op.output_fields)
+    _CheckOutputFields(utils.FieldSet(constants.VF_NODE, constants.VF_PHYS,
+                                      constants.VF_VG, constants.VF_NAME,
+                                      constants.VF_SIZE, constants.VF_INSTANCE),
+                       self.op.output_fields)
 
   def ExpandNames(self):
     self.share_locks = ShareAll()
@@ -1337,24 +1270,24 @@ class LUNodeQueryvols(NoHooksLU):
         continue
 
       node_vols = sorted(nresult.payload,
-                         key=operator.itemgetter("dev"))
+                         key=operator.itemgetter(constants.VF_DEV))
 
       for vol in node_vols:
         node_output = []
         for field in self.op.output_fields:
-          if field == "node":
+          if field == constants.VF_NODE:
             val = self.cfg.GetNodeName(node_uuid)
-          elif field == "phys":
-            val = vol["dev"]
-          elif field == "vg":
-            val = vol["vg"]
-          elif field == "name":
-            val = vol["name"]
-          elif field == "size":
-            val = int(float(vol["size"]))
-          elif field == "instance":
-            inst = vol2inst.get((node_uuid, vol["vg"] + "/" + vol["name"]),
-                                None)
+          elif field == constants.VF_PHYS:
+            val = vol[constants.VF_DEV]
+          elif field == constants.VF_VG:
+            val = vol[constants.VF_VG]
+          elif field == constants.VF_NAME:
+            val = vol[constants.VF_NAME]
+          elif field == constants.VF_SIZE:
+            val = int(float(vol[constants.VF_SIZE]))
+          elif field == constants.VF_INSTANCE:
+            inst = vol2inst.get((node_uuid, vol[constants.VF_VG] + "/" +
+                                 vol[constants.VF_NAME]), None)
             if inst is not None:
               val = inst.name
             else:
@@ -1372,13 +1305,11 @@ class LUNodeQueryStorage(NoHooksLU):
   """Logical unit for getting information on storage units on node(s).
 
   """
-  _FIELDS_STATIC = utils.FieldSet(constants.SF_NODE)
   REQ_BGL = False
 
   def CheckArguments(self):
-    _CheckOutputFields(static=self._FIELDS_STATIC,
-                       dynamic=utils.FieldSet(*constants.VALID_STORAGE_FIELDS),
-                       selected=self.op.output_fields)
+    _CheckOutputFields(utils.FieldSet(*constants.VALID_STORAGE_FIELDS),
+                       self.op.output_fields)
 
   def ExpandNames(self):
     self.share_locks = ShareAll()
@@ -1393,16 +1324,42 @@ class LUNodeQueryStorage(NoHooksLU):
         locking.LEVEL_NODE_ALLOC: locking.ALL_SET,
         }
 
+  def _DetermineStorageType(self):
+    """Determines the default storage type of the cluster.
+
+    """
+    enabled_disk_templates = self.cfg.GetClusterInfo().enabled_disk_templates
+    default_storage_type = \
+        constants.MAP_DISK_TEMPLATE_STORAGE_TYPE[enabled_disk_templates[0]]
+    return default_storage_type
+
   def CheckPrereq(self):
     """Check prerequisites.
 
     """
-    CheckStorageTypeEnabled(self.cfg.GetClusterInfo(), self.op.storage_type)
+    if self.op.storage_type:
+      CheckStorageTypeEnabled(self.cfg.GetClusterInfo(), self.op.storage_type)
+      self.storage_type = self.op.storage_type
+    else:
+      self.storage_type = self._DetermineStorageType()
+      supported_storage_types = constants.STS_REPORT_NODE_STORAGE
+      if self.storage_type not in supported_storage_types:
+        raise errors.OpPrereqError(
+            "Storage reporting for storage type '%s' is not supported. Please"
+            " use the --storage-type option to specify one of the supported"
+            " storage types (%s) or set the default disk template to one that"
+            " supports storage reporting." %
+            (self.storage_type, utils.CommaJoin(supported_storage_types)))
 
   def Exec(self, feedback_fn):
     """Computes the list of nodes and their attributes.
 
     """
+    if self.op.storage_type:
+      self.storage_type = self.op.storage_type
+    else:
+      self.storage_type = self._DetermineStorageType()
+
     self.node_uuids = self.owned_locks(locking.LEVEL_NODE)
 
     # Always get name to sort by
@@ -1419,9 +1376,9 @@ class LUNodeQueryStorage(NoHooksLU):
     field_idx = dict([(name, idx) for (idx, name) in enumerate(fields)])
     name_idx = field_idx[constants.SF_NAME]
 
-    st_args = _GetStorageTypeArgs(self.cfg, self.op.storage_type)
+    st_args = _GetStorageTypeArgs(self.cfg, self.storage_type)
     data = self.rpc.call_storage_list(self.node_uuids,
-                                      self.op.storage_type, st_args,
+                                      self.storage_type, st_args,
                                       self.op.name, fields)
 
     result = []
@@ -1449,7 +1406,7 @@ class LUNodeQueryStorage(NoHooksLU):
           if field == constants.SF_NODE:
             val = node_name
           elif field == constants.SF_TYPE:
-            val = self.op.storage_type
+            val = self.storage_type
           elif field in field_idx:
             val = row[field_idx[field]]
           else:
@@ -1534,7 +1491,7 @@ class LUNodeRemove(LogicalUnit):
       "Not owning BGL"
 
     # Promote nodes to master candidate as needed
-    AdjustCandidatePool(self, exceptions=[self.node.uuid])
+    AdjustCandidatePool(self, [self.node.uuid], feedback_fn)
     self.context.RemoveNode(self.node)
 
     # Run post hooks on the node before it's removed
@@ -1548,8 +1505,16 @@ class LUNodeRemove(LogicalUnit):
       self.LogWarning("Errors encountered on the remote node while leaving"
                       " the cluster: %s", msg)
 
+    cluster = self.cfg.GetClusterInfo()
+
+    # Remove node from candidate certificate list
+    if self.node.master_candidate:
+      utils.RemoveNodeFromCandidateCerts(self.node.uuid,
+                                         cluster.candidate_certs)
+      self.cfg.Update(cluster, feedback_fn)
+
     # Remove node from our /etc/hosts
-    if self.cfg.GetClusterInfo().modify_etc_hosts:
+    if cluster.modify_etc_hosts:
       master_node_uuid = self.cfg.GetMasterNode()
       result = self.rpc.call_etc_hosts_modify(master_node_uuid,
                                               constants.ETC_HOSTS_REMOVE,
