@@ -144,7 +144,7 @@ class AsyncStreamServer(GanetiBaseAsyncoreDispatcher):
   _REQUEST_QUEUE_SIZE = 5
 
   def __init__(self, family, address):
-    """Constructor for AsyncUnixStreamSocket
+    """Constructor for AsyncStreamServer
 
     @type family: integer
     @param family: socket family (one of socket.AF_*)
@@ -457,7 +457,7 @@ class AsyncAwaker(GanetiBaseAsyncoreDispatcher):
       self.out_socket.send(chr(0))
 
 
-class _ShutdownCheck:
+class _ShutdownCheck(object):
   """Logic for L{Mainloop} shutdown.
 
   """
