@@ -144,6 +144,19 @@ def ParamInSet(required, my_set):
   return (required, fn, err, None, None)
 
 
+class HvInstanceState(object):
+  RUNNING = 0
+  SHUTDOWN = 1
+
+  @staticmethod
+  def IsRunning(s):
+    return s == HvInstanceState.RUNNING
+
+  @staticmethod
+  def IsShutdown(s):
+    return s == HvInstanceState.SHUTDOWN
+
+
 class BaseHypervisor(object):
   """Abstract virtualisation technology interface
 
@@ -173,7 +186,8 @@ class BaseHypervisor(object):
     """Start an instance."""
     raise NotImplementedError
 
-  def StopInstance(self, instance, force=False, retry=False, name=None):
+  def StopInstance(self, instance, force=False, retry=False, name=None,
+                   timeout=None):
     """Stop an instance
 
     @type instance: L{objects.Instance}
@@ -186,6 +200,12 @@ class BaseHypervisor(object):
     @param name: if this parameter is passed, the the instance object
         should not be used (will be passed as None), and the shutdown
         must be done by name only
+    @type timeout: int or None
+    @param timeout: if the parameter is not None, a soft shutdown operation will
+        be killed after the specified number of seconds. A hard (forced)
+        shutdown cannot have a timeout
+    @raise errors.HypervisorError: when a parameter is not valid or
+        the instance failed to be stopped
 
     """
     raise NotImplementedError
@@ -218,6 +238,7 @@ class BaseHypervisor(object):
     @type hvparams: dict of strings
     @param hvparams: hvparams to be used with this instance
 
+    @rtype: (string, string, int, int, HvInstanceState, int)
     @return: tuple (name, id, memory, vcpus, state, times)
 
     """
@@ -228,7 +249,9 @@ class BaseHypervisor(object):
 
     @type hvparams: dict of strings
     @param hvparams: hypervisor parameter
-    @return: list of tuples (name, id, memory, vcpus, stat, times)
+
+    @rtype: (string, string, int, int, HvInstanceState, int)
+    @return: list of tuples (name, id, memory, vcpus, state, times)
 
     """
     raise NotImplementedError
@@ -252,7 +275,8 @@ class BaseHypervisor(object):
     raise NotImplementedError
 
   @classmethod
-  def GetInstanceConsole(cls, instance, primary_node, hvparams, beparams):
+  def GetInstanceConsole(cls, instance, primary_node, node_group,
+                         hvparams, beparams):
     """Return information for connecting to the console of an instance.
 
     """
@@ -563,3 +587,50 @@ class BaseHypervisor(object):
       return "; ".join(msgs)
     else:
       return None
+
+  # pylint: disable=R0201,W0613
+  def HotAddDevice(self, instance, dev_type, device, extra, seq):
+    """Hot-add a device.
+
+    """
+    raise errors.HotplugError("Hotplug is not supported by this hypervisor")
+
+  # pylint: disable=R0201,W0613
+  def HotDelDevice(self, instance, dev_type, device, extra, seq):
+    """Hot-del a device.
+
+    """
+    raise errors.HotplugError("Hotplug is not supported by this hypervisor")
+
+  # pylint: disable=R0201,W0613
+  def HotModDevice(self, instance, dev_type, device, extra, seq):
+    """Hot-mod a device.
+
+    """
+    raise errors.HotplugError("Hotplug is not supported by this hypervisor")
+
+  # pylint: disable=R0201,W0613
+  def VerifyHotplugSupport(self, instance, action, dev_type):
+    """Verifies that hotplug is supported.
+
+    Given the target device and hotplug action checks if hotplug is
+    actually supported.
+
+    @type instance: L{objects.Instance}
+    @param instance: the instance object
+    @type action: string
+    @param action: one of the supported hotplug commands
+    @type dev_type: string
+    @param dev_type: one of the supported device types to hotplug
+    @raise errors.HotplugError: if hotplugging is not supported
+
+    """
+    raise errors.HotplugError("Hotplug is not supported.")
+
+  def HotplugSupported(self, instance):
+    """Checks if hotplug is supported.
+
+    By default is not. Currently only KVM hypervisor supports it.
+
+    """
+    raise errors.HotplugError("Hotplug is not supported by this hypervisor")
