@@ -123,7 +123,7 @@ class ChrootManager(hv_base.BaseHypervisor):
     dir_name = self._InstanceDir(instance_name)
     if not self._IsDirLive(dir_name):
       raise HypervisorError("Instance %s is not running" % instance_name)
-    return (instance_name, 0, 0, 0, 0, 0)
+    return (instance_name, 0, 0, 0, hv_base.HvInstanceState.RUNNING, 0)
 
   def GetAllInstancesInfo(self, hvparams=None):
     """Get properties of all instances.
@@ -171,7 +171,8 @@ class ChrootManager(hv_base.BaseHypervisor):
       raise HypervisorError("Can't run the chroot start script: %s" %
                             result.output)
 
-  def StopInstance(self, instance, force=False, retry=False, name=None):
+  def StopInstance(self, instance, force=False, retry=False, name=None,
+                   timeout=None):
     """Stop an instance.
 
     This method has complicated cleanup tests, as we must:
@@ -180,6 +181,8 @@ class ChrootManager(hv_base.BaseHypervisor):
       - finally unmount the instance dir
 
     """
+    assert(timeout is None or force is not None)
+
     if name is None:
       name = instance.name
 
@@ -187,9 +190,14 @@ class ChrootManager(hv_base.BaseHypervisor):
     if not os.path.exists(root_dir) or not self._IsDirLive(root_dir):
       return
 
+    timeout_cmd = []
+    if timeout is not None:
+      timeout_cmd.extend(["timeout", str(timeout)])
+
     # Run the chroot stop script only once
     if not retry and not force:
-      result = utils.RunCmd(["chroot", root_dir, "/ganeti-chroot", "stop"])
+      result = utils.RunCmd(timeout_cmd.extend(["chroot", root_dir,
+                                                "/ganeti-chroot", "stop"]))
       if result.failed:
         raise HypervisorError("Can't run the chroot stop script: %s" %
                               result.output)
@@ -260,7 +268,7 @@ class ChrootManager(hv_base.BaseHypervisor):
 
   @classmethod
   def GetInstanceConsole(cls, instance, primary_node, # pylint: disable=W0221
-                         hvparams, beparams, root_dir=None):
+                         node_group, hvparams, beparams, root_dir=None):
     """Return information for connecting to the console of an instance.
 
     """
@@ -269,9 +277,11 @@ class ChrootManager(hv_base.BaseHypervisor):
       if not os.path.ismount(root_dir):
         raise HypervisorError("Instance %s is not running" % instance.name)
 
+    ndparams = node_group.FillND(primary_node)
     return objects.InstanceConsole(instance=instance.name,
                                    kind=constants.CONS_SSH,
                                    host=primary_node.name,
+                                   port=ndparams.get(constants.ND_SSH_PORT),
                                    user=constants.SSH_CONSOLE_USER,
                                    command=["chroot", root_dir])
 
