@@ -189,7 +189,8 @@ class LXCHypervisor(hv_base.BaseHypervisor):
 
     cpu_list = self._GetCgroupCpuList(instance_name)
     memory = self._GetCgroupMemoryLimit(instance_name) / (1024 ** 2)
-    return (instance_name, 0, memory, len(cpu_list), 0, 0)
+    return (instance_name, 0, memory, len(cpu_list),
+            hv_base.HvInstanceState.RUNNING, 0)
 
   def GetAllInstancesInfo(self, hvparams=None):
     """Get properties of all instances.
@@ -329,7 +330,8 @@ class LXCHypervisor(hv_base.BaseHypervisor):
       raise HypervisorError("Running the lxc-start script failed: %s" %
                             result.output)
 
-  def StopInstance(self, instance, force=False, retry=False, name=None):
+  def StopInstance(self, instance, force=False, retry=False, name=None,
+                   timeout=None):
     """Stop an instance.
 
     This method has complicated cleanup tests, as we must:
@@ -338,9 +340,15 @@ class LXCHypervisor(hv_base.BaseHypervisor):
       - finally unmount the instance dir
 
     """
+    assert(timeout is None or force is not None)
+
     if name is None:
       name = instance.name
 
+    timeout_cmd = []
+    if timeout is not None:
+      timeout_cmd.extend(["timeout", str(timeout)])
+
     root_dir = self._InstanceDir(name)
     if not os.path.exists(root_dir):
       return
@@ -353,7 +361,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
           raise HypervisorError("Running 'poweroff' on the instance"
                                 " failed: %s" % result.output)
       time.sleep(2)
-      result = utils.RunCmd(["lxc-stop", "-n", name])
+      result = utils.RunCmd(timeout_cmd.extend(["lxc-stop", "-n", name]))
       if result.failed:
         logging.warning("Error while doing lxc-stop for %s: %s", name,
                         result.output)
@@ -362,12 +370,12 @@ class LXCHypervisor(hv_base.BaseHypervisor):
       return
 
     for mpath in self._GetMountSubdirs(root_dir):
-      result = utils.RunCmd(["umount", mpath])
+      result = utils.RunCmd(timeout_cmd.extend(["umount", mpath]))
       if result.failed:
         logging.warning("Error while umounting subpath %s for instance %s: %s",
                         mpath, name, result.output)
 
-    result = utils.RunCmd(["umount", root_dir])
+    result = utils.RunCmd(timeout_cmd.extend(["umount", root_dir]))
     if result.failed and force:
       msg = ("Processes still alive in the chroot: %s" %
              utils.RunCmd("fuser -vm %s" % root_dir).output)
@@ -406,13 +414,16 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     return self.GetLinuxNodeInfo()
 
   @classmethod
-  def GetInstanceConsole(cls, instance, primary_node, hvparams, beparams):
+  def GetInstanceConsole(cls, instance, primary_node, node_group,
+                         hvparams, beparams):
     """Return a command for connecting to the console of an instance.
 
     """
+    ndparams = node_group.FillND(primary_node)
     return objects.InstanceConsole(instance=instance.name,
                                    kind=constants.CONS_SSH,
                                    host=primary_node.name,
+                                   port=ndparams.get(constants.ND_SSH_PORT),
                                    user=constants.SSH_CONSOLE_USER,
                                    command=["lxc-console", "-n", instance.name])
 
