@@ -85,8 +85,8 @@ def _CreateConfigCpus(cpu_mask):
 
 
 def _RunInstanceList(fn, instance_list_errors):
-  """Helper function for L{_GetInstanceList} to retrieve the list of instances
-  from xen.
+  """Helper function for L{_GetAllInstanceList} to retrieve the list
+  of instances from xen.
 
   @type fn: callable
   @param fn: Function to query xen for the list of instances
@@ -132,6 +132,7 @@ def _ParseInstanceList(lines, include_node):
       data[1] = int(data[1])
       data[2] = int(data[2])
       data[3] = int(data[3])
+      data[4] = _XenToHypervisorInstanceState(data[4])
       data[5] = float(data[5])
     except (TypeError, ValueError), err:
       raise errors.HypervisorError("Can't parse instance list,"
@@ -144,8 +145,8 @@ def _ParseInstanceList(lines, include_node):
   return result
 
 
-def _GetInstanceList(fn, include_node, _timeout=5):
-  """Return the list of running instances.
+def _GetAllInstanceList(fn, include_node, _timeout=5):
+  """Return the list of instances including running and shutdown.
 
   See L{_RunInstanceList} and L{_ParseInstanceList} for parameter details.
 
@@ -168,15 +169,57 @@ def _GetInstanceList(fn, include_node, _timeout=5):
   return _ParseInstanceList(lines, include_node)
 
 
+# Determine whether an instance is running
+#
+# An instance is running if it is in the following Xen states:
+# running, blocked, or paused.
+#
+# For some strange reason, Xen once printed 'rb----' which does not
+# make any sense because an instance cannot be both running and
+# blocked.  Fortunately, for Ganeti 'running' or 'blocked' is the same
+# as 'running'.
 def _IsInstanceRunning(instance_info):
   return instance_info == "r-----" \
-      or instance_info == "-b----"
+      or instance_info == "-b----" \
+      or instance_info == "rb----" \
+      or instance_info == "--p---"
 
 
 def _IsInstanceShutdown(instance_info):
   return instance_info == "---s--"
 
 
+def _XenToHypervisorInstanceState(instance_info):
+  if _IsInstanceRunning(instance_info):
+    return hv_base.HvInstanceState.RUNNING
+  elif _IsInstanceShutdown(instance_info):
+    return hv_base.HvInstanceState.SHUTDOWN
+  else:
+    raise errors.HypervisorError("hv_xen._XenToHypervisorInstanceState:"
+                                 " unhandled Xen instance state '%s'" %
+                                   instance_info)
+
+
+def _GetRunningInstanceList(fn, include_node, _timeout=5):
+  """Return the list of running instances.
+
+  See L{_GetAllInstanceList} for parameter details.
+
+  """
+  instances = _GetAllInstanceList(fn, include_node, _timeout)
+  return [i for i in instances if hv_base.HvInstanceState.IsRunning(i[4])]
+
+
+def _GetShutdownInstanceList(fn, include_node, _timeout=5):
+  """Return the list of shutdown instances.
+
+  See L{_GetAllInstanceList} for parameter details.
+
+  """
+  instances = _GetAllInstanceList(fn, include_node, _timeout)
+  return [i for i in instances if hv_base.HvInstanceState.IsShutdown(i[4])]
+
+
 def _ParseNodeInfo(info):
   """Return information about the node.
 
@@ -297,7 +340,7 @@ def _GetConfigFileDiskData(block_devices, blockdev_prefix,
 
   disk_data = []
 
-  for sd_suffix, (cfdev, dev_path) in zip(_letters, block_devices):
+  for sd_suffix, (cfdev, dev_path, _) in zip(_letters, block_devices):
     sd_name = blockdev_prefix + sd_suffix
 
     if cfdev.mode == constants.DISK_RDWR:
@@ -305,8 +348,8 @@ def _GetConfigFileDiskData(block_devices, blockdev_prefix,
     else:
       mode = "r"
 
-    if cfdev.dev_type in [constants.DT_FILE, constants.DT_SHARED_FILE]:
-      driver = _FILE_DRIVER_MAP[cfdev.physical_id[0]]
+    if cfdev.dev_type in constants.DTS_FILEBASED:
+      driver = _FILE_DRIVER_MAP[cfdev.logical_id[0]]
     else:
       driver = "phy"
 
@@ -315,6 +358,19 @@ def _GetConfigFileDiskData(block_devices, blockdev_prefix,
   return disk_data
 
 
+def _QuoteCpuidField(data):
+  """Add quotes around the CPUID field only if necessary.
+
+  Xen CPUID fields come in two shapes: LIBXL strings, which need quotes around
+  them, and lists of XEND strings, which don't.
+
+  @param data: Either type of parameter.
+  @return: The quoted version thereof.
+
+  """
+  return "'%s'" % data if data.startswith("host") else data
+
+
 class XenHypervisor(hv_base.BaseHypervisor):
   """Xen generic hypervisor interface
 
@@ -383,15 +439,23 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     return cmd
 
-  def _RunXen(self, args, hvparams):
+  def _RunXen(self, args, hvparams, timeout=None):
     """Wrapper around L{utils.process.RunCmd} to run Xen command.
 
     @type hvparams: dict of strings
     @param hvparams: dictionary of hypervisor params
+    @type timeout: int or None
+    @param timeout: if a timeout (in seconds) is specified, the command will be
+                    terminated after that number of seconds.
     @see: L{utils.process.RunCmd}
 
     """
-    cmd = [self._GetCommand(hvparams)]
+    cmd = []
+
+    if timeout is not None:
+      cmd.extend(["timeout", str(timeout)])
+
+    cmd.extend([self._GetCommand(hvparams)])
     cmd.extend(args)
 
     return self._run_cmd_fn(cmd)
@@ -408,12 +472,13 @@ class XenHypervisor(hv_base.BaseHypervisor):
     return utils.PathJoin(self._cfgdir, instance_name)
 
   @classmethod
-  def _WriteNICInfoFile(cls, instance_name, idx, nic):
+  def _WriteNICInfoFile(cls, instance, idx, nic):
     """Write the Xen config file for the instance.
 
     This version of the function just writes the config file from static data.
 
     """
+    instance_name = instance.name
     dirs = [(dname, constants.RUN_DIRS_MODE)
             for dname in cls._DIRS + [cls._InstanceNICDir(instance_name)]]
     utils.EnsureDirs(dirs)
@@ -421,26 +486,22 @@ class XenHypervisor(hv_base.BaseHypervisor):
     cfg_file = cls._InstanceNICFile(instance_name, idx)
     data = StringIO()
 
+    data.write("TAGS=%s\n" % r"\ ".join(instance.GetTags()))
     if nic.netinfo:
       netinfo = objects.Network.FromDict(nic.netinfo)
-      data.write("NETWORK_NAME=%s\n" % netinfo.name)
-      if netinfo.network:
-        data.write("NETWORK_SUBNET=%s\n" % netinfo.network)
-      if netinfo.gateway:
-        data.write("NETWORK_GATEWAY=%s\n" % netinfo.gateway)
-      if netinfo.network6:
-        data.write("NETWORK_SUBNET6=%s\n" % netinfo.network6)
-      if netinfo.gateway6:
-        data.write("NETWORK_GATEWAY6=%s\n" % netinfo.gateway6)
-      if netinfo.mac_prefix:
-        data.write("NETWORK_MAC_PREFIX=%s\n" % netinfo.mac_prefix)
-      if netinfo.tags:
-        data.write("NETWORK_TAGS=%s\n" % r"\ ".join(netinfo.tags))
+      for k, v in netinfo.HooksDict().iteritems():
+        data.write("%s=%s\n" % (k, v))
 
     data.write("MAC=%s\n" % nic.mac)
-    data.write("IP=%s\n" % nic.ip)
+    if nic.ip:
+      data.write("IP=%s\n" % nic.ip)
+    data.write("INTERFACE_INDEX=%s\n" % str(idx))
+    if nic.name:
+      data.write("INTERFACE_NAME=%s\n" % nic.name)
+    data.write("INTERFACE_UUID=%s\n" % nic.uuid)
     data.write("MODE=%s\n" % nic.nicparams[constants.NIC_MODE])
     data.write("LINK=%s\n" % nic.nicparams[constants.NIC_LINK])
+    data.write("VLAN=%s\n" % nic.nicparams[constants.NIC_VLAN])
 
     try:
       utils.WriteFile(cfg_file, data=data.getvalue())
@@ -521,22 +582,29 @@ class XenHypervisor(hv_base.BaseHypervisor):
     return new_filename
 
   def _GetInstanceList(self, include_node, hvparams):
-    """Wrapper around module level L{_GetInstanceList}.
+    """Wrapper around module level L{_GetAllInstanceList}.
 
     @type hvparams: dict of strings
     @param hvparams: hypervisor parameters to be used on this node
 
     """
-    return _GetInstanceList(lambda: self._RunXen(["list"], hvparams),
-                            include_node)
+    return _GetAllInstanceList(lambda: self._RunXen(["list"], hvparams),
+                               include_node)
 
   def ListInstances(self, hvparams=None):
     """Get the list of running instances.
 
+    @type hvparams: dict of strings
+    @param hvparams: the instance's hypervisor params
+
+    @rtype: list of strings
+    @return: names of running instances
+
     """
-    instance_list = self._GetInstanceList(False, hvparams)
-    names = [info[0] for info in instance_list]
-    return names
+    instance_list = _GetRunningInstanceList(
+      lambda: self._RunXen(["list"], hvparams),
+      False)
+    return [info[0] for info in instance_list]
 
   def GetInstanceInfo(self, instance_name, hvparams=None):
     """Get instance properties.
@@ -562,7 +630,9 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     @type hvparams: dict of strings
     @param hvparams: hypervisor parameters
-    @return: list of tuples (name, id, memory, vcpus, stat, times)
+
+    @rtype: (string, string, int, int, HypervisorInstanceState, int)
+    @return: list of tuples (name, id, memory, vcpus, state, times)
 
     """
     return self._GetInstanceList(False, hvparams)
@@ -604,26 +674,34 @@ class XenHypervisor(hv_base.BaseHypervisor):
                                    (instance.name, result.fail_reason,
                                     result.output, stashed_config))
 
-  def StopInstance(self, instance, force=False, retry=False, name=None):
+  def StopInstance(self, instance, force=False, retry=False, name=None,
+                   timeout=None):
     """Stop an instance.
 
+    A soft shutdown can be interrupted. A hard shutdown tries forever.
+
     """
+    assert(timeout is None or force is not None)
+
     if name is None:
       name = instance.name
 
-    return self._StopInstance(name, force, instance.hvparams)
+    return self._StopInstance(name, force, instance.hvparams, timeout)
 
-  def _ShutdownInstance(self, name, hvparams):
+  def _ShutdownInstance(self, name, hvparams, timeout):
     """Shutdown an instance if the instance is running.
 
+    The '-w' flag waits for shutdown to complete which avoids the need
+    to poll in the case where we want to destroy the domain
+    immediately after shutdown.
+
     @type name: string
     @param name: name of the instance to stop
     @type hvparams: dict of string
     @param hvparams: hypervisor parameters of the instance
-
-    The '-w' flag waits for shutdown to complete which avoids the need
-    to poll in the case where we want to destroy the domain
-    immediately after shutdown.
+    @type timeout: int or None
+    @param timeout: a timeout after which the shutdown command should be killed,
+                    or None for no timeout
 
     """
     instance_info = self.GetInstanceInfo(name, hvparams=hvparams)
@@ -632,7 +710,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
       logging.info("Failed to shutdown instance %s, not running", name)
       return None
 
-    return self._RunXen(["shutdown", "-w", name], hvparams)
+    return self._RunXen(["shutdown", "-w", name], hvparams, timeout)
 
   def _DestroyInstance(self, name, hvparams):
     """Destroy an instance if the instance if the instance exists.
@@ -651,7 +729,24 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     return self._RunXen(["destroy", name], hvparams)
 
-  def _StopInstance(self, name, force, hvparams):
+  # Destroy a domain only if necessary
+  #
+  # This method checks if the domain has already been destroyed before
+  # issuing the 'destroy' command.  This step is necessary to handle
+  # domains created by other versions of Ganeti.  For example, an
+  # instance created with 2.10 will be destroy by the
+  # '_ShutdownInstance', thus not requiring an additional destroy,
+  # which would cause an error if issued.  See issue 619.
+  def _DestroyInstanceIfAlive(self, name, hvparams):
+    instance_info = self.GetInstanceInfo(name, hvparams=hvparams)
+
+    if instance_info is None:
+      raise errors.HypervisorError("Failed to destroy instance %s, already"
+                                   " destroyed" % name)
+    else:
+      self._DestroyInstance(name, hvparams)
+
+  def _StopInstance(self, name, force, hvparams, timeout):
     """Stop an instance.
 
     @type name: string
@@ -663,12 +758,22 @@ class XenHypervisor(hv_base.BaseHypervisor):
     @type hvparams: dict of string
     @param hvparams: hypervisor parameters of the instance
 
+    @type timeout: int or None
+    @param timeout: a timeout after which the shutdown command should be killed,
+                    or None for no timeout
+
     """
+    instance_info = self.GetInstanceInfo(name, hvparams=hvparams)
+
+    if instance_info is None:
+      raise errors.HypervisorError("Failed to shutdown instance %s,"
+                                   " not running" % name)
+
     if force:
-      result = self._DestroyInstance(name, hvparams)
+      result = self._DestroyInstanceIfAlive(name, hvparams)
     else:
-      self._ShutdownInstance(name, hvparams)
-      result = self._DestroyInstance(name, hvparams)
+      self._ShutdownInstance(name, hvparams, timeout)
+      result = self._DestroyInstanceIfAlive(name, hvparams)
 
     if result is not None and result.failed and \
           self.GetInstanceInfo(name, hvparams=hvparams) is not None:
@@ -753,14 +858,17 @@ class XenHypervisor(hv_base.BaseHypervisor):
     return _GetNodeInfo(result.stdout, instance_list)
 
   @classmethod
-  def GetInstanceConsole(cls, instance, primary_node, hvparams, beparams):
+  def GetInstanceConsole(cls, instance, primary_node, node_group,
+                         hvparams, beparams):
     """Return a command for connecting to the console of an instance.
 
     """
     xen_cmd = XenHypervisor._GetCommandFromHvparams(hvparams)
+    ndparams = node_group.FillND(primary_node)
     return objects.InstanceConsole(instance=instance.name,
                                    kind=constants.CONS_SSH,
                                    host=primary_node.name,
+                                   port=ndparams.get(constants.ND_SSH_PORT),
                                    user=constants.SSH_CONSOLE_USER,
                                    command=[pathutils.XEN_CONSOLE_WRAPPER,
                                             xen_cmd, instance.name])
@@ -997,6 +1105,15 @@ class XenHypervisor(hv_base.BaseHypervisor):
                                      % (constants.XEN_CMD_XL, result.stderr))
 
 
+def WriteXenConfigEvents(config, hvp):
+  config.write("on_poweroff = 'preserve'\n")
+  if hvp[constants.HV_REBOOT_BEHAVIOR] == constants.INSTANCE_REBOOT_ALLOWED:
+    config.write("on_reboot = 'restart'\n")
+  else:
+    config.write("on_reboot = 'destroy'\n")
+  config.write("on_crash = 'restart'\n")
+
+
 class XenPvmHypervisor(XenHypervisor):
   """Xen PVM hypervisor interface"""
 
@@ -1021,6 +1138,8 @@ class XenPvmHypervisor(XenHypervisor):
     constants.HV_VIF_SCRIPT: hv_base.OPT_FILE_CHECK,
     constants.HV_XEN_CMD:
       hv_base.ParamInSet(True, constants.KNOWN_XEN_COMMANDS),
+    constants.HV_XEN_CPUID: hv_base.NO_CHECK,
+    constants.HV_SOUNDHW: hv_base.NO_CHECK,
     }
 
   def _GetConfig(self, instance, startup_memory, block_devices):
@@ -1079,10 +1198,14 @@ class XenPvmHypervisor(XenHypervisor):
         nic_str += ", ip=%s" % ip
       if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_BRIDGED:
         nic_str += ", bridge=%s" % nic.nicparams[constants.NIC_LINK]
+      if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_OVS:
+        nic_str += ", bridge=%s" % nic.nicparams[constants.NIC_LINK]
+        if nic.nicparams[constants.NIC_VLAN]:
+          nic_str += "%s" % nic.nicparams[constants.NIC_VLAN]
       if hvp[constants.HV_VIF_SCRIPT]:
         nic_str += ", script=%s" % hvp[constants.HV_VIF_SCRIPT]
       vif_data.append("'%s'" % nic_str)
-      self._WriteNICInfoFile(instance.name, idx, nic)
+      self._WriteNICInfoFile(instance, idx, nic)
 
     disk_data = \
       _GetConfigFileDiskData(block_devices, hvp[constants.HV_BLOCKDEV_PREFIX])
@@ -1092,14 +1215,17 @@ class XenPvmHypervisor(XenHypervisor):
 
     if hvp[constants.HV_ROOT_PATH]:
       config.write("root = '%s'\n" % hvp[constants.HV_ROOT_PATH])
-    config.write("on_poweroff = 'destroy'\n")
-    if hvp[constants.HV_REBOOT_BEHAVIOR] == constants.INSTANCE_REBOOT_ALLOWED:
-      config.write("on_reboot = 'restart'\n")
-    else:
-      config.write("on_reboot = 'destroy'\n")
-    config.write("on_crash = 'restart'\n")
+
+    WriteXenConfigEvents(config, hvp)
     config.write("extra = '%s'\n" % hvp[constants.HV_KERNEL_ARGS])
 
+    cpuid = hvp[constants.HV_XEN_CPUID]
+    if cpuid:
+      config.write("cpuid = %s\n" % _QuoteCpuidField(cpuid))
+
+    if hvp[constants.HV_SOUNDHW]:
+      config.write("soundhw = '%s'\n" % hvp[constants.HV_SOUNDHW])
+
     return config.getvalue()
 
 
@@ -1150,6 +1276,8 @@ class XenHvmHypervisor(XenHypervisor):
     constants.HV_VIRIDIAN: hv_base.NO_CHECK,
     constants.HV_XEN_CMD:
       hv_base.ParamInSet(True, constants.KNOWN_XEN_COMMANDS),
+    constants.HV_XEN_CPUID: hv_base.NO_CHECK,
+    constants.HV_SOUNDHW: hv_base.NO_CHECK,
     }
 
   def _GetConfig(self, instance, startup_memory, block_devices):
@@ -1253,7 +1381,7 @@ class XenHvmHypervisor(XenHypervisor):
       if hvp[constants.HV_VIF_SCRIPT]:
         nic_str += ", script=%s" % hvp[constants.HV_VIF_SCRIPT]
       vif_data.append("'%s'" % nic_str)
-      self._WriteNICInfoFile(instance.name, idx, nic)
+      self._WriteNICInfoFile(instance, idx, nic)
 
     config.write("vif = [%s]\n" % ",".join(vif_data))
 
@@ -1272,11 +1400,14 @@ class XenHvmHypervisor(XenHypervisor):
     if pci_pass:
       pci_pass_arr = pci_pass.split(";")
       config.write("pci = %s\n" % pci_pass_arr)
-    config.write("on_poweroff = 'destroy'\n")
-    if hvp[constants.HV_REBOOT_BEHAVIOR] == constants.INSTANCE_REBOOT_ALLOWED:
-      config.write("on_reboot = 'restart'\n")
-    else:
-      config.write("on_reboot = 'destroy'\n")
-    config.write("on_crash = 'restart'\n")
+
+    WriteXenConfigEvents(config, hvp)
+
+    cpuid = hvp[constants.HV_XEN_CPUID]
+    if cpuid:
+      config.write("cpuid = %s\n" % _QuoteCpuidField(cpuid))
+
+    if hvp[constants.HV_SOUNDHW]:
+      config.write("soundhw = '%s'\n" % hvp[constants.HV_SOUNDHW])
 
     return config.getvalue()
