@@ -969,7 +969,7 @@ class _AcquireTimeout(Exception):
   """
 
 
-class LockSet:
+class LockSet(object):
   """Implements a set of locks.
 
   This abstraction implements a set of shared locks for the same resource type,
@@ -1618,7 +1618,7 @@ BGL = "BGL"
 NAL = "NAL"
 
 
-class GanetiLockManager:
+class GanetiLockManager(object):
   """The Ganeti Locking Library
 
   The purpose of this small library is to manage locking for ganeti clusters
