@@ -659,7 +659,7 @@ def FormatProgress(progress):
   return utils.CommaJoin(parts)
 
 
-class ImportExportLoop:
+class ImportExportLoop(object):
   MIN_DELAY = 1.0
   MAX_DELAY = 20.0
 
@@ -1007,7 +1007,7 @@ def _GetInstDiskMagic(base, instance_name, index):
 
 
 def TransferInstanceData(lu, feedback_fn, src_node_uuid, dest_node_uuid,
-                         dest_ip, instance, all_transfers):
+                         dest_ip, compress, instance, all_transfers):
   """Transfers an instance's data from one node to another.
 
   @param lu: Logical unit instance
@@ -1018,6 +1018,8 @@ def TransferInstanceData(lu, feedback_fn, src_node_uuid, dest_node_uuid,
   @param dest_node_uuid: Destination node UUID
   @type dest_ip: string
   @param dest_ip: IP address of destination node
+  @type compress: string
+  @param compress: one of L{constants.IEC_ALL}
   @type instance: L{objects.Instance}
   @param instance: Instance object
   @type all_transfers: list of L{DiskTransfer} instances
@@ -1027,9 +1029,6 @@ def TransferInstanceData(lu, feedback_fn, src_node_uuid, dest_node_uuid,
            each transfer
 
   """
-  # Disable compression for all moves as these are all within the same cluster
-  compress = constants.IEC_NONE
-
   src_node_name = lu.cfg.GetNodeName(src_node_uuid)
   dest_node_name = lu.cfg.GetNodeName(dest_node_uuid)
 
@@ -1140,7 +1139,7 @@ class _RemoteExportCb(ImportExportCbBase):
       finished_fn()
 
 
-class ExportInstanceHelper:
+class ExportInstanceHelper(object):
   def __init__(self, lu, feedback_fn, instance):
     """Initializes this class.
 
@@ -1165,10 +1164,11 @@ class ExportInstanceHelper:
 
     instance = self._instance
     src_node = instance.primary_node
+    src_node_name = self._lu.cfg.GetNodeName(src_node)
 
     for idx, disk in enumerate(instance.disks):
       self._feedback_fn("Creating a snapshot of disk/%s on node %s" %
-                        (idx, src_node))
+                        (idx, src_node_name))
 
       # result.payload will be a snapshot of an lvm leaf of the one we
       # passed
@@ -1177,17 +1177,16 @@ class ExportInstanceHelper:
       msg = result.fail_msg
       if msg:
         self._lu.LogWarning("Could not snapshot disk/%s on node %s: %s",
-                            idx, src_node, msg)
+                            idx, src_node_name, msg)
       elif (not isinstance(result.payload, (tuple, list)) or
             len(result.payload) != 2):
         self._lu.LogWarning("Could not snapshot disk/%s on node %s: invalid"
-                            " result '%s'", idx, src_node, result.payload)
+                            " result '%s'", idx, src_node_name, result.payload)
       else:
         disk_id = tuple(result.payload)
         disk_params = constants.DISK_LD_DEFAULTS[constants.DT_PLAIN].copy()
         new_dev = objects.Disk(dev_type=constants.DT_PLAIN, size=disk.size,
-                               logical_id=disk_id, physical_id=disk_id,
-                               iv_name=disk.iv_name,
+                               logical_id=disk_id, iv_name=disk.iv_name,
                                params=disk_params)
 
       self._snap_disks.append(new_dev)
@@ -1205,22 +1204,27 @@ class ExportInstanceHelper:
     disk = self._snap_disks[disk_index]
     if disk and not self._removed_snaps[disk_index]:
       src_node = self._instance.primary_node
+      src_node_name = self._lu.cfg.GetNodeName(src_node)
 
       self._feedback_fn("Removing snapshot of disk/%s on node %s" %
-                        (disk_index, src_node))
+                        (disk_index, src_node_name))
 
-      result = self._lu.rpc.call_blockdev_remove(src_node, disk)
+      result = self._lu.rpc.call_blockdev_remove(src_node,
+                                                 (disk, self._instance))
       if result.fail_msg:
         self._lu.LogWarning("Could not remove snapshot for disk/%d from node"
-                            " %s: %s", disk_index, src_node, result.fail_msg)
+                            " %s: %s", disk_index, src_node_name,
+                            result.fail_msg)
       else:
         self._removed_snaps[disk_index] = True
 
-  def LocalExport(self, dest_node):
+  def LocalExport(self, dest_node, compress):
     """Intra-cluster instance export.
 
     @type dest_node: L{objects.Node}
     @param dest_node: Destination node
+    @type compress: string
+    @param compress: one of L{constants.IEC_ALL}
 
     """
     instance = self._instance
@@ -1236,13 +1240,13 @@ class ExportInstanceHelper:
         continue
 
       path = utils.PathJoin(pathutils.EXPORT_DIR, "%s.new" % instance.name,
-                            dev.physical_id[1])
+                            dev.logical_id[1])
 
       finished_fn = compat.partial(self._TransferFinished, idx)
 
       # FIXME: pass debug option from opcode to backend
       dt = DiskTransfer("snapshot/%s" % idx,
-                        constants.IEIO_SCRIPT, (dev, idx),
+                        constants.IEIO_SCRIPT, ((dev, instance), idx),
                         constants.IEIO_FILE, (path, ),
                         finished_fn)
       transfers.append(dt)
@@ -1251,6 +1255,7 @@ class ExportInstanceHelper:
     dresults = TransferInstanceData(self._lu, self._feedback_fn,
                                     src_node_uuid, dest_node.uuid,
                                     dest_node.secondary_ip,
+                                    compress,
                                     instance, transfers)
 
     assert len(dresults) == len(instance.disks)
@@ -1266,7 +1271,7 @@ class ExportInstanceHelper:
 
     return (fin_resu, dresults)
 
-  def RemoteExport(self, disk_info, key_name, dest_ca_pem, timeouts):
+  def RemoteExport(self, disk_info, key_name, dest_ca_pem, compress, timeouts):
     """Inter-cluster instance export.
 
     @type disk_info: list
@@ -1275,6 +1280,8 @@ class ExportInstanceHelper:
     @param key_name: Name of X509 key to use
     @type dest_ca_pem: string
     @param dest_ca_pem: Destination X509 CA in PEM format
+    @type compress: string
+    @param compress: one of L{constants.IEC_ALL}
     @type timeouts: L{ImportExportTimeouts}
     @param timeouts: Timeouts for this import
 
@@ -1294,13 +1301,15 @@ class ExportInstanceHelper:
 
         opts = objects.ImportExportOptions(key_name=key_name,
                                            ca_pem=dest_ca_pem,
-                                           magic=magic, ipv6=ipv6)
+                                           magic=magic,
+                                           compress=compress,
+                                           ipv6=ipv6)
 
         self._feedback_fn("Sending disk %s to %s:%s" % (idx, host, port))
         finished_fn = compat.partial(self._TransferFinished, idx)
         ieloop.Add(DiskExport(self._lu, instance.primary_node,
                               opts, host, port, instance, "disk%d" % idx,
-                              constants.IEIO_SCRIPT, (dev, idx),
+                              constants.IEIO_SCRIPT, ((dev, instance), idx),
                               timeouts, cbs, private=(idx, finished_fn)))
 
       ieloop.Run()
@@ -1427,7 +1436,7 @@ class _RemoteImportCb(ImportExportCbBase):
 
 
 def RemoteImport(lu, feedback_fn, instance, pnode, source_x509_ca,
-                 cds, timeouts):
+                 cds, compress, timeouts):
   """Imports an instance from another cluster.
 
   @param lu: Logical unit instance
@@ -1440,6 +1449,8 @@ def RemoteImport(lu, feedback_fn, instance, pnode, source_x509_ca,
   @param source_x509_ca: Import source's X509 CA
   @type cds: string
   @param cds: Cluster domain secret
+  @type compress: string
+  @param compress: one of L{constants.IEC_ALL}
   @type timeouts: L{ImportExportTimeouts}
   @param timeouts: Timeouts for this import
 
@@ -1478,11 +1489,13 @@ def RemoteImport(lu, feedback_fn, instance, pnode, source_x509_ca,
         # Import daemon options
         opts = objects.ImportExportOptions(key_name=x509_key_name,
                                            ca_pem=source_ca_pem,
-                                           magic=magic, ipv6=ipv6)
+                                           magic=magic,
+                                           compress=compress,
+                                           ipv6=ipv6)
 
         ieloop.Add(DiskImport(lu, instance.primary_node, opts, instance,
                               "disk%d" % idx,
-                              constants.IEIO_SCRIPT, (dev, idx),
+                              constants.IEIO_SCRIPT, ((dev, instance), idx),
                               timeouts, cbs, private=(idx, )))
 
       ieloop.Run()
@@ -1635,6 +1648,7 @@ def ComputeDiskSize(disk_template, disks):
       sum(d[constants.IDISK_SIZE] + constants.DRBD_META_SIZE for d in disks),
     constants.DT_FILE: sum(d[constants.IDISK_SIZE] for d in disks),
     constants.DT_SHARED_FILE: sum(d[constants.IDISK_SIZE] for d in disks),
+    constants.DT_GLUSTER: sum(d[constants.IDISK_SIZE] for d in disks),
     constants.DT_BLOCK: 0,
     constants.DT_RBD: sum(d[constants.IDISK_SIZE] for d in disks),
     constants.DT_EXT: sum(d[constants.IDISK_SIZE] for d in disks),
