@@ -75,10 +75,8 @@ def FillDict(defaults_dict, custom_dict, skip_keys=None):
   ret_dict.update(custom_dict)
   if skip_keys:
     for k in skip_keys:
-      try:
+      if k in ret_dict:
         del ret_dict[k]
-      except KeyError:
-        pass
   return ret_dict
 
 
@@ -213,6 +211,16 @@ class ConfigObject(outils.ValidatedSlots):
   def Validate(self):
     """Validates the slots.
 
+    This method returns L{None} if the validation succeeds, or raises
+    an exception otherwise.
+
+    This method must be implemented by the child classes.
+
+    @rtype: NoneType
+    @return: L{None}, if the validation succeeds
+
+    @raise Exception: validation fails
+
     """
 
   def ToDict(self):
@@ -266,6 +274,10 @@ class ConfigObject(outils.ValidatedSlots):
     """Implement __repr__ for ConfigObjects."""
     return repr(self.ToDict())
 
+  def __eq__(self, other):
+    """Implement __eq__ for ConfigObjects."""
+    return isinstance(other, self.__class__) and self.ToDict() == other.ToDict()
+
   def UpgradeConfig(self):
     """Fill defaults for missing configuration values.
 
@@ -447,10 +459,7 @@ class ConfigData(ConfigObject):
       InstancePolicy.UpgradeDiskTemplates(
         nodegroup.ipolicy, self.cluster.enabled_disk_templates)
     if self.cluster.drbd_usermode_helper is None:
-      # To decide if we set an helper let's check if at least one instance has
-      # a DRBD disk. This does not cover all the possible scenarios but it
-      # gives a good approximation.
-      if self.HasAnyDiskOfType(constants.DT_DRBD8):
+      if self.cluster.IsDiskTemplateEnabled(constants.DT_DRBD8):
         self.cluster.drbd_usermode_helper = constants.DEFAULT_DRBD_HELPER
     if self.networks is None:
       self.networks = {}
@@ -484,7 +493,8 @@ class ConfigData(ConfigObject):
 
 class NIC(ConfigObject):
   """Config object representing a network card."""
-  __slots__ = ["name", "mac", "ip", "network", "nicparams", "netinfo"] + _UUID
+  __slots__ = ["name", "mac", "ip", "network",
+               "nicparams", "netinfo", "pci"] + _UUID
 
   @classmethod
   def CheckParameterSyntax(cls, nicparams):
@@ -507,9 +517,11 @@ class NIC(ConfigObject):
 
 class Disk(ConfigObject):
   """Config object representing a block device."""
-  __slots__ = (["name", "dev_type", "logical_id", "physical_id",
-                "children", "iv_name", "size", "mode", "params", "spindles"] +
-               _UUID)
+  __slots__ = (["name", "dev_type", "logical_id", "children", "iv_name",
+                "size", "mode", "params", "spindles", "pci"] + _UUID +
+               # dynamic_params is special. It depends on the node this instance
+               # is sent to, and should not be persisted.
+               ["dynamic_params"])
 
   def CreateOnSecondary(self):
     """Test if this device needs to be created on a secondary node."""
@@ -584,7 +596,8 @@ class Disk(ConfigObject):
     """
     if self.dev_type in [constants.DT_PLAIN, constants.DT_FILE,
                          constants.DT_BLOCK, constants.DT_RBD,
-                         constants.DT_EXT, constants.DT_SHARED_FILE]:
+                         constants.DT_EXT, constants.DT_SHARED_FILE,
+                         constants.DT_GLUSTER]:
       result = [node_uuid]
     elif self.dev_type in constants.DTS_DRBD:
       result = [self.logical_id[0], self.logical_id[1]]
@@ -661,7 +674,7 @@ class Disk(ConfigObject):
     """
     if self.dev_type in (constants.DT_PLAIN, constants.DT_FILE,
                          constants.DT_RBD, constants.DT_EXT,
-                         constants.DT_SHARED_FILE):
+                         constants.DT_SHARED_FILE, constants.DT_GLUSTER):
       self.size += amount
     elif self.dev_type == constants.DT_DRBD8:
       if self.children:
@@ -697,51 +710,53 @@ class Disk(ConfigObject):
         child.UnsetSize()
     self.size = 0
 
-  def SetPhysicalID(self, target_node_uuid, nodes_ip):
-    """Convert the logical ID to the physical ID.
+  def UpdateDynamicDiskParams(self, target_node_uuid, nodes_ip):
+    """Updates the dynamic disk params for the given node.
 
-    This is used only for drbd, which needs ip/port configuration.
-
-    The routine descends down and updates its children also, because
-    this helps when the only the top device is passed to the remote
-    node.
+    This is mainly used for drbd, which needs ip/port configuration.
 
     Arguments:
       - target_node_uuid: the node UUID we wish to configure for
       - nodes_ip: a mapping of node name to ip
 
-    The target_node must exist in in nodes_ip, and must be one of the
-    nodes in the logical ID for each of the DRBD devices encountered
-    in the disk tree.
+    The target_node must exist in nodes_ip, and should be one of the
+    nodes in the logical ID if this device is a DRBD device.
 
     """
     if self.children:
       for child in self.children:
-        child.SetPhysicalID(target_node_uuid, nodes_ip)
+        child.UpdateDynamicDiskParams(target_node_uuid, nodes_ip)
 
-    if self.logical_id is None and self.physical_id is not None:
-      return
-    if self.dev_type in constants.DTS_DRBD:
-      pnode_uuid, snode_uuid, port, pminor, sminor, secret = self.logical_id
+    dyn_disk_params = {}
+    if self.logical_id is not None and self.dev_type in constants.DTS_DRBD:
+      pnode_uuid, snode_uuid, _, pminor, sminor, _ = self.logical_id
       if target_node_uuid not in (pnode_uuid, snode_uuid):
-        raise errors.ConfigurationError("DRBD device not knowing node %s" %
-                                        target_node_uuid)
+        # disk object is being sent to neither the primary nor the secondary
+        # node. reset the dynamic parameters, the target node is not
+        # supposed to use them.
+        self.dynamic_params = dyn_disk_params
+        return
+
       pnode_ip = nodes_ip.get(pnode_uuid, None)
       snode_ip = nodes_ip.get(snode_uuid, None)
       if pnode_ip is None or snode_ip is None:
         raise errors.ConfigurationError("Can't find primary or secondary node"
                                         " for %s" % str(self))
-      p_data = (pnode_ip, port)
-      s_data = (snode_ip, port)
       if pnode_uuid == target_node_uuid:
-        self.physical_id = p_data + s_data + (pminor, secret)
+        dyn_disk_params[constants.DDP_LOCAL_IP] = pnode_ip
+        dyn_disk_params[constants.DDP_REMOTE_IP] = snode_ip
+        dyn_disk_params[constants.DDP_LOCAL_MINOR] = pminor
+        dyn_disk_params[constants.DDP_REMOTE_MINOR] = sminor
       else: # it must be secondary, we tested above
-        self.physical_id = s_data + p_data + (sminor, secret)
-    else:
-      self.physical_id = self.logical_id
-    return
+        dyn_disk_params[constants.DDP_LOCAL_IP] = snode_ip
+        dyn_disk_params[constants.DDP_REMOTE_IP] = pnode_ip
+        dyn_disk_params[constants.DDP_LOCAL_MINOR] = sminor
+        dyn_disk_params[constants.DDP_REMOTE_MINOR] = pminor
 
-  def ToDict(self):
+    self.dynamic_params = dyn_disk_params
+
+  # pylint: disable=W0221
+  def ToDict(self, include_dynamic_params=False):
     """Disk-specific conversion to standard python types.
 
     This replaces the children lists of objects with lists of
@@ -749,6 +764,8 @@ class Disk(ConfigObject):
 
     """
     bo = super(Disk, self).ToDict()
+    if not include_dynamic_params and "dynamic_params" in bo:
+      del bo["dynamic_params"]
 
     for attr in ("children",):
       alist = bo.get(attr, None)
@@ -766,8 +783,6 @@ class Disk(ConfigObject):
       obj.children = outils.ContainerFromDicts(obj.children, list, Disk)
     if obj.logical_id and isinstance(obj.logical_id, list):
       obj.logical_id = tuple(obj.logical_id)
-    if obj.physical_id and isinstance(obj.physical_id, list):
-      obj.physical_id = tuple(obj.physical_id)
     if obj.dev_type in constants.DTS_DRBD:
       # we need a tuple of length six here
       if len(obj.logical_id) < 6:
@@ -783,22 +798,16 @@ class Disk(ConfigObject):
     elif self.dev_type in constants.DTS_DRBD:
       node_a, node_b, port, minor_a, minor_b = self.logical_id[:5]
       val = "<DRBD8("
-      if self.physical_id is None:
-        phy = "unconfigured"
-      else:
-        phy = ("configured as %s:%s %s:%s" %
-               (self.physical_id[0], self.physical_id[1],
-                self.physical_id[2], self.physical_id[3]))
 
-      val += ("hosts=%s/%d-%s/%d, port=%s, %s, " %
-              (node_a, minor_a, node_b, minor_b, port, phy))
+      val += ("hosts=%s/%d-%s/%d, port=%s, " %
+              (node_a, minor_a, node_b, minor_b, port))
       if self.children and self.children.count(None) == 0:
         val += "backend=%s, metadev=%s" % (self.children[0], self.children[1])
       else:
         val += "no local storage"
     else:
-      val = ("<Disk(type=%s, logical_id=%s, physical_id=%s, children=%s" %
-             (self.dev_type, self.logical_id, self.physical_id, self.children))
+      val = ("<Disk(type=%s, logical_id=%s, children=%s" %
+             (self.dev_type, self.logical_id, self.children))
     if self.iv_name is None:
       val += ", not visible"
     else:
@@ -864,6 +873,7 @@ class Disk(ConfigObject):
 
     result = list()
     dt_params = disk_params[disk_template]
+
     if disk_template == constants.DT_DRBD8:
       result.append(FillDict(constants.DISK_LD_DEFAULTS[constants.DT_DRBD8], {
         constants.LDP_RESYNC_RATE: dt_params[constants.DRBD_RESYNC_RATE],
@@ -891,24 +901,12 @@ class Disk(ConfigObject):
         constants.LDP_STRIPES: dt_params[constants.DRBD_META_STRIPES],
         }))
 
-    elif disk_template in (constants.DT_FILE, constants.DT_SHARED_FILE):
-      result.append(constants.DISK_LD_DEFAULTS[disk_template])
-
-    elif disk_template == constants.DT_PLAIN:
-      result.append(FillDict(constants.DISK_LD_DEFAULTS[constants.DT_PLAIN], {
-        constants.LDP_STRIPES: dt_params[constants.LV_STRIPES],
-        }))
-
-    elif disk_template == constants.DT_BLOCK:
-      result.append(constants.DISK_LD_DEFAULTS[constants.DT_BLOCK])
-
-    elif disk_template == constants.DT_RBD:
-      result.append(FillDict(constants.DISK_LD_DEFAULTS[constants.DT_RBD], {
-        constants.LDP_POOL: dt_params[constants.RBD_POOL],
-        }))
-
-    elif disk_template == constants.DT_EXT:
-      result.append(constants.DISK_LD_DEFAULTS[constants.DT_EXT])
+    else:
+      defaults = constants.DISK_LD_DEFAULTS[disk_template]
+      values = {}
+      for field in defaults:
+        values[field] = dt_params[field]
+      result.append(FillDict(defaults, values))
 
     return result
 
@@ -1076,6 +1074,7 @@ class Instance(TaggableObject):
     "beparams",
     "osparams",
     "admin_state",
+    "admin_state_source",
     "nics",
     "disks",
     "disk_template",
@@ -1117,10 +1116,11 @@ class Instance(TaggableObject):
           _Helper(nodes, child)
 
     all_nodes = set()
-    all_nodes.add(self.primary_node)
     for device in self.disks:
       _Helper(all_nodes, device)
-    return tuple(all_nodes)
+    # ensure that the primary node is always the first
+    all_nodes.discard(self.primary_node)
+    return (self.primary_node, ) + tuple(all_nodes)
 
   all_nodes = property(_ComputeAllNodes, None, None,
                        "List of names of all the nodes of the instance")
@@ -1238,6 +1238,8 @@ class Instance(TaggableObject):
     """Fill defaults for missing configuration values.
 
     """
+    if self.admin_state_source is None:
+      self.admin_state_source = constants.ADMIN_SOURCE
     for nic in self.nics:
       nic.UpgradeConfig()
     for disk in self.disks:
@@ -1519,6 +1521,9 @@ class NodeGroup(TaggableObject):
     if self.networks is None:
       self.networks = {}
 
+    for network, netparams in self.networks.items():
+      self.networks[network] = FillDict(constants.NICC_DEFAULTS, netparams)
+
   def FillND(self, node):
     """Return filled out ndparams for L{objects.Node}
 
@@ -1564,6 +1569,7 @@ class Cluster(TaggableObject):
     "cluster_name",
     "file_storage_dir",
     "shared_file_storage_dir",
+    "gluster_storage_dir",
     "enabled_hypervisors",
     "hvparams",
     "ipolicy",
@@ -1579,6 +1585,7 @@ class Cluster(TaggableObject):
     "maintain_node_health",
     "uid_pool",
     "default_iallocator",
+    "default_iallocator_params",
     "hidden_os",
     "blacklisted_os",
     "primary_ip_family",
@@ -1586,6 +1593,9 @@ class Cluster(TaggableObject):
     "hv_state_static",
     "disk_state_static",
     "enabled_disk_templates",
+    "candidate_certs",
+    "max_running_jobs",
+    "enabled_user_shutdown",
     ] + _TIMESTAMPS + _UUID
 
   def UpgradeConfig(self):
@@ -1656,6 +1666,9 @@ class Cluster(TaggableObject):
     if self.default_iallocator is None:
       self.default_iallocator = ""
 
+    if self.default_iallocator_params is None:
+      self.default_iallocator_params = {}
+
     # reserved_lvs added before 2.2
     if self.reserved_lvs is None:
       self.reserved_lvs = []
@@ -1682,6 +1695,10 @@ class Cluster(TaggableObject):
     if self.shared_file_storage_dir is None:
       self.shared_file_storage_dir = ""
 
+    # gluster_storage_dir added in 2.11
+    if self.gluster_storage_dir is None:
+      self.gluster_storage_dir = ""
+
     if self.use_external_mip_script is None:
       self.use_external_mip_script = False
 
@@ -1705,6 +1722,15 @@ class Cluster(TaggableObject):
         raise errors.ConfigurationError(msg)
       self.ipolicy = FillIPolicy(constants.IPOLICY_DEFAULTS, self.ipolicy)
 
+    if self.candidate_certs is None:
+      self.candidate_certs = {}
+
+    if self.max_running_jobs is None:
+      self.max_running_jobs = constants.LUXID_MAXIMAL_RUNNING_JOBS_DEFAULT
+
+    if self.enabled_user_shutdown is None:
+      self.enabled_user_shutdown = False
+
   @property
   def primary_hypervisor(self):
     """The first hypervisor is the primary.
@@ -1896,6 +1922,16 @@ class Cluster(TaggableObject):
     """
     return self.SimpleFillND(nodegroup.FillND(node))
 
+  def FillNDGroup(self, nodegroup):
+    """Return filled out ndparams for just L{objects.NodeGroup}
+
+    @type nodegroup: L{objects.NodeGroup}
+    @param nodegroup: A Node object to fill
+    @return a copy of the node group's ndparams with defaults filled
+
+    """
+    return self.SimpleFillND(nodegroup.SimpleFillND({}))
+
   def SimpleFillND(self, ndparams):
     """Fill a given ndparams dict with defaults.
 
@@ -2143,7 +2179,6 @@ class InstanceConsole(ConfigObject):
     assert self.display or self.kind in [constants.CONS_MESSAGE,
                                          constants.CONS_SPICE,
                                          constants.CONS_SSH]
-    return True
 
 
 class Network(TaggableObject):
@@ -2202,7 +2237,8 @@ class Network(TaggableObject):
     return obj
 
 
-class SerializableConfigParser(ConfigParser.SafeConfigParser):
+# need to inherit object in order to use super()
+class SerializableConfigParser(ConfigParser.SafeConfigParser, object):
   """Simple wrapper over ConfigParse that allows serialization.
 
   This class is basically ConfigParser.SafeConfigParser with two
@@ -2224,6 +2260,23 @@ class SerializableConfigParser(ConfigParser.SafeConfigParser):
     cfp.readfp(buf)
     return cfp
 
+  def get(self, section, option, **kwargs):
+    value = None
+    try:
+      value = super(SerializableConfigParser, self).get(section, option,
+                                                        **kwargs)
+      if value.lower() == constants.VALUE_NONE:
+        value = None
+    except ConfigParser.NoOptionError:
+      r = re.compile(r"(disk|nic)\d+_name|nic\d+_(network|vlan)")
+      match = r.match(option)
+      if match:
+        pass
+      else:
+        raise
+
+    return value
+
 
 class LvmPvInfo(ConfigObject):
   """Information about an LVM physical volume (PV).
