@@ -65,6 +65,7 @@ from ganeti import ht
 from ganeti import runtime
 from ganeti import qlang
 from ganeti import jstore
+from ganeti.hypervisor import hv_base
 
 from ganeti.constants import (QFT_UNKNOWN, QFT_TEXT, QFT_BOOL, QFT_NUMBER,
                               QFT_UNIT, QFT_TIMESTAMP, QFT_OTHER,
@@ -201,7 +202,7 @@ def GetAllFields(fielddefs):
   return [fdef for (fdef, _, _, _) in fielddefs]
 
 
-class _FilterHints:
+class _FilterHints(object):
   """Class for filter analytics.
 
   When filters are used, the user of the L{Query} class usually doesn't know
@@ -387,7 +388,7 @@ def _MakeComparisonChecks(fn):
     ]
 
 
-class _FilterCompilerHelper:
+class _FilterCompilerHelper(object):
   """Converts a query filter to a callable usable for filtering.
 
   """
@@ -655,7 +656,7 @@ def _CompileFilter(fields, hints, qfilter):
   return _FilterCompilerHelper(fields)(hints, qfilter)
 
 
-class Query:
+class Query(object):
   def __init__(self, fieldlist, selected, qfilter=None, namefield=None):
     """Initializes this class.
 
@@ -1087,7 +1088,7 @@ def _GetItemTimestampFields(datatype):
     ]
 
 
-class NodeQueryData:
+class NodeQueryData(object):
   """Data container for node data queries.
 
   """
@@ -1406,7 +1407,7 @@ def _BuildNodeFields():
   return _PrepareFieldList(fields, [])
 
 
-class InstanceQueryData:
+class InstanceQueryData(object):
   """Data container for instance data queries.
 
   """
@@ -1523,6 +1524,43 @@ def _GetInstLiveData(name):
   return fn
 
 
+def _GetLiveInstStatus(ctx, instance, instance_state):
+  hvparams = ctx.cluster.FillHV(instance, skip_globals=True)
+
+  allow_userdown = \
+      ctx.cluster.enabled_user_shutdown and \
+      (instance.hypervisor != constants.HT_KVM or
+       hvparams[constants.HV_KVM_USER_SHUTDOWN])
+
+  if instance.uuid in ctx.wrongnode_inst:
+    return constants.INSTST_WRONGNODE
+  else:
+    if hv_base.HvInstanceState.IsShutdown(instance_state):
+      if instance.admin_state == constants.ADMINST_UP and allow_userdown:
+        return constants.INSTST_USERDOWN
+      elif instance.admin_state == constants.ADMINST_UP:
+        return constants.INSTST_ERRORDOWN
+      else:
+        return constants.INSTST_ADMINDOWN
+    else:
+      if instance.admin_state == constants.ADMINST_UP:
+        return constants.INSTST_RUNNING
+      else:
+        return constants.INSTST_ERRORUP
+
+
+def _GetDeadInstStatus(inst):
+  if inst.admin_state == constants.ADMINST_UP:
+    return constants.INSTST_ERRORDOWN
+  elif inst.admin_state == constants.ADMINST_DOWN:
+    if inst.admin_state_source == constants.USER_SOURCE:
+      return constants.INSTST_USERDOWN
+    else:
+      return constants.INSTST_ADMINDOWN
+  else:
+    return constants.INSTST_ADMINOFFLINE
+
+
 def _GetInstStatus(ctx, inst):
   """Get instance status.
 
@@ -1537,20 +1575,12 @@ def _GetInstStatus(ctx, inst):
   if inst.primary_node in ctx.bad_nodes:
     return constants.INSTST_NODEDOWN
 
-  if bool(ctx.live_data.get(inst.uuid)):
-    if inst.uuid in ctx.wrongnode_inst:
-      return constants.INSTST_WRONGNODE
-    elif inst.admin_state == constants.ADMINST_UP:
-      return constants.INSTST_RUNNING
-    else:
-      return constants.INSTST_ERRORUP
+  instance_live_data = ctx.live_data.get(inst.uuid)
 
-  if inst.admin_state == constants.ADMINST_UP:
-    return constants.INSTST_ERRORDOWN
-  elif inst.admin_state == constants.ADMINST_DOWN:
-    return constants.INSTST_ADMINDOWN
-
-  return constants.INSTST_ADMINOFFLINE
+  if bool(instance_live_data):
+    return _GetLiveInstStatus(ctx, inst, instance_live_data["state"])
+  else:
+    return _GetDeadInstStatus(inst)
 
 
 def _GetInstDisk(index, cb):
@@ -1722,6 +1752,24 @@ def _GetInstNicBridge(ctx, index, _):
     return _FS_UNAVAIL
 
 
+def _GetInstNicVLan(ctx, index, _):
+  """Get a NIC's VLAN.
+
+  @type ctx: L{InstanceQueryData}
+  @type index: int
+  @param index: NIC index
+
+  """
+  assert len(ctx.inst_nicparams) >= index
+
+  nicparams = ctx.inst_nicparams[index]
+
+  if nicparams[constants.NIC_MODE] == constants.NIC_MODE_OVS:
+    return nicparams[constants.NIC_VLAN]
+  else:
+    return _FS_UNAVAIL
+
+
 def _GetInstAllNicNetworkNames(ctx, inst):
   """Get all network names for an instance.
 
@@ -1743,6 +1791,30 @@ def _GetInstAllNicNetworkNames(ctx, inst):
   return result
 
 
+def _GetInstAllNicVlans(ctx, inst):
+  """Get all network VLANs for an instance.
+
+  @type ctx: L{InstanceQueryData}
+  @type inst: L{objects.Instance}
+  @param inst: Instance object
+
+  """
+  assert len(ctx.inst_nicparams) == len(inst.nics)
+
+  result = []
+
+  for nicp in ctx.inst_nicparams:
+    if nicp[constants.NIC_MODE] in \
+          [constants.NIC_MODE_BRIDGED, constants.NIC_MODE_OVS]:
+      result.append(nicp[constants.NIC_VLAN])
+    else:
+      result.append(None)
+
+  assert len(result) == len(inst.nics)
+
+  return result
+
+
 def _GetInstAllNicBridges(ctx, inst):
   """Get all network bridges for an instance.
 
@@ -1825,6 +1897,9 @@ def _GetInstanceNetworkFields():
                 "List containing each network interface's link"), IQ_CONFIG, 0,
      lambda ctx, inst: [nicp[constants.NIC_LINK]
                         for nicp in ctx.inst_nicparams]),
+    (_MakeField("nic.vlans", "NIC_VLANs", QFT_OTHER,
+                "List containing each network interface's VLAN"),
+     IQ_CONFIG, 0, _GetInstAllNicVlans),
     (_MakeField("nic.bridges", "NIC_bridges", QFT_OTHER,
                 "List containing each network interface's bridge"),
      IQ_CONFIG, 0, _GetInstAllNicBridges),
@@ -1861,6 +1936,9 @@ def _GetInstanceNetworkFields():
       (_MakeField("nic.bridge/%s" % i, "NicBridge/%s" % i, QFT_TEXT,
                   "Bridge of %s network interface" % numtext),
        IQ_CONFIG, 0, _GetInstNic(i, _GetInstNicBridge)),
+      (_MakeField("nic.vlan/%s" % i, "NicVLAN/%s" % i, QFT_TEXT,
+                  "VLAN of %s network interface" % numtext),
+       IQ_CONFIG, 0, _GetInstNic(i, _GetInstNicVLan)),
       (_MakeField("nic.network/%s" % i, "NicNetwork/%s" % i, QFT_TEXT,
                   "Network of %s network interface" % numtext),
        IQ_CONFIG, 0, _GetInstNic(i, _GetInstNicNetwork)),
@@ -2121,13 +2199,16 @@ def _BuildInstanceFields():
      lambda ctx, inst: map(compat.partial(_GetInstNodeGroup, ctx, None),
                            inst.secondary_nodes)),
     (_MakeField("admin_state", "InstanceState", QFT_TEXT,
-                "Desired state of instance"),
+                "Desired state of the instance"),
      IQ_CONFIG, 0, _GetItemAttr("admin_state")),
     (_MakeField("admin_up", "Autostart", QFT_BOOL,
-                "Desired state of instance"),
+                "Desired state of the instance"),
      IQ_CONFIG, 0, lambda ctx, inst: inst.admin_state == constants.ADMINST_UP),
+    (_MakeField("admin_state_source", "InstanceStateSource", QFT_TEXT,
+                "Who last changed the desired state of the instance"),
+     IQ_CONFIG, 0, _GetItemAttr("admin_state_source")),
     (_MakeField("disks_active", "DisksActive", QFT_BOOL,
-                "Desired state of instance disks"),
+                "Desired state of the instance disks"),
      IQ_CONFIG, 0, _GetItemAttr("disks_active")),
     (_MakeField("tags", "Tags", QFT_OTHER, "Tags"), IQ_CONFIG, 0,
      lambda ctx, inst: list(inst.GetTags())),
@@ -2157,7 +2238,8 @@ def _BuildInstanceFields():
   status_values = (constants.INSTST_RUNNING, constants.INSTST_ADMINDOWN,
                    constants.INSTST_WRONGNODE, constants.INSTST_ERRORUP,
                    constants.INSTST_ERRORDOWN, constants.INSTST_NODEDOWN,
-                   constants.INSTST_NODEOFFLINE, constants.INSTST_ADMINOFFLINE)
+                   constants.INSTST_NODEOFFLINE, constants.INSTST_ADMINOFFLINE,
+                   constants.INSTST_USERDOWN)
   status_doc = ("Instance status; \"%s\" if instance is set to be running"
                 " and actually is, \"%s\" if instance is stopped and"
                 " is not running, \"%s\" if instance running, but not on its"
@@ -2165,10 +2247,12 @@ def _BuildInstanceFields():
                 " stopped, but is actually running, \"%s\" if instance should"
                 " run, but doesn't, \"%s\" if instance's primary node is down,"
                 " \"%s\" if instance's primary node is marked offline,"
-                " \"%s\" if instance is offline and does not use dynamic"
+                " \"%s\" if instance is offline and does not use dynamic,"
+                " \"%s\" if the user shutdown the instance"
                 " resources" % status_values)
   fields.append((_MakeField("status", "Status", QFT_TEXT, status_doc),
                  IQ_LIVE, 0, _GetInstStatus))
+
   assert set(status_values) == constants.INSTST_ALL, \
          "Status documentation mismatch"
 
@@ -2189,7 +2273,7 @@ def _BuildInstanceFields():
   return _PrepareFieldList(fields, aliases)
 
 
-class LockQueryData:
+class LockQueryData(object):
   """Data container for lock data queries.
 
   """
@@ -2251,7 +2335,7 @@ def _BuildLockFields():
     ], [])
 
 
-class GroupQueryData:
+class GroupQueryData(object):
   """Data container for node group data queries.
 
   """
@@ -2611,7 +2695,7 @@ _CLUSTER_SIMPLE_FIELDS = {
   }
 
 
-class ClusterQueryData:
+class ClusterQueryData(object):
   def __init__(self, cluster, nodes, drain_flag, watcher_pause):
     """Initializes this class.
 
@@ -2683,7 +2767,7 @@ def _BuildClusterFields():
     ("name", "cluster_name")])
 
 
-class NetworkQueryData:
+class NetworkQueryData(object):
   """Data container for network data queries.
 
   """
