@@ -30,6 +30,7 @@
 import logging
 
 from ganeti import luxi
+import ganeti.rpc.errors as rpcerr
 from ganeti import rapi
 from ganeti import http
 from ganeti import errors
@@ -56,7 +57,7 @@ def _BuildOpcodeAttributes():
 
   """
   return [(method, "%s_OPCODE" % method, "%s_RENAME" % method,
-           "Get%sOpInput" % method.capitalize())
+           "%s_ALIASES" % method, "Get%sOpInput" % method.capitalize())
           for method in _SUPPORTED_METHODS]
 
 
@@ -359,16 +360,16 @@ class ResourceBase(object):
         instead of the masterd socket
 
     """
-    if query and constants.ENABLE_SPLIT_QUERY:
+    if query:
       address = pathutils.QUERY_SOCKET
     else:
       address = None
     # Could be a function, pylint: disable=R0201
     try:
       return self._client_cls(address=address)
-    except luxi.NoMasterError, err:
+    except rpcerr.NoMasterError, err:
       raise http.HttpBadGateway("Can't connect to master daemon: %s" % err)
-    except luxi.PermissionError:
+    except rpcerr.PermissionError:
       raise http.HttpInternalServerError("Internal error: no permission to"
                                          " connect to the master daemon")
 
@@ -391,12 +392,12 @@ class ResourceBase(object):
       raise http.HttpServiceUnavailable("Job queue is full, needs archiving")
     except errors.JobQueueDrainError:
       raise http.HttpServiceUnavailable("Job queue is drained, cannot submit")
-    except luxi.NoMasterError, err:
+    except rpcerr.NoMasterError, err:
       raise http.HttpBadGateway("Master seems to be unreachable: %s" % err)
-    except luxi.PermissionError:
+    except rpcerr.PermissionError:
       raise http.HttpInternalServerError("Internal error: no permission to"
                                          " connect to the master daemon")
-    except luxi.TimeoutError, err:
+    except rpcerr.TimeoutError, err:
       raise http.HttpGatewayTimeout("Timeout while talking to the master"
                                     " daemon: %s" % err)
 
@@ -406,7 +407,7 @@ def GetResourceOpcodes(cls):
 
   """
   return frozenset(filter(None, (getattr(cls, op_attr, None)
-                                 for (_, op_attr, _, _) in OPCODE_ATTRS)))
+                                 for (_, op_attr, _, _, _) in OPCODE_ATTRS)))
 
 
 def GetHandlerAccess(handler, method):
@@ -420,6 +421,22 @@ def GetHandlerAccess(handler, method):
   return getattr(handler, "%s_ACCESS" % method, None)
 
 
+def GetHandler(get_fn, aliases):
+  result = get_fn()
+  if not isinstance(result, dict) or aliases is None:
+    return result
+
+  for (param, alias) in aliases.items():
+    if param in result:
+      if alias in result:
+        raise http.HttpBadRequest("Parameter '%s' has an alias of '%s', but"
+                                  " both values are present in response" %
+                                  (param, alias))
+      result[alias] = result[param]
+
+  return result
+
+
 class _MetaOpcodeResource(type):
   """Meta class for RAPI resources.
 
@@ -431,13 +448,23 @@ class _MetaOpcodeResource(type):
     # Access to private attributes of a client class, pylint: disable=W0212
     obj = type.__call__(mcs, *args, **kwargs)
 
-    for (method, op_attr, rename_attr, fn_attr) in OPCODE_ATTRS:
+    for (method, op_attr, rename_attr, aliases_attr, fn_attr) in OPCODE_ATTRS:
       if hasattr(obj, method):
-        # If the method handler is already defined, "*_RENAME" or "Get*OpInput"
-        # shouldn't be (they're only used by the automatically generated
-        # handler)
+        # If the method handler is already defined, "*_RENAME" or
+        # "Get*OpInput" shouldn't be (they're only used by the automatically
+        # generated handler)
         assert not hasattr(obj, rename_attr)
         assert not hasattr(obj, fn_attr)
+
+        # The aliases are allowed only on GET calls
+        assert not hasattr(obj, aliases_attr) or method == http.HTTP_GET
+
+        # GET methods can add aliases of values they return under a different
+        # name
+        if method == http.HTTP_GET and hasattr(obj, aliases_attr):
+          setattr(obj, method,
+                  compat.partial(GetHandler, getattr(obj, method),
+                                 getattr(obj, aliases_attr)))
       else:
         # Try to generate handler method on handler instance
         try:
@@ -462,11 +489,19 @@ class OpcodeResource(ResourceBase):
   method to do their own opcode input processing (e.g. for static values). The
   C{$METHOD$_RENAME} variable defines which values are renamed (see
   L{baserlib.FillOpcode}).
+  Still default behavior cannot be totally overriden. There are opcode params
+  that are available to all opcodes, e.g. "depends". In case those params
+  (currently only "depends") are found in the original request's body, they are
+  added to the dictionary of parsed parameters and eventually passed to the
+  opcode. If the parsed body is not represented as a dictionary object, the
+  values are not added.
 
   @cvar GET_OPCODE: Set this to a class derived from L{opcodes.OpCode} to
     automatically generate a GET handler submitting the opcode
   @cvar GET_RENAME: Set this to rename parameters in the GET handler (see
     L{baserlib.FillOpcode})
+  @cvar GET_ALIASES: Set this to duplicate return values in GET results (see
+    L{baserlib.GetHandler})
   @ivar GetGetOpInput: Define this to override the default method for
     getting opcode parameters (see L{baserlib.OpcodeResource._GetDefaultData})
 
@@ -528,8 +563,18 @@ class OpcodeResource(ResourceBase):
       }
     return common_static
 
+  def _GetDepends(self):
+    ret = {}
+    if isinstance(self.request_body, dict):
+      depends = self.getBodyParameter("depends", None)
+      if depends:
+        ret.update({"depends": depends})
+    return ret
+
   def _GenericHandler(self, opcode, rename, fn):
     (body, specific_static) = fn()
+    if isinstance(body, dict):
+      body.update(self._GetDepends())
     static = self._GetCommonStatic()
     if specific_static:
       static.update(specific_static)
