@@ -220,6 +220,10 @@ class R_2_info(baserlib.OpcodeResource):
 
   """
   GET_OPCODE = opcodes.OpClusterQuery
+  GET_ALIASES = {
+    "volume_group_name": "vg_name",
+    "drbd_usermode_helper": "drbd_helper",
+    }
 
   def GET(self):
     """Returns cluster information.
@@ -392,7 +396,6 @@ class R_2_nodes(baserlib.OpcodeResource):
   """/2/nodes resource.
 
   """
-  GET_OPCODE = opcodes.OpNodeQuery
 
   def GET(self):
     """Returns a list of all nodes.
@@ -414,7 +417,9 @@ class R_2_nodes_name(baserlib.OpcodeResource):
   """/2/nodes/[node_name] resource.
 
   """
-  GET_OPCODE = opcodes.OpNodeQuery
+  GET_ALIASES = {
+    "sip": "secondary_ip",
+    }
 
   def GET(self):
     """Send information about a node.
@@ -657,7 +662,6 @@ class R_2_networks(baserlib.OpcodeResource):
   """/2/networks resource.
 
   """
-  GET_OPCODE = opcodes.OpNetworkQuery
   POST_OPCODE = opcodes.OpNetworkAdd
   POST_RENAME = {
     "name": "network_name",
@@ -773,7 +777,6 @@ class R_2_groups(baserlib.OpcodeResource):
   """/2/groups resource.
 
   """
-  GET_OPCODE = opcodes.OpGroupQuery
   POST_OPCODE = opcodes.OpGroupAdd
   POST_RENAME = {
     "name": "group_name",
@@ -840,6 +843,11 @@ class R_2_groups_name_modify(baserlib.OpcodeResource):
 
   """
   PUT_OPCODE = opcodes.OpGroupSetParams
+  PUT_RENAME = {
+    "custom_ndparams": "ndparams",
+    "custom_ipolicy": "ipolicy",
+    "custom_diskparams": "diskparams",
+    }
 
   def GetPutOpInput(self):
     """Changes some parameters of node group.
@@ -911,7 +919,6 @@ class R_2_instances(baserlib.OpcodeResource):
   """/2/instances resource.
 
   """
-  GET_OPCODE = opcodes.OpInstanceQuery
   POST_OPCODE = opcodes.OpInstanceCreate
   POST_RENAME = {
     "os": "os_type",
@@ -922,7 +929,7 @@ class R_2_instances(baserlib.OpcodeResource):
     """Returns a list of all available instances.
 
     """
-    client = self.GetClient()
+    client = self.GetClient(query=True)
 
     use_locking = self.useLocking()
     if self.useBulk():
@@ -980,12 +987,19 @@ class R_2_instances_multi_alloc(baserlib.OpcodeResource):
       raise http.HttpBadRequest("Request is missing required 'instances' field"
                                 " in body")
 
-    op_id = {
-      "OP_ID": self.POST_OPCODE.OP_ID, # pylint: disable=E1101
-      }
+    # Unlike most other RAPI calls, this one is composed of individual opcodes,
+    # and we have to do the filling ourselves
+    OPCODE_RENAME = {
+      "os": "os_type",
+      "name": "instance_name",
+    }
+
     body = objects.FillDict(self.request_body, {
-      "instances": [objects.FillDict(inst, op_id)
-                    for inst in self.request_body["instances"]],
+      "instances": [
+        baserlib.FillOpcode(opcodes.OpInstanceCreate, inst, {},
+                            rename=OPCODE_RENAME)
+        for inst in self.request_body["instances"]
+        ],
       })
 
     return (body, {
@@ -997,14 +1011,13 @@ class R_2_instances_name(baserlib.OpcodeResource):
   """/2/instances/[instance_name] resource.
 
   """
-  GET_OPCODE = opcodes.OpInstanceQuery
   DELETE_OPCODE = opcodes.OpInstanceRemove
 
   def GET(self):
     """Send information about an instance.
 
     """
-    client = self.GetClient()
+    client = self.GetClient(query=True)
     instance_name = self.items[0]
 
     result = baserlib.HandleItemQueryErrors(client.QueryInstances,
@@ -1019,7 +1032,7 @@ class R_2_instances_name(baserlib.OpcodeResource):
 
     """
     assert len(self.items) == 1
-    return ({}, {
+    return (self.request_body, {
       "instance_name": self.items[0],
       "ignore_failures": False,
       "dry_run": self.dryRun(),
@@ -1058,7 +1071,7 @@ class R_2_instances_name_reboot(baserlib.OpcodeResource):
     ignore_secondaries=[False|True] parameters.
 
     """
-    return ({}, {
+    return (self.request_body, {
       "instance_name": self.items[0],
       "reboot_type":
         self.queryargs.get("type", [constants.INSTANCE_REBOOT_HARD])[0],
@@ -1346,6 +1359,10 @@ class R_2_instances_name_modify(baserlib.OpcodeResource):
 
   """
   PUT_OPCODE = opcodes.OpInstanceSetParams
+  PUT_RENAME = {
+    "custom_beparams": "beparams",
+    "custom_hvparams": "hvparams",
+    }
 
   def GetPutOpInput(self):
     """Changes parameters of an instance.
@@ -1389,11 +1406,13 @@ class R_2_instances_name_console(baserlib.ResourceBase):
              L{objects.InstanceConsole}
 
     """
-    client = self.GetClient()
+    instance_name = self.items[0]
+    client = self.GetClient(query=True)
 
-    ((console, ), ) = client.QueryInstances([self.items[0]], ["console"], False)
+    ((console, oper_state), ) = \
+        client.QueryInstances([instance_name], ["console", "oper_state"], False)
 
-    if console is None:
+    if not oper_state:
       raise http.HttpServiceUnavailable("Instance console unavailable")
 
     assert isinstance(console, dict)
@@ -1437,7 +1456,8 @@ class R_2_query(baserlib.ResourceBase):
   PUT_OPCODE = opcodes.OpQuery
 
   def _Query(self, fields, qfilter):
-    return self.GetClient().Query(self.items[0], fields, qfilter).ToDict()
+    client = self.GetClient()
+    return client.Query(self.items[0], fields, qfilter).ToDict()
 
   def GET(self):
     """Returns resource information.
