@@ -62,7 +62,7 @@ ACCEPT_OFFLINE_NODE = object()
  ED_OBJECT_DICT_LIST,
  ED_INST_DICT,
  ED_INST_DICT_HVP_BEP_DP,
- ED_NODE_TO_DISK_DICT,
+ ED_NODE_TO_DISK_DICT_DP,
  ED_INST_DICT_OSP_DP,
  ED_IMPEXP_IO,
  ED_FILE_DETAILS,
@@ -72,7 +72,8 @@ ACCEPT_OFFLINE_NODE = object()
  ED_DISKS_DICT_DP,
  ED_MULTI_DISKS_DICT_DP,
  ED_SINGLE_DISK_DICT_DP,
- ED_NIC_DICT) = range(1, 16)
+ ED_NIC_DICT,
+ ED_DEVICE_DICT) = range(1, 17)
 
 
 def _Prepare(calls):
@@ -83,7 +84,7 @@ def _Prepare(calls):
 
 
 def _MigrationStatusPostProc(result):
-  """Post-processor for L{rpc.RpcRunner.call_instance_get_migration_status}.
+  """Post-processor for L{rpc.node.RpcRunner.call_instance_get_migration_status}
 
   """
   if not result.fail_msg and result.payload is not None:
@@ -92,7 +93,7 @@ def _MigrationStatusPostProc(result):
 
 
 def _BlockdevFindPostProc(result):
-  """Post-processor for L{rpc.RpcRunner.call_blockdev_find}.
+  """Post-processor for L{rpc.node.RpcRunner.call_blockdev_find}.
 
   """
   if not result.fail_msg and result.payload is not None:
@@ -101,7 +102,7 @@ def _BlockdevFindPostProc(result):
 
 
 def _BlockdevGetMirrorStatusPostProc(result):
-  """Post-processor for L{rpc.RpcRunner.call_blockdev_getmirrorstatus}.
+  """Post-processor for call_blockdev_getmirrorstatus.
 
   """
   if not result.fail_msg:
@@ -121,7 +122,7 @@ def _BlockdevGetMirrorStatusMultiPreProc(node, args):
 
 
 def _BlockdevGetMirrorStatusMultiPostProc(result):
-  """Post-processor for L{rpc.RpcRunner.call_blockdev_getmirrorstatus_multi}.
+  """Post-processor for call_blockdev_getmirrorstatus_multi.
 
   """
   if not result.fail_msg:
@@ -143,13 +144,8 @@ def _NodeInfoPreProc(node, args):
     return args
 
 
-def _DrbdCallsPreProc(node, args):
-  """Add the target node UUID as additional field for DRBD related calls."""
-  return args + [node]
-
-
 def _OsGetPostProc(result):
-  """Post-processor for L{rpc.RpcRunner.call_os_get}.
+  """Post-processor for L{rpc.node.RpcRunner.call_os_get}.
 
   """
   if not result.fail_msg and isinstance(result.payload, dict):
@@ -296,6 +292,17 @@ _INSTANCE_CALLS = [
     ("reinstall", None, None),
     ("debug", None, None),
     ], None, None, "Starts an instance"),
+  ("hotplug_device", SINGLE, None, constants.RPC_TMO_NORMAL, [
+    ("instance", ED_INST_DICT, "Instance object"),
+    ("action", None, "Hotplug Action"),
+    ("dev_type", None, "Device type"),
+    ("device", ED_DEVICE_DICT, "Device dict"),
+    ("extra", None, "Extra info for device (dev_path for disk)"),
+    ("seq", None, "Device seq"),
+    ], None, None, "Hoplug a device to a running instance"),
+  ("hotplug_supported", SINGLE, None, constants.RPC_TMO_NORMAL, [
+    ("instance", ED_INST_DICT, "Instance object"),
+    ], None, None, "Check if hotplug is supported"),
   ]
 
 _IMPEXP_CALLS = [
@@ -351,7 +358,7 @@ _BLOCKDEV_CALLS = [
     ], None, None,
    "Gets the sizes of requested block devices present on a node"),
   ("blockdev_create", SINGLE, None, constants.RPC_TMO_NORMAL, [
-    ("bdev", ED_OBJECT_DICT, None),
+    ("bdev", ED_SINGLE_DISK_DICT_DP, None),
     ("size", None, None),
     ("owner", None, None),
     ("on_primary", None, None),
@@ -365,7 +372,7 @@ _BLOCKDEV_CALLS = [
     ], None, None,
     "Request wipe at given offset with given size of a block device"),
   ("blockdev_remove", SINGLE, None, constants.RPC_TMO_NORMAL, [
-    ("bdev", ED_OBJECT_DICT, None),
+    ("bdev", ED_SINGLE_DISK_DICT_DP, None),
     ], None, None, "Request removal of a given block device"),
   ("blockdev_pause_resume_sync", SINGLE, None, constants.RPC_TMO_NORMAL, [
     ("disks", ED_DISKS_DICT_DP, None),
@@ -373,7 +380,7 @@ _BLOCKDEV_CALLS = [
     ], None, None, "Request a pause/resume of given block device"),
   ("blockdev_assemble", SINGLE, None, constants.RPC_TMO_NORMAL, [
     ("disk", ED_SINGLE_DISK_DICT_DP, None),
-    ("owner", None, None),
+    ("instance", ED_INST_DICT, None),
     ("on_primary", None, None),
     ("idx", None, None),
     ], None, None, "Request assembling of a given block device"),
@@ -382,41 +389,37 @@ _BLOCKDEV_CALLS = [
     ], None, None, "Request shutdown of a given block device"),
   ("blockdev_addchildren", SINGLE, None, constants.RPC_TMO_NORMAL, [
     ("bdev", ED_SINGLE_DISK_DICT_DP, None),
-    ("ndevs", ED_OBJECT_DICT_LIST, None),
+    ("ndevs", ED_DISKS_DICT_DP, None),
     ], None, None,
    "Request adding a list of children to a (mirroring) device"),
   ("blockdev_removechildren", SINGLE, None, constants.RPC_TMO_NORMAL, [
-    ("bdev", ED_OBJECT_DICT, None),
-    ("ndevs", ED_OBJECT_DICT_LIST, None),
+    ("bdev", ED_SINGLE_DISK_DICT_DP, None),
+    ("ndevs", ED_DISKS_DICT_DP, None),
     ], None, None,
    "Request removing a list of children from a (mirroring) device"),
   ("blockdev_close", SINGLE, None, constants.RPC_TMO_NORMAL, [
     ("instance_name", None, None),
-    ("disks", ED_OBJECT_DICT_LIST, None),
+    ("disks", ED_DISKS_DICT_DP, None),
     ], None, None, "Closes the given block devices"),
   ("blockdev_getdimensions", SINGLE, None, constants.RPC_TMO_NORMAL, [
-    ("disks", ED_OBJECT_DICT_LIST, None),
+    ("disks", ED_MULTI_DISKS_DICT_DP, None),
     ], None, None, "Returns size and spindles of the given disks"),
   ("drbd_disconnect_net", MULTI, None, constants.RPC_TMO_NORMAL, [
-    ("nodes_ip", None, None),
-    ("disks", ED_OBJECT_DICT_LIST, None),
-    ], _DrbdCallsPreProc, None,
+    ("disks", ED_DISKS_DICT_DP, None),
+    ], None, None,
    "Disconnects the network of the given drbd devices"),
   ("drbd_attach_net", MULTI, None, constants.RPC_TMO_NORMAL, [
-    ("nodes_ip", None, None),
     ("disks", ED_DISKS_DICT_DP, None),
     ("instance_name", None, None),
     ("multimaster", None, None),
-    ], _DrbdCallsPreProc, None, "Connects the given DRBD devices"),
+    ], None, None, "Connects the given DRBD devices"),
   ("drbd_wait_sync", MULTI, None, constants.RPC_TMO_SLOW, [
-    ("nodes_ip", None, None),
     ("disks", ED_DISKS_DICT_DP, None),
-    ], _DrbdCallsPreProc, None,
+    ], None, None,
    "Waits for the synchronization of drbd devices is complete"),
   ("drbd_needs_activation", SINGLE, None, constants.RPC_TMO_NORMAL, [
-    ("nodes_ip", None, None),
     ("disks", ED_MULTI_DISKS_DICT_DP, None),
-    ], _DrbdCallsPreProc, None,
+    ], None, None,
    "Returns the drbd disks which need activation"),
   ("blockdev_grow", SINGLE, None, constants.RPC_TMO_NORMAL, [
     ("cf_bdev", ED_SINGLE_DISK_DICT_DP, None),
@@ -426,12 +429,6 @@ _BLOCKDEV_CALLS = [
     ("es_flag", None, None),
     ], None, None, "Request growing of the given block device by a"
    " given amount"),
-  ("blockdev_export", SINGLE, None, constants.RPC_TMO_1DAY, [
-    ("cf_bdev", ED_SINGLE_DISK_DICT_DP, None),
-    ("dest_node_ip", None, None),
-    ("dest_path", None, None),
-    ("cluster_name", None, None),
-    ], None, None, "Export a given disk to another node"),
   ("blockdev_snapshot", SINGLE, None, constants.RPC_TMO_NORMAL, [
     ("cf_bdev", ED_SINGLE_DISK_DICT_DP, None),
     ], None, None, "Export a given disk to another node"),
@@ -439,7 +436,7 @@ _BLOCKDEV_CALLS = [
     ("devlist", ED_BLOCKDEV_RENAME, None),
     ], None, None, "Request rename of the given block devices"),
   ("blockdev_find", SINGLE, None, constants.RPC_TMO_NORMAL, [
-    ("disk", ED_OBJECT_DICT, None),
+    ("disk", ED_SINGLE_DISK_DICT_DP, None),
     ], None, _BlockdevFindPostProc,
     "Request identification of a given block device"),
   ("blockdev_getmirrorstatus", SINGLE, None, constants.RPC_TMO_NORMAL, [
@@ -447,12 +444,12 @@ _BLOCKDEV_CALLS = [
     ], None, _BlockdevGetMirrorStatusPostProc,
     "Request status of a (mirroring) device"),
   ("blockdev_getmirrorstatus_multi", MULTI, None, constants.RPC_TMO_NORMAL, [
-    ("node_disks", ED_NODE_TO_DISK_DICT, None),
+    ("node_disks", ED_NODE_TO_DISK_DICT_DP, None),
     ], _BlockdevGetMirrorStatusMultiPreProc,
    _BlockdevGetMirrorStatusMultiPostProc,
     "Request status of (mirroring) devices from multiple nodes"),
   ("blockdev_setinfo", SINGLE, None, constants.RPC_TMO_NORMAL, [
-    ("disk", ED_OBJECT_DICT, None),
+    ("disk", ED_SINGLE_DISK_DICT_DP, None),
     ("info", None, None),
     ], None, None, "Sets metadata information on a given block device"),
   ]
@@ -492,6 +489,9 @@ _NODE_CALLS = [
     ("checkdict", None, "What to verify"),
     ("cluster_name", None, "Cluster name"),
     ("all_hvparams", None, "Dictionary mapping hypervisor names to hvparams"),
+    ("node_groups", None, "node names mapped to their group uuids"),
+    ("groups_cfg", None,
+      "a dictionary mapping group uuids to their configuration"),
     ], None, None, "Request verification of given parameters"),
   ("node_volumes", MULTI, None, constants.RPC_TMO_FAST, [], None, None,
    "Gets all volumes on node(s)"),
@@ -501,6 +501,18 @@ _NODE_CALLS = [
     ("hypervisor", None, "Hypervisor type"),
     ("hvparams", None, "Hypervisor parameters"),
     ], None, None, "Tries to powercycle a node"),
+  ("node_configure_ovs", SINGLE, None, constants.RPC_TMO_NORMAL, [
+    ("ovs_name", None, "Name of the OpenvSwitch to create"),
+    ("ovs_link", None, "Link of the OpenvSwitch to the outside"),
+    ], None, None, "This will create and setup the OpenvSwitch"),
+  ("node_crypto_tokens", SINGLE, None, constants.RPC_TMO_NORMAL, [
+    ("token_request", None,
+     "List of tuples of requested crypto token types, actions"),
+    ], None, None, "Handle crypto tokens of the node."),
+  ("node_ensure_daemon", MULTI, None, constants.RPC_TMO_URGENT, [
+    ("daemon", None, "Daemon name"),
+    ("run", None, "Whether the daemon should be running or stopped"),
+    ], None, None, "Ensure daemon is running on the node."),
   ]
 
 _MISC_CALLS = [
@@ -538,6 +550,7 @@ _MISC_CALLS = [
   ("iallocator_runner", SINGLE, None, constants.RPC_TMO_NORMAL, [
     ("name", None, "Iallocator name"),
     ("idata", None, "JSON-encoded input string"),
+    ("default_iallocator_params", None, "Additional iallocator parameters"),
     ], None, None, "Call an iallocator on a remote node"),
   ("test_delay", MULTI, None, _TestDelayTimeout, [
     ("duration", None, None),
@@ -600,8 +613,8 @@ CALLS = {
       ("modify_ssh_setup", None, None),
       ], None, None,
      "Requests a node to clean the cluster information it has"),
-    ("master_info", MULTI, None, constants.RPC_TMO_URGENT, [], None, None,
-     "Query master info"),
+    ("master_node_name", MULTI, None, constants.RPC_TMO_URGENT, [], None, None,
+     "Returns the master node name"),
     ]),
   "RpcClientDnsOnly": _Prepare([
     ("version", MULTI, ACCEPT_OFFLINE_NODE, constants.RPC_TMO_URGENT, [], None,
@@ -610,6 +623,9 @@ CALLS = {
       ("checkdict", None, "What to verify"),
       ("cluster_name", None, "Cluster name"),
       ("hvparams", None, "Dictionary mapping hypervisor names to hvparams"),
+      ("node_groups", None, "node names mapped to their group uuids"),
+      ("groups_cfg", None,
+       "a dictionary mapping group uuids to their configuration"),
       ], None, None, "Request verification of given parameters"),
     ]),
   "RpcClientConfig": _Prepare([
