@@ -26,10 +26,15 @@
 import grp
 import pwd
 import threading
+import os
 import platform
 
 from ganeti import constants
 from ganeti import errors
+from ganeti import luxi
+from ganeti.rpc.errors import NoMasterError
+from ganeti import pathutils
+from ganeti import ssconf
 from ganeti import utils
 
 
@@ -68,7 +73,7 @@ def GetGid(group, _getgrnam):
     raise errors.ConfigurationError("Group '%s' not found (%s)" % (group, err))
 
 
-class GetentResolver:
+class GetentResolver(object):
   """Resolves Ganeti uids and gids by name.
 
   @ivar masterd_uid: The resolved uid of the masterd user
@@ -235,3 +240,48 @@ def GetArchInfo():
                                  " initialized")
 
   return _arch
+
+
+def GetClient(query=True):
+  """Connects to the a luxi socket and returns a client.
+
+  @type query: boolean
+  @param query: this signifies that the client will only be
+      used for queries; if the build-time parameter
+      enable-split-queries is enabled, then the client will be
+      connected to the query socket instead of the masterd socket
+
+  """
+  override_socket = os.getenv(constants.LUXI_OVERRIDE, "")
+  if override_socket:
+    if override_socket == constants.LUXI_OVERRIDE_MASTER:
+      address = pathutils.MASTER_SOCKET
+    elif override_socket == constants.LUXI_OVERRIDE_QUERY:
+      address = pathutils.QUERY_SOCKET
+    else:
+      address = override_socket
+  elif query:
+    address = pathutils.QUERY_SOCKET
+  else:
+    address = None
+  # TODO: Cache object?
+  try:
+    client = luxi.Client(address=address)
+  except NoMasterError:
+    ss = ssconf.SimpleStore()
+
+    # Try to read ssconf file
+    try:
+      ss.GetMasterNode()
+    except errors.ConfigurationError:
+      raise errors.OpPrereqError("Cluster not initialized or this machine is"
+                                 " not part of a cluster",
+                                 errors.ECODE_INVAL)
+
+    master, myself = ssconf.GetMasterAndMyself(ss=ss)
+    if master != myself:
+      raise errors.OpPrereqError("This is not the master node, please connect"
+                                 " to node '%s' and rerun the command" %
+                                 master, errors.ECODE_INVAL)
+    raise
+  return client
