@@ -48,11 +48,13 @@ from ganeti import opcodes
 from ganeti import jqueue
 from ganeti import locking
 from ganeti import luxi
+import ganeti.rpc.errors as rpcerr
 from ganeti import utils
 from ganeti import errors
 from ganeti import ssconf
 from ganeti import workerpool
-from ganeti import rpc
+import ganeti.rpc.node as rpc
+import ganeti.rpc.client as rpccl
 from ganeti import bootstrap
 from ganeti import netutils
 from ganeti import objects
@@ -61,6 +63,8 @@ from ganeti import runtime
 from ganeti import pathutils
 from ganeti import ht
 
+from ganeti.utils import version
+
 
 CLIENT_REQUEST_WORKERS = 16
 
@@ -90,8 +94,8 @@ class ClientRequestWorker(workerpool.BaseWorker):
     client_ops = ClientOps(server)
 
     try:
-      (method, args, version) = luxi.ParseRequest(message)
-    except luxi.ProtocolError, err:
+      (method, args, ver) = rpccl.ParseRequest(message)
+    except rpcerr.ProtocolError, err:
       logging.error("Protocol Error: %s", err)
       client.close_log()
       return
@@ -99,9 +103,9 @@ class ClientRequestWorker(workerpool.BaseWorker):
     success = False
     try:
       # Verify client's version if there was one in the request
-      if version is not None and version != constants.LUXI_VERSION:
+      if ver is not None and ver != constants.LUXI_VERSION:
         raise errors.LuxiError("LUXI version mismatch, server %s, request %s" %
-                               (constants.LUXI_VERSION, version))
+                               (constants.LUXI_VERSION, ver))
 
       result = client_ops.handle_request(method, args)
       success = True
@@ -115,7 +119,7 @@ class ClientRequestWorker(workerpool.BaseWorker):
       result = "Caught exception: %s" % str(err[1])
 
     try:
-      reply = luxi.FormatResponse(success, result)
+      reply = rpccl.FormatResponse(success, result)
       client.send_message(reply)
       # awake the main thread so that it can write out the data.
       server.awaker.signal()
@@ -141,7 +145,7 @@ class MasterClientHandler(daemon.AsyncTerminatedMessageStream):
     self.server.request_workers.AddTask((self.server, message, self))
 
 
-class _MasterShutdownCheck:
+class _MasterShutdownCheck(object):
   """Logic for master daemon shutdown.
 
   """
@@ -265,7 +269,7 @@ class MasterServer(daemon.AsyncStreamServer):
         self.context.jobqueue.Shutdown()
 
 
-class ClientOps:
+class ClientOps(object):
   """Class holding high-level client operations."""
   def __init__(self, server):
     self.server = server
@@ -293,6 +297,19 @@ class ClientOps:
       _LogNewJob(True, job_id, ops)
       return job_id
 
+    elif method == luxi.REQ_PICKUP_JOB:
+      logging.info("Picking up new job from queue")
+      (job_id, ) = args
+      queue.PickupJob(job_id)
+
+    elif method == luxi.REQ_SUBMIT_JOB_TO_DRAINED_QUEUE:
+      logging.info("Forcefully receiving new job")
+      (job_def, ) = args
+      ops = [opcodes.OpCode.LoadOpCode(state) for state in job_def]
+      job_id = queue.SubmitJobToDrainedQueue(ops)
+      _LogNewJob(True, job_id, ops)
+      return job_id
+
     elif method == luxi.REQ_SUBMIT_MANY_JOBS:
       logging.info("Receiving multiple jobs")
       (job_defs, ) = args
@@ -346,7 +363,8 @@ class ClientOps:
       elif what == constants.QR_JOB:
         return queue.QueryJobs(fields, qfilter)
       elif what in constants.QR_VIA_LUXI:
-        raise NotImplementedError
+        luxi_client = runtime.GetClient(query=True)
+        result = luxi_client.Query(what, fields, qfilter).ToDict()
       else:
         raise errors.OpPrereqError("Resource type '%s' unknown" % what,
                                    errors.ECODE_INVAL)
@@ -374,53 +392,6 @@ class ClientOps:
       logging.info("Received job query request for %s", msg)
       return queue.OldStyleQueryJobs(job_ids, fields)
 
-    elif method == luxi.REQ_QUERY_INSTANCES:
-      (names, fields, use_locking) = args
-      logging.info("Received instance query request for %s", names)
-      if use_locking:
-        raise errors.OpPrereqError("Sync queries are not allowed",
-                                   errors.ECODE_INVAL)
-      op = opcodes.OpInstanceQuery(names=names, output_fields=fields,
-                                   use_locking=use_locking)
-      return self._Query(op)
-
-    elif method == luxi.REQ_QUERY_NODES:
-      (names, fields, use_locking) = args
-      logging.info("Received node query request for %s", names)
-      if use_locking:
-        raise errors.OpPrereqError("Sync queries are not allowed",
-                                   errors.ECODE_INVAL)
-      op = opcodes.OpNodeQuery(names=names, output_fields=fields,
-                               use_locking=use_locking)
-      return self._Query(op)
-
-    elif method == luxi.REQ_QUERY_GROUPS:
-      (names, fields, use_locking) = args
-      logging.info("Received group query request for %s", names)
-      if use_locking:
-        raise errors.OpPrereqError("Sync queries are not allowed",
-                                   errors.ECODE_INVAL)
-      op = opcodes.OpGroupQuery(names=names, output_fields=fields)
-      return self._Query(op)
-
-    elif method == luxi.REQ_QUERY_NETWORKS:
-      (names, fields, use_locking) = args
-      logging.info("Received network query request for %s", names)
-      if use_locking:
-        raise errors.OpPrereqError("Sync queries are not allowed",
-                                   errors.ECODE_INVAL)
-      op = opcodes.OpNetworkQuery(names=names, output_fields=fields)
-      return self._Query(op)
-
-    elif method == luxi.REQ_QUERY_EXPORTS:
-      (nodes, use_locking) = args
-      if use_locking:
-        raise errors.OpPrereqError("Sync queries are not allowed",
-                                   errors.ECODE_INVAL)
-      logging.info("Received exports query request")
-      op = opcodes.OpBackupQuery(nodes=nodes, use_locking=use_locking)
-      return self._Query(op)
-
     elif method == luxi.REQ_QUERY_CONFIG_VALUES:
       (fields, ) = args
       logging.info("Received config values query request for %s", fields)
@@ -685,8 +656,8 @@ def CheckMasterd(options, args):
   try:
     config.ConfigWriter()
   except errors.ConfigVersionMismatch, err:
-    v1 = "%s.%s.%s" % constants.SplitVersion(err.args[0])
-    v2 = "%s.%s.%s" % constants.SplitVersion(err.args[1])
+    v1 = "%s.%s.%s" % version.SplitVersion(err.args[0])
+    v2 = "%s.%s.%s" % version.SplitVersion(err.args[1])
     print >> sys.stderr,  \
         ("Configuration version mismatch. The current Ganeti software"
          " expects version %s, but the on-disk configuration file has"
