@@ -71,7 +71,7 @@ def ErrnoOrStr(err):
   return detail
 
 
-class FileStatHelper:
+class FileStatHelper(object):
   """Helper to store file handle's C{fstat}.
 
   Useful in combination with L{ReadFile}'s C{preread} parameter.
@@ -681,8 +681,9 @@ def PathJoin(*args):
   @raise ValueError: for invalid paths
 
   """
-  # ensure we're having at least one path passed in
-  assert args
+  # ensure we're having at least two paths passed in
+  if len(args) <= 1:
+    raise errors.ProgrammerError("PathJoin requires two arguments")
   # ensure the first component is an absolute and normalized path name
   root = args[0]
   if not IsNormAbsPath(root):
