@@ -239,7 +239,14 @@ def ShellQuoteArgs(args):
   return " ".join([ShellQuote(i) for i in args])
 
 
-class ShellWriter:
+def ShellCombineCommands(cmdlist):
+  """Out of a list of shell comands construct a single one.
+
+  """
+  return ["/bin/sh", "-c", " && ".join(ShellQuoteArgs(c) for c in cmdlist)]
+
+
+class ShellWriter(object):
   """Helper class to write scripts with indentation.
 
   """
@@ -453,6 +460,19 @@ def UnescapeAndSplit(text, sep=","):
   return rlist
 
 
+def EscapeAndJoin(slist, sep=","):
+  """Encode a list in a way parsable by UnescapeAndSplit.
+
+  @type slist: list of strings
+  @param slist: the strings to be encoded
+  @rtype: string
+  @return: the encoding of the list oas a string
+
+  """
+  return sep.join([re.sub("\\" + sep, "\\\\" + sep,
+                          re.sub(r"\\", r"\\\\", v)) for v in slist])
+
+
 def CommaJoin(names):
   """Nicely join a set of identifiers.
 
@@ -510,7 +530,7 @@ def FormatSeconds(secs):
   return " ".join(parts)
 
 
-class LineSplitter:
+class LineSplitter(object):
   """Splits data chunks into lines separated by newline.
 
   Instances provide a file-like interface.
