@@ -32,6 +32,7 @@ from ganeti import compat
 from ganeti import ht
 
 import qa_error
+import qa_logging
 
 
 _INSTANCE_CHECK_KEY = "instance-check"
@@ -40,6 +41,13 @@ _VCLUSTER_MASTER_KEY = "vcluster-master"
 _VCLUSTER_BASEDIR_KEY = "vcluster-basedir"
 _ENABLED_DISK_TEMPLATES_KEY = "enabled-disk-templates"
 
+# The constants related to JSON patching (as per RFC6902) that modifies QA's
+# configuration.
+_QA_BASE_PATH = os.path.dirname(__file__)
+_QA_DEFAULT_PATCH = "qa-patch.json"
+_QA_PATCH_DIR = "patch"
+_QA_PATCH_ORDER_FILE = "order"
+
 #: QA configuration (L{_QaConfig})
 _config = None
 
@@ -250,6 +258,108 @@ class _QaConfig(object):
     #: Cluster-wide run-time value of the exclusive storage flag
     self._exclusive_storage = None
 
+  @staticmethod
+  def LoadPatch(patch_dict, rel_path):
+    """ Loads a single patch.
+
+    @type patch_dict: dict of string to dict
+    @param patch_dict: A dictionary storing patches by relative path.
+    @type rel_path: string
+    @param rel_path: The relative path to the patch, might or might not exist.
+
+    """
+    try:
+      full_path = os.path.join(_QA_BASE_PATH, rel_path)
+      patch = serializer.LoadJson(utils.ReadFile(full_path))
+      patch_dict[rel_path] = patch
+    except IOError:
+      pass
+
+  @staticmethod
+  def LoadPatches():
+    """ Finds and loads all patches supported by the QA.
+
+    @rtype: dict of string to dict
+    @return: A dictionary of relative path to patch content.
+
+    """
+    patches = {}
+    _QaConfig.LoadPatch(patches, _QA_DEFAULT_PATCH)
+    patch_dir_path = os.path.join(_QA_BASE_PATH, _QA_PATCH_DIR)
+    if os.path.exists(patch_dir_path):
+      for filename in os.listdir(patch_dir_path):
+        if filename.endswith(".json"):
+          _QaConfig.LoadPatch(patches, os.path.join(_QA_PATCH_DIR, filename))
+    return patches
+
+  @staticmethod
+  def ApplyPatch(data, patch_module, patches, patch_path):
+    """Applies a single patch.
+
+    @type data: dict (deserialized json)
+    @param data: The QA configuration to modify
+    @type patch_module: module
+    @param patch_module: The json patch module, loaded dynamically
+    @type patches: dict of string to dict
+    @param patches: The dictionary of patch path to content
+    @type patch_path: string
+    @param patch_path: The path to the patch, relative to the QA directory
+
+    """
+    patch_content = patches[patch_path]
+    print qa_logging.FormatInfo("Applying patch %s" % patch_path)
+    if not patch_content and patch_path != _QA_DEFAULT_PATCH:
+      print qa_logging.FormatWarning("The patch %s added by the user is empty" %
+                                     patch_path)
+    patch_module.apply_patch(data, patch_content, in_place=True)
+
+  @staticmethod
+  def ApplyPatches(data, patch_module, patches):
+    """Applies any patches present, and returns the modified QA configuration.
+
+    First, patches from the patch directory are applied. They are ordered
+    alphabetically, unless there is an ``order`` file present - any patches
+    listed within are applied in that order, and any remaining ones in
+    alphabetical order again. Finally, the default patch residing in the
+    top-level QA directory is applied.
+
+    @type data: dict (deserialized json)
+    @param data: The QA configuration to modify
+    @type patch_module: module
+    @param patch_module: The json patch module, loaded dynamically
+    @type patches: dict of string to dict
+    @param patches: The dictionary of patch path to content
+
+    """
+    ordered_patches = []
+    order_path = os.path.join(_QA_BASE_PATH, _QA_PATCH_DIR,
+                              _QA_PATCH_ORDER_FILE)
+    if os.path.exists(order_path):
+      order_file = open(order_path, 'r')
+      ordered_patches = order_file.read().splitlines()
+      # Removes empty lines
+      ordered_patches = filter(None, ordered_patches)
+
+    # Add the patch dir
+    ordered_patches = map(lambda x: os.path.join(_QA_PATCH_DIR, x),
+                          ordered_patches)
+
+    # First the ordered patches
+    for patch in ordered_patches:
+      if patch not in patches:
+        raise qa_error.Error("Patch %s specified in the ordering file does not "
+                             "exist" % patch)
+      _QaConfig.ApplyPatch(data, patch_module, patches, patch)
+
+    # Then the other non-default ones
+    for patch in sorted(patches):
+      if patch != _QA_DEFAULT_PATCH and patch not in ordered_patches:
+        _QaConfig.ApplyPatch(data, patch_module, patches, patch)
+
+    # Finally the default one
+    if _QA_DEFAULT_PATCH in patches:
+      _QaConfig.ApplyPatch(data, patch_module, patches, _QA_DEFAULT_PATCH)
+
   @classmethod
   def Load(cls, filename):
     """Loads a configuration file and produces a configuration object.
@@ -261,6 +371,21 @@ class _QaConfig(object):
     """
     data = serializer.LoadJson(utils.ReadFile(filename))
 
+    # Patch the document using JSON Patch (RFC6902) in file _PATCH_JSON, if
+    # available
+    try:
+      patches = _QaConfig.LoadPatches()
+      # Try to use the module only if there is a non-empty patch present
+      if any(patches.values()):
+        mod = __import__("jsonpatch", fromlist=[])
+        _QaConfig.ApplyPatches(data, mod, patches)
+    except IOError:
+      pass
+    except ImportError:
+      raise qa_error.Error("For the QA JSON patching feature to work, you "
+                           "need to install Python modules 'jsonpatch' and "
+                           "'jsonpointer'.")
+
     result = cls(dict(map(_ConvertResources,
                           data.items()))) # pylint: disable=E1103
     result.Validate()
@@ -462,9 +587,9 @@ class _QaConfig(object):
     """
     enabled_disk_templates = self.GetEnabledDiskTemplates()
     if storage_type == constants.ST_LVM_PV:
-      disk_templates = utils.GetDiskTemplatesOfStorageType(constants.ST_LVM_VG)
+      disk_templates = utils.GetDiskTemplatesOfStorageTypes(constants.ST_LVM_VG)
     else:
-      disk_templates = utils.GetDiskTemplatesOfStorageType(storage_type)
+      disk_templates = utils.GetDiskTemplatesOfStorageTypes(storage_type)
     return bool(set(enabled_disk_templates).intersection(set(disk_templates)))
 
   def AreSpindlesSupported(self):
@@ -535,7 +660,7 @@ def get(name, default=None):
   return GetConfig().get(name, default=default)
 
 
-class Either:
+class Either(object):
   def __init__(self, tests):
     """Initializes this class.
 
@@ -684,6 +809,48 @@ def AcquireInstance(_cfg=None):
   return instance
 
 
+def AcquireManyInstances(num, _cfg=None):
+  """Return instances that are not in use.
+
+  @type num: int
+  @param num: Number of instances; can be 0.
+  @rtype: list of instances
+  @return: C{num} different instances
+
+  """
+
+  if _cfg is None:
+    cfg = GetConfig()
+  else:
+    cfg = _cfg
+
+  # Filter out unwanted instances
+  instances = filter(lambda inst: not inst.used, cfg["instances"])
+
+  if len(instances) < num:
+    raise qa_error.OutOfInstancesError(
+      "Not enough instances left (%d needed, %d remaining)" %
+      (num, len(instances))
+    )
+
+  instances = []
+
+  try:
+    for _ in range(0, num):
+      n = AcquireInstance(_cfg=cfg)
+      instances.append(n)
+  except qa_error.OutOfInstancesError:
+    ReleaseManyInstances(instances)
+    raise
+
+  return instances
+
+
+def ReleaseManyInstances(instances):
+  for instance in instances:
+    instance.Release()
+
+
 def SetExclusiveStorage(value):
   """Wrapper for L{_QaConfig.SetExclusiveStorage}.
 
@@ -757,7 +924,25 @@ def AcquireNode(exclude=None, _cfg=None):
   return sorted(nodes, key=_NodeSortKey)[0].Use()
 
 
-def AcquireManyNodes(num, exclude=None):
+class AcquireManyNodesCtx(object):
+  """Returns the least used nodes for use with a `with` block
+
+  """
+  def __init__(self, num, exclude=None, cfg=None):
+    self._num = num
+    self._exclude = exclude
+    self._cfg = cfg
+
+  def __enter__(self):
+    self._nodes = AcquireManyNodes(self._num, exclude=self._exclude,
+                                   cfg=self._cfg)
+    return self._nodes
+
+  def __exit__(self, exc_type, exc_value, exc_tb):
+    ReleaseManyNodes(self._nodes)
+
+
+def AcquireManyNodes(num, exclude=None, cfg=None):
   """Return the least used nodes.
 
   @type num: int
@@ -779,7 +964,7 @@ def AcquireManyNodes(num, exclude=None):
 
   try:
     for _ in range(0, num):
-      n = AcquireNode(exclude=exclude)
+      n = AcquireNode(exclude=exclude, _cfg=cfg)
       nodes.append(n)
       exclude.append(n)
   except qa_error.OutOfNodesError:
