@@ -32,10 +32,10 @@ import qa_config
 import qa_error
 import qa_utils
 
-from qa_utils import AssertCommand, AssertEqual
+from qa_utils import AssertCommand, AssertEqual, AssertIn, GetCommandOutput
 
 
-def _NodeAdd(node, readd=False):
+def NodeAdd(node, readd=False, group=None):
   if not readd and node.added:
     raise qa_error.Error("Node %s already in cluster" % node.primary)
   elif readd and not node.added:
@@ -46,6 +46,8 @@ def _NodeAdd(node, readd=False):
     cmd.append("--secondary-ip=%s" % node.secondary)
   if readd:
     cmd.append("--readd")
+  if group is not None:
+    cmd.extend(["--node-group", group])
   cmd.append(node.primary)
 
   AssertCommand(cmd)
@@ -56,7 +58,7 @@ def _NodeAdd(node, readd=False):
     node.MarkAdded()
 
 
-def _NodeRemove(node):
+def NodeRemove(node):
   AssertCommand(["gnt-node", "remove", node.primary])
   node.MarkRemoved()
 
@@ -72,7 +74,7 @@ def TestNodeAddAll():
   master = qa_config.GetMasterNode()
   for node in qa_config.get("nodes"):
     if node != master:
-      _NodeAdd(node, readd=False)
+      NodeAdd(node, readd=False)
 
 
 def MarkNodeAddedAll():
@@ -92,12 +94,12 @@ def TestNodeRemoveAll():
   master = qa_config.GetMasterNode()
   for node in qa_config.get("nodes"):
     if node != master:
-      _NodeRemove(node)
+      NodeRemove(node)
 
 
 def TestNodeReadd(node):
   """gnt-node add --readd"""
-  _NodeAdd(node, readd=True)
+  NodeAdd(node, readd=True)
 
 
 def TestNodeInfo():
@@ -129,8 +131,7 @@ def TestNodeStorage():
 
     # Test all storage fields
     cmd = ["gnt-node", "list-storage", "--storage-type", storage_type,
-           "--output=%s" % ",".join(list(constants.VALID_STORAGE_FIELDS) +
-                                    [constants.SF_NODE, constants.SF_TYPE])]
+           "--output=%s" % ",".join(list(constants.VALID_STORAGE_FIELDS))]
     AssertCommand(cmd)
 
     # Get list of valid storage devices
@@ -451,9 +452,30 @@ def TestNodeListFields():
   qa_utils.GenericQueryFieldsTest("gnt-node", query.NODE_FIELDS.keys())
 
 
-def TestNodeListDrbd(node):
+def TestNodeListDrbd(node, is_drbd):
   """gnt-node list-drbd"""
-  AssertCommand(["gnt-node", "list-drbd", node.primary])
+  master = qa_config.GetMasterNode()
+  result_output = GetCommandOutput(master.primary,
+                                   "gnt-node list-drbd --no-header %s" %
+                                   node.primary)
+  # Meaningful to note: there is but one instance, and the node is either the
+  # primary or one of the secondaries
+  if is_drbd:
+    # Invoked for both primary and secondary
+    per_disk_info = result_output.splitlines()
+    for line in per_disk_info:
+      try:
+        drbd_node, _, _, _, _, drbd_peer = line.split()
+      except ValueError:
+        raise qa_error.Error("Could not examine list-drbd output: expected a"
+                             " single row of 6 entries, found the following:"
+                             " %s" % line)
+
+      AssertIn(node.primary, [drbd_node, drbd_peer],
+               msg="The output %s does not contain the node" % line)
+  else:
+    # Output should be empty, barring newlines
+    AssertEqual(result_output.strip(), "")
 
 
 def _BuildSetESCmd(action, value, node_name):
