@@ -1,7 +1,7 @@
 #
 #
 
-# Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Google Inc.
+# Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Google Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,28 +23,32 @@
 
 """
 
-import tempfile
+import functools
+import itertools
 import random
 import re
-import itertools
+import tempfile
 
-from ganeti import utils
+from ganeti import cli
+from ganeti import compat
 from ganeti import constants
 from ganeti import errors
-from ganeti import cli
-from ganeti import rapi
-from ganeti import objects
-from ganeti import query
-from ganeti import compat
-from ganeti import qlang
 from ganeti import pathutils
+from ganeti import objects
+from ganeti import opcodes
+from ganeti import query
+from ganeti import qlang
+from ganeti import rapi
+from ganeti import utils
 
+from ganeti.http.auth import ParsePasswordFile
 import ganeti.rapi.client        # pylint: disable=W0611
 import ganeti.rapi.client_utils
 
 import qa_config
-import qa_utils
 import qa_error
+import qa_logging
+import qa_utils
 
 from qa_instance import IsFailoverSupported
 from qa_instance import IsMigrationSupported
@@ -90,8 +94,8 @@ def Setup(username, password):
 
   if qa_config.UseVirtualCluster():
     # TODO: Implement full support for RAPI on virtual clusters
-    print qa_utils.FormatWarning("RAPI tests are not yet supported on"
-                                 " virtual clusters and will be disabled")
+    print qa_logging.FormatWarning("RAPI tests are not yet supported on"
+                                   " virtual clusters and will be disabled")
 
     assert _rapi_client is None
   else:
@@ -102,6 +106,33 @@ def Setup(username, password):
 
     print "RAPI protocol version: %s" % _rapi_client.GetVersion()
 
+  return _rapi_client
+
+
+def LookupRapiSecret(rapi_user):
+  """Find the RAPI secret for the given user.
+
+  @param rapi_user: Login user
+  @return: Login secret for the user
+
+  """
+  CTEXT = "{CLEARTEXT}"
+  master = qa_config.GetMasterNode()
+  cmd = ["cat", qa_utils.MakeNodePath(master, pathutils.RAPI_USERS_FILE)]
+  file_content = qa_utils.GetCommandOutput(master.primary,
+                                           utils.ShellQuoteArgs(cmd))
+  users = ParsePasswordFile(file_content)
+  entry = users.get(rapi_user)
+  if not entry:
+    raise qa_error.Error("User %s not found in RAPI users file" % rapi_user)
+  secret = entry.password
+  if secret.upper().startswith(CTEXT):
+    secret = secret[len(CTEXT):]
+  elif secret.startswith("{"):
+    raise qa_error.Error("Unsupported password schema for RAPI user %s:"
+                         " not a clear text password" % rapi_user)
+  return secret
+
 
 INSTANCE_FIELDS = ("name", "os", "pnode", "snodes",
                    "admin_state",
@@ -160,6 +191,74 @@ def _DoTests(uris):
   return results
 
 
+# pylint: disable=W0212
+# Due to _SendRequest usage
+def _DoGetPutTests(get_uri, modify_uri, opcode_params, rapi_only_aliases=None,
+                   modify_method="PUT", exceptions=None, set_exceptions=None):
+  """ Test if all params of an object can be retrieved, and set as well.
+
+  @type get_uri: string
+  @param get_uri: The URI from which information about the object can be
+                  retrieved.
+  @type modify_uri: string
+  @param modify_uri: The URI which can be used to modify the object.
+  @type opcode_params: list of tuple
+  @param opcode_params: The parameters of the underlying opcode, used to
+                        determine which parameters are actually present.
+  @type rapi_only_aliases: list of string or None
+  @param rapi_only_aliases: Aliases for parameters which differ from the opcode,
+                            and become renamed before opcode submission.
+  @type modify_method: string
+  @param modify_method: The method to be used in the modification.
+  @type exceptions: list of string or None
+  @param exceptions: The parameters which have not been exposed and should not
+                     be tested at all.
+  @type set_exceptions: list of string or None
+  @param set_exceptions: The parameters whose setting should not be tested as a
+                         part of this test.
+
+  """
+
+  assert get_uri.startswith("/")
+  assert modify_uri.startswith("/")
+
+  if exceptions is None:
+    exceptions = []
+  if set_exceptions is None:
+    set_exceptions = []
+
+  print "Testing get/modify symmetry of %s and %s" % (get_uri, modify_uri)
+
+  # First we see if all parameters of the opcode are returned through RAPI
+  params_of_interest = map(lambda x: x[0], opcode_params)
+
+  # The RAPI-specific aliases are to be checked as well
+  if rapi_only_aliases is not None:
+    params_of_interest.extend(rapi_only_aliases)
+
+  info = _rapi_client._SendRequest("GET", get_uri, None, {})
+
+  missing_params = filter(lambda x: x not in info and x not in exceptions,
+                          params_of_interest)
+  if missing_params:
+    raise qa_error.Error("The parameters %s which can be set through the "
+                         "appropriate opcode are not present in the response "
+                         "from %s" % (','.join(missing_params), get_uri))
+
+  print "GET successful at %s" % get_uri
+
+  # Then if we can perform a set with the same values as received
+  put_payload = {}
+  for param in params_of_interest:
+    if param not in exceptions and param not in set_exceptions:
+      put_payload[param] = info[param]
+
+  _rapi_client._SendRequest(modify_method, modify_uri, None, put_payload)
+
+  print "%s successful at %s" % (modify_method, modify_uri)
+# pylint: enable=W0212
+
+
 def _VerifyReturnsJob(data):
   if not isinstance(data, int):
     AssertMatch(data, r"^\d+$")
@@ -241,6 +340,21 @@ def TestEmptyCluster():
     else:
       raise qa_error.Error("Non-implemented method didn't fail")
 
+  # Test GET/PUT symmetry
+  LEGITIMATELY_MISSING = [
+    "force",       # Standard option
+    "add_uids",    # Modifies UID pool, is not a param itself
+    "remove_uids", # Same as above
+  ]
+  NOT_EXPOSED_YET = ["hv_state", "disk_state", "modify_etc_hosts"]
+  # The nicparams are returned under the default entry, yet accepted as they
+  # are - this is a TODO to fix!
+  DEFAULT_ISSUES = ["nicparams"]
+
+  _DoGetPutTests("/2/info", "/2/modify", opcodes.OpClusterSetParams.OP_PARAMS,
+                 exceptions=(LEGITIMATELY_MISSING + NOT_EXPOSED_YET),
+                 set_exceptions=DEFAULT_ISSUES)
+
 
 def TestRapiQuery():
   """Testing resource queries via remote API.
@@ -257,10 +371,13 @@ def TestRapiQuery():
   for what in constants.QR_VIA_RAPI:
     if what == constants.QR_JOB:
       namefield = "id"
+      trivial_filter = [qlang.OP_GE, namefield, 0]
     elif what == constants.QR_EXPORT:
       namefield = "export"
+      trivial_filter = [qlang.OP_REGEXP, ".*", namefield]
     else:
       namefield = "name"
+      trivial_filter = [qlang.OP_REGEXP, ".*", namefield]
 
     all_fields = query.ALL_FIELDS[what].keys()
     rnd.shuffle(all_fields)
@@ -322,22 +439,24 @@ def TestRapiQuery():
       # Note the spaces
       ("/2/query/%s?fields=%s,%%20%s%%09,%s%%20" %
        (what, namefield, namefield, namefield),
-       compat.partial(_Check, [namefield] * 3), "GET", None),
+       compat.partial(_Check, [namefield] * 3), "GET", None)])
 
-      # PUT with fields in query
-      ("/2/query/%s?fields=%s" % (what, namefield),
-       compat.partial(_Check, [namefield]), "PUT", {}),
+    if what in constants.QR_VIA_RAPI_PUT:
+      _DoTests([
+        # PUT with fields in query
+        ("/2/query/%s?fields=%s" % (what, namefield),
+         compat.partial(_Check, [namefield]), "PUT", {}),
 
-      ("/2/query/%s" % what, compat.partial(_Check, [namefield] * 4), "PUT", {
-         "fields": [namefield] * 4,
-         }),
+        ("/2/query/%s" % what, compat.partial(_Check, [namefield] * 4), "PUT", {
+           "fields": [namefield] * 4,
+           }),
 
-      ("/2/query/%s" % what, compat.partial(_Check, all_fields), "PUT", {
-         "fields": all_fields,
-         }),
+        ("/2/query/%s" % what, compat.partial(_Check, all_fields), "PUT", {
+           "fields": all_fields,
+           }),
 
-      ("/2/query/%s" % what, compat.partial(_Check, [namefield] * 4), "PUT", {
-         "fields": [namefield] * 4
+        ("/2/query/%s" % what, compat.partial(_Check, [namefield] * 4), "PUT", {
+           "fields": [namefield] * 4
          })])
 
     def _CheckFilter():
@@ -345,7 +464,7 @@ def TestRapiQuery():
         # With filter
         ("/2/query/%s" % what, compat.partial(_Check, all_fields), "PUT", {
            "fields": all_fields,
-           "filter": [qlang.OP_TRUE, namefield],
+           "filter": trivial_filter
            }),
         ])
 
@@ -358,7 +477,8 @@ def TestRapiQuery():
       else:
         raise qa_error.Error("Filtering locks didn't fail")
     else:
-      _CheckFilter()
+      if what in constants.QR_VIA_RAPI_PUT:
+        _CheckFilter()
 
     if what == constants.QR_NODE:
       # Test with filter
@@ -439,6 +559,20 @@ def TestNode(node):
     ("/2/nodes?bulk=1", _VerifyNodesBulk, "GET", None),
     ])
 
+  # Not parameters of the node, but controlling opcode behavior
+  LEGITIMATELY_MISSING = ["force", "powered"]
+  # Identifying the node - RAPI provides these itself
+  IDENTIFIERS = ["node_name", "node_uuid"]
+  # As the name states, these can be set but not retrieved yet
+  NOT_EXPOSED_YET = ["hv_state", "disk_state", "auto_promote"]
+
+  _DoGetPutTests("/2/nodes/%s" % node.primary,
+                 "/2/nodes/%s/modify" % node.primary,
+                 opcodes.OpNodeSetParams.OP_PARAMS,
+                 modify_method="POST",
+                 exceptions=(LEGITIMATELY_MISSING + NOT_EXPOSED_YET +
+                             IDENTIFIERS))
+
 
 def _FilterTags(seq):
   """Removes unwanted tags from a sequence.
@@ -560,6 +694,27 @@ def TestRapiNodeGroups():
 
   _WaitForRapiJob(job_id)
 
+  # Test for get/set symmetry
+
+  # Identifying the node - RAPI provides these itself
+  IDENTIFIERS = ["group_name"]
+  # As the name states, not exposed yet
+  NOT_EXPOSED_YET = ["hv_state", "disk_state"]
+
+  # The parameters we do not want to get and set (as that sets the
+  # group-specific params to the filled ones)
+  FILLED_PARAMS = ["ndparams", "ipolicy", "diskparams"]
+
+  # The aliases that we can use to perform this test with the group-specific
+  # params
+  CUSTOM_PARAMS = ["custom_ndparams", "custom_ipolicy", "custom_diskparams"]
+
+  _DoGetPutTests("/2/groups/%s" % group3, "/2/groups/%s/modify" % group3,
+                 opcodes.OpGroupSetParams.OP_PARAMS,
+                 rapi_only_aliases=CUSTOM_PARAMS,
+                 exceptions=(IDENTIFIERS + NOT_EXPOSED_YET),
+                 set_exceptions=FILLED_PARAMS)
+
   # Delete groups
   for group in [group1, group3]:
     (job_id, ) = _DoTests([
@@ -622,6 +777,76 @@ def TestRapiInstanceAdd(node, use_client):
     raise
 
 
+def _GenInstanceAllocationDict(node, instance):
+  """Creates an instance allocation dict to be used with the RAPI"""
+  instance.SetDiskTemplate(constants.DT_PLAIN)
+
+  disks = [{"size": utils.ParseUnit(d.get("size")),
+              "name": str(d.get("name"))}
+             for d in qa_config.GetDiskOptions()]
+
+  nic0_mac = instance.GetNicMacAddr(0, constants.VALUE_GENERATE)
+  nics = [{
+    constants.INIC_MAC: nic0_mac,
+    }]
+
+  beparams = {
+    constants.BE_MAXMEM: utils.ParseUnit(qa_config.get(constants.BE_MAXMEM)),
+    constants.BE_MINMEM: utils.ParseUnit(qa_config.get(constants.BE_MINMEM)),
+    }
+
+  return _rapi_client.InstanceAllocation(constants.INSTANCE_CREATE,
+                                         instance.name,
+                                         constants.DT_PLAIN,
+                                         disks, nics,
+                                         os=qa_config.get("os"),
+                                         pnode=node.primary,
+                                         beparams=beparams)
+
+
+def TestRapiInstanceMultiAlloc(node):
+  """Test adding two new instances via the RAPI instance-multi-alloc method"""
+  if not qa_config.IsTemplateSupported(constants.DT_PLAIN):
+    return
+
+  JOBS_KEY = "jobs"
+
+  instance_one = qa_config.AcquireInstance()
+  instance_two = qa_config.AcquireInstance()
+  instance_list = [instance_one, instance_two]
+  try:
+    rapi_dicts = map(functools.partial(_GenInstanceAllocationDict, node),
+                     instance_list)
+
+    job_id = _rapi_client.InstancesMultiAlloc(rapi_dicts)
+
+    results, = _WaitForRapiJob(job_id)
+
+    if JOBS_KEY not in results:
+      raise qa_error.Error("RAPI instance-multi-alloc did not deliver "
+                           "information about created jobs")
+
+    if len(results[JOBS_KEY]) != len(instance_list):
+      raise qa_error.Error("RAPI instance-multi-alloc failed to return the "
+                           "desired number of jobs!")
+
+    for success, job in results[JOBS_KEY]:
+      if success:
+        _WaitForRapiJob(job)
+      else:
+        raise qa_error.Error("Failed to create instance in "
+                             "instance-multi-alloc call")
+  except:
+    # Note that although released, it may be that some of the instance creations
+    # have in fact succeeded. Handling this in a better way may be possible, but
+    # is not necessary as the QA has already failed at this point.
+    for instance in instance_list:
+      instance.Release()
+    raise
+
+  return (instance_one, instance_two)
+
+
 @InstanceCheck(None, INST_DOWN, FIRST_ARG)
 def TestRapiInstanceRemove(instance, use_client):
   """Test removing instance via RAPI"""
@@ -644,8 +869,8 @@ def TestRapiInstanceRemove(instance, use_client):
 def TestRapiInstanceMigrate(instance):
   """Test migrating instance via RAPI"""
   if not IsMigrationSupported(instance):
-    print qa_utils.FormatInfo("Instance doesn't support migration, skipping"
-                              " test")
+    print qa_logging.FormatInfo("Instance doesn't support migration, skipping"
+                                " test")
     return
   # Move to secondary node
   _WaitForRapiJob(_rapi_client.MigrateInstance(instance.name))
@@ -658,8 +883,8 @@ def TestRapiInstanceMigrate(instance):
 def TestRapiInstanceFailover(instance):
   """Test failing over instance via RAPI"""
   if not IsFailoverSupported(instance):
-    print qa_utils.FormatInfo("Instance doesn't support failover, skipping"
-                              " test")
+    print qa_logging.FormatInfo("Instance doesn't support failover, skipping"
+                                " test")
     return
   # Move to secondary node
   _WaitForRapiJob(_rapi_client.FailoverInstance(instance.name))
@@ -699,7 +924,7 @@ def TestRapiInstanceRenameAndBack(rename_source, rename_target):
 def TestRapiInstanceReinstall(instance):
   """Test reinstalling an instance via RAPI"""
   if instance.disk_template == constants.DT_DISKLESS:
-    print qa_utils.FormatInfo("Test not supported for diskless instances")
+    print qa_logging.FormatInfo("Test not supported for diskless instances")
     return
 
   _WaitForRapiJob(_rapi_client.ReinstallInstance(instance.name))
@@ -715,8 +940,8 @@ def TestRapiInstanceReinstall(instance):
 def TestRapiInstanceReplaceDisks(instance):
   """Test replacing instance disks via RAPI"""
   if not IsDiskReplacingSupported(instance):
-    print qa_utils.FormatInfo("Instance doesn't support disk replacing,"
-                              " skipping test")
+    print qa_logging.FormatInfo("Instance doesn't support disk replacing,"
+                                " skipping test")
     return
   fn = _rapi_client.ReplaceInstanceDisks
   _WaitForRapiJob(fn(instance.name,
@@ -762,7 +987,7 @@ def TestRapiInstanceConsole(instance):
   """Test getting instance console information via RAPI"""
   result = _rapi_client.GetInstanceConsole(instance.name)
   console = objects.InstanceConsole.FromDict(result)
-  AssertEqual(console.Validate(), True)
+  AssertEqual(console.Validate(), None)
   AssertEqual(console.instance, qa_utils.ResolveInstanceName(instance.name))
 
 
@@ -786,7 +1011,7 @@ def GetOperatingSystems():
 
 
 def TestInterClusterInstanceMove(src_instance, dest_instance,
-                                 inodes, tnode):
+                                 inodes, tnode, perform_checks=True):
   """Test tools/move-instance"""
   master = qa_config.GetMasterNode()
 
@@ -794,7 +1019,9 @@ def TestInterClusterInstanceMove(src_instance, dest_instance,
   rapi_pw_file.write(_rapi_password)
   rapi_pw_file.flush()
 
-  dest_instance.SetDiskTemplate(src_instance.disk_template)
+  # Needed only if checks are to be performed
+  if perform_checks:
+    dest_instance.SetDiskTemplate(src_instance.disk_template)
 
   # TODO: Run some instance tests before moving back
 
@@ -810,26 +1037,32 @@ def TestInterClusterInstanceMove(src_instance, dest_instance,
   pnode = inodes[0]
   # note: pnode:snode are the *current* nodes, so we move it first to
   # tnode:pnode, then back to pnode:snode
-  for si, di, pn, sn in [(src_instance.name, dest_instance.name,
-                          tnode.primary, pnode.primary),
-                         (dest_instance.name, src_instance.name,
-                          pnode.primary, snode.primary)]:
+  for current_src_inst, current_dest_inst, target_pnode, target_snode in \
+    [(src_instance.name, dest_instance.name, tnode.primary, pnode.primary),
+     (dest_instance.name, src_instance.name, pnode.primary, snode.primary)]:
     cmd = [
       "../tools/move-instance",
       "--verbose",
       "--src-ca-file=%s" % _rapi_ca.name,
       "--src-username=%s" % _rapi_username,
       "--src-password-file=%s" % rapi_pw_file.name,
-      "--dest-instance-name=%s" % di,
-      "--dest-primary-node=%s" % pn,
-      "--dest-secondary-node=%s" % sn,
+      "--dest-instance-name=%s" % current_dest_inst,
+      "--dest-primary-node=%s" % target_pnode,
+      "--dest-secondary-node=%s" % target_snode,
       "--net=0:mac=%s" % constants.VALUE_GENERATE,
       master.primary,
       master.primary,
-      si,
+      current_src_inst,
       ]
 
-    qa_utils.RunInstanceCheck(di, False)
+    # Some uses of this test might require that RAPI-only commands are used,
+    # and the checks are command-line based.
+
+    if perform_checks:
+      qa_utils.RunInstanceCheck(current_dest_inst, False)
+
     AssertEqual(StartLocalCommand(cmd).wait(), 0)
-    qa_utils.RunInstanceCheck(si, False)
-    qa_utils.RunInstanceCheck(di, True)
+
+    if perform_checks:
+      qa_utils.RunInstanceCheck(current_src_inst, False)
+      qa_utils.RunInstanceCheck(current_dest_inst, True)
