@@ -24,10 +24,12 @@
 """
 
 import copy
+import datetime
 import operator
 import os
 import random
 import re
+import socket
 import subprocess
 import sys
 import tempfile
@@ -45,14 +47,12 @@ from ganeti import ht
 from ganeti import pathutils
 from ganeti import vcluster
 
+import colors
 import qa_config
 import qa_error
 
+from qa_logging import FormatInfo
 
-_INFO_SEQ = None
-_WARNING_SEQ = None
-_ERROR_SEQ = None
-_RESET_SEQ = None
 
 _MULTIPLEXERS = {}
 
@@ -70,67 +70,47 @@ _QA_OUTPUT = pathutils.GetLogFilename("qa-output")
  RETURN_VALUE) = range(1000, 1002)
 
 
-def _SetupColours():
-  """Initializes the colour constants.
+def _RaiseWithInfo(msg, error_desc):
+  """Raises a QA error with the given content, and adds a message if present.
 
   """
-  # pylint: disable=W0603
-  # due to global usage
-  global _INFO_SEQ, _WARNING_SEQ, _ERROR_SEQ, _RESET_SEQ
-
-  # Don't use colours if stdout isn't a terminal
-  if not sys.stdout.isatty():
-    return
-
-  try:
-    import curses
-  except ImportError:
-    # Don't use colours if curses module can't be imported
-    return
-
-  curses.setupterm()
-
-  _RESET_SEQ = curses.tigetstr("op")
-
-  setaf = curses.tigetstr("setaf")
-  _INFO_SEQ = curses.tparm(setaf, curses.COLOR_GREEN)
-  _WARNING_SEQ = curses.tparm(setaf, curses.COLOR_YELLOW)
-  _ERROR_SEQ = curses.tparm(setaf, curses.COLOR_RED)
+  if msg:
+    output = "%s: %s" % (msg, error_desc)
+  else:
+    output = error_desc
+  raise qa_error.Error(output)
 
 
-_SetupColours()
-
-
-def AssertIn(item, sequence):
+def AssertIn(item, sequence, msg=None):
   """Raises an error when item is not in sequence.
 
   """
   if item not in sequence:
-    raise qa_error.Error("%r not in %r" % (item, sequence))
+    _RaiseWithInfo(msg, "%r not in %r" % (item, sequence))
 
 
-def AssertNotIn(item, sequence):
+def AssertNotIn(item, sequence, msg=None):
   """Raises an error when item is in sequence.
 
   """
   if item in sequence:
-    raise qa_error.Error("%r in %r" % (item, sequence))
+    _RaiseWithInfo(msg, "%r in %r" % (item, sequence))
 
 
-def AssertEqual(first, second):
+def AssertEqual(first, second, msg=None):
   """Raises an error when values aren't equal.
 
   """
   if not first == second:
-    raise qa_error.Error("%r == %r" % (first, second))
+    _RaiseWithInfo(msg, "%r == %r" % (first, second))
 
 
-def AssertMatch(string, pattern):
+def AssertMatch(string, pattern, msg=None):
   """Raises an error when string doesn't match regexp pattern.
 
   """
   if not re.match(pattern, string):
-    raise qa_error.Error("%r doesn't match /%r/" % (string, pattern))
+    _RaiseWithInfo(msg, "%r doesn't match /%r/" % (string, pattern))
 
 
 def _GetName(entity, fn):
@@ -163,7 +143,7 @@ def _AssertRetCode(rcode, fail, cmdstr, nodename):
                          (cmdstr, nodename, rcode))
 
 
-def AssertCommand(cmd, fail=False, node=None, log_cmd=True):
+def AssertCommand(cmd, fail=False, node=None, log_cmd=True, max_seconds=None):
   """Checks that a remote command succeeds.
 
   @param cmd: either a string (the command to execute) or a list (to
@@ -175,6 +155,9 @@ def AssertCommand(cmd, fail=False, node=None, log_cmd=True):
       dict or a string)
   @param log_cmd: if False, the command won't be logged (simply passed to
       StartSSH)
+  @type max_seconds: double
+  @param max_seconds: fail if the command takes more than C{max_seconds}
+      seconds
   @return: the return code of the command
   @raise qa_error.Error: if the command fails when it shouldn't or vice versa
 
@@ -189,9 +172,17 @@ def AssertCommand(cmd, fail=False, node=None, log_cmd=True):
   else:
     cmdstr = utils.ShellQuoteArgs(cmd)
 
+  start = datetime.datetime.now()
   rcode = StartSSH(nodename, cmdstr, log_cmd=log_cmd).wait()
+  duration_seconds = TimedeltaToTotalSeconds(datetime.datetime.now() - start)
   _AssertRetCode(rcode, fail, cmdstr, nodename)
 
+  if max_seconds is not None:
+    if duration_seconds > max_seconds:
+      raise qa_error.Error(
+        "Cmd '%s' took %f seconds, maximum of %f was exceeded" %
+        (cmdstr, duration_seconds, max_seconds))
+
   return rcode
 
 
@@ -285,7 +276,8 @@ def StartLocalCommand(cmd, _nolog_opts=False, log_cmd=True, **kwargs):
       pcmd = [i for i in cmd if not i.startswith("-")]
     else:
       pcmd = cmd
-    print "Command: %s" % utils.ShellQuoteArgs(pcmd)
+    print "%s %s" % (colors.colorize("Command:", colors.CYAN),
+                     utils.ShellQuoteArgs(pcmd))
   return subprocess.Popen(cmd, shell=False, **kwargs)
 
 
@@ -577,17 +569,6 @@ def GenericQueryFieldsTest(cmd, fields):
               constants.EXIT_UNKNOWN_FIELD)
 
 
-def _FormatWithColor(text, seq):
-  if not seq:
-    return text
-  return "%s%s%s" % (seq, text, _RESET_SEQ)
-
-
-FormatWarning = lambda text: _FormatWithColor(text, _WARNING_SEQ)
-FormatError = lambda text: _FormatWithColor(text, _ERROR_SEQ)
-FormatInfo = lambda text: _FormatWithColor(text, _INFO_SEQ)
-
-
 def AddToEtcHosts(hostnames):
   """Adds hostnames to /etc/hosts.
 
@@ -904,3 +885,26 @@ def ParseIPolicy(policy):
     else:
       ret_policy[key] = val
   return (ret_policy, ret_specs)
+
+
+def UsesIPv6Connection(host, port):
+  """Returns True if the connection to a given host/port could go through IPv6.
+
+  """
+  return any(t[0] == socket.AF_INET6 for t in socket.getaddrinfo(host, port))
+
+
+def TimedeltaToTotalSeconds(td):
+  """Returns the total seconds in a C{datetime.timedelta} object.
+
+  This performs the same task as the C{datetime.timedelta.total_seconds()}
+  method which is present in Python 2.7 onwards.
+
+  @type td: datetime.timedelta
+  @param td: timedelta object to convert
+  @rtype float
+  @return: total seconds in the timedelta object
+
+  """
+  return ((td.microseconds + (td.seconds + td.days * 24.0 * 3600.0) * 10 ** 6) /
+          10 ** 6)
