@@ -32,10 +32,8 @@ module Ganeti.ConfigReader
 
 import Control.Concurrent
 import Control.Exception
-import Control.Monad (liftM, unless)
+import Control.Monad (unless)
 import Data.IORef
-import System.Posix.Files
-import System.Posix.Types
 import System.INotify
 
 import Ganeti.BasicTypes
@@ -51,12 +49,6 @@ import Ganeti.Utils
 -- executed.
 type ConfigReader = IO (Result ConfigData)
 
--- | File stat identifier.
-type FStat = (EpochTime, FileID, FileOffset)
-
--- | Null 'FStat' value.
-nullFStat :: FStat
-nullFStat = (-1, -1, -1)
 
 -- | Reload model data type.
 data ReloadModel = ReloadNotify      -- ^ We are using notifications
@@ -123,7 +115,8 @@ updateConfig path save_fn = do
   newcfg <- loadConfig path
   let !newdata = case newcfg of
                    Ok !cfg -> Ok cfg
-                   Bad _ -> Bad "Cannot load configuration"
+                   Bad msg -> Bad $ "Cannot load configuration from " ++ path
+                                    ++ ": " ++ msg
   save_fn newdata
   case newcfg of
     Ok cfg -> logInfo ("Loaded new config, serial " ++
@@ -150,27 +143,6 @@ safeUpdateConfig path oldfstat save_fn =
              return (nullFStat, ConfigIOError)
           )
 
--- | Computes the file cache data from a FileStatus structure.
-buildFileStatus :: FileStatus -> FStat
-buildFileStatus ofs =
-    let modt = modificationTime ofs
-        inum = fileID ofs
-        fsize = fileSize ofs
-    in (modt, inum, fsize)
-
--- | Wrapper over 'buildFileStatus'. This reads the data from the
--- filesystem and then builds our cache structure.
-getFStat :: FilePath -> IO FStat
-getFStat p = liftM buildFileStatus (getFileStatus p)
-
--- | Check if the file needs reloading
-needsReload :: FStat -> FilePath -> IO (Maybe FStat)
-needsReload oldstat path = do
-  newstat <- getFStat path
-  return $ if newstat /= oldstat
-             then Just newstat
-             else Nothing
-
 -- ** Watcher threads
 
 -- $watcher
@@ -196,7 +168,7 @@ onWatcherTimer :: IO Bool -> FilePath -> (Result ConfigData -> IO ())
                -> MVar ServerState -> IO ()
 onWatcherTimer inotiaction path save_fn state = do
   threadDelay watchInterval
-  logDebug "Watcher timer fired"
+  logDebug "Config-reader watcher timer fired"
   modifyMVar_ state (onWatcherInner path save_fn)
   _ <- inotiaction
   onWatcherTimer inotiaction path save_fn state
@@ -329,4 +301,4 @@ initConfigReader cfg_transform ioref = do
   -- fork the polling timer
   unless has_inotify $ do
     _ <- forkIO $ onPollTimer inotiaction conf_file save_fn statemvar
-    return ()
\ No newline at end of file
+    return ()
