@@ -36,6 +36,7 @@ module Ganeti.HTools.CLI
   , parseYesNo
   , parseISpecString
   , shTemplate
+  , maybeSaveCommands
   , maybePrintNodes
   , maybePrintInsts
   , maybeShowWarnings
@@ -48,7 +49,10 @@ module Ganeti.HTools.CLI
   , oDiskTemplate
   , oSpindleUse
   , oDynuFile
+  , oMonD
+  , oMonDDataFile
   , oEvacMode
+  , oRestrictedMigrate
   , oExInst
   , oExTags
   , oExecJobs
@@ -56,7 +60,10 @@ module Ganeti.HTools.CLI
   , oFullEvacuation
   , oGroup
   , oIAllocSrc
+  , oIgnoreDyn
   , oIgnoreNonRedundant
+  , oIndependentGroups
+  , oAcceptExisting
   , oInstMoves
   , oJobDelay
   , genOLuxiSocket
@@ -67,6 +74,7 @@ module Ganeti.HTools.CLI
   , oMinDisk
   , oMinGain
   , oMinGainLim
+  , oMinResources
   , oMinScore
   , oNoHeaders
   , oNoSimulation
@@ -89,6 +97,7 @@ module Ganeti.HTools.CLI
   , oShowComp
   , oSkipNonRedundant
   , oStdSpec
+  , oTargetResources
   , oTieredSpec
   , oVerbose
   , oPriority
@@ -121,7 +130,14 @@ data Options = Options
   , optDiskTemplate :: Maybe DiskTemplate  -- ^ Override for the disk template
   , optSpindleUse  :: Maybe Int      -- ^ Override for the spindle usage
   , optDynuFile    :: Maybe FilePath -- ^ Optional file with dynamic use data
+  , optIgnoreDynu  :: Bool           -- ^ Do not use dynamic use data
+  , optIndependentGroups :: Bool     -- ^ consider groups independently
+  , optAcceptExisting :: Bool        -- ^ accept existing N+1 violations
+  , optMonD        :: Bool           -- ^ Query MonDs
+  , optMonDFile    :: Maybe FilePath -- ^ Optional file with data provided
+                                     -- ^ by MonDs
   , optEvacMode    :: Bool           -- ^ Enable evacuation mode
+  , optRestrictedMigrate :: Bool     -- ^ Disallow replace-primary moves
   , optExInst      :: [String]       -- ^ Instances to be excluded
   , optExTags      :: Maybe [String] -- ^ Tags to use for exclusion
   , optExecJobs    :: Bool           -- ^ Execute the commands via Luxi
@@ -140,6 +156,7 @@ data Options = Options
   , optMdsk        :: Double         -- ^ Max disk usage ratio for nodes
   , optMinGain     :: Score          -- ^ Min gain we aim for in a step
   , optMinGainLim  :: Score          -- ^ Limit below which we apply mingain
+  , optMinResources :: Double        -- ^ Minimal resources for hsqueeze
   , optMinScore    :: Score          -- ^ The minimum score we aim for
   , optNoHeaders   :: Bool           -- ^ Do not show a header line
   , optNoSimulation :: Bool          -- ^ Skip the rebalancing dry-run
@@ -159,6 +176,7 @@ data Options = Options
   , optShowVer     :: Bool           -- ^ Just show the program version
   , optSkipNonRedundant :: Bool      -- ^ Skip nodes with non-redundant instance
   , optStdSpec     :: Maybe RSpec    -- ^ Requested standard specs
+  , optTargetResources :: Double     -- ^ Target resources for squeezing
   , optTestCount   :: Maybe Int      -- ^ Optional test count override
   , optTieredSpec  :: Maybe RSpec    -- ^ Requested specs for tiered mode
   , optReplay      :: Maybe String   -- ^ Unittests: RNG state
@@ -172,10 +190,16 @@ defaultOptions  = Options
   { optDataFile    = Nothing
   , optDiskMoves   = True
   , optInstMoves   = True
+  , optIndependentGroups = False
+  , optAcceptExisting = False
   , optDiskTemplate = Nothing
   , optSpindleUse  = Nothing
+  , optIgnoreDynu  = False
   , optDynuFile    = Nothing
+  , optMonD        = False
+  , optMonDFile = Nothing
   , optEvacMode    = False
+  , optRestrictedMigrate = False
   , optExInst      = []
   , optExTags      = Nothing
   , optExecJobs    = False
@@ -194,6 +218,7 @@ defaultOptions  = Options
   , optMdsk        = defReservedDiskRatio
   , optMinGain     = 1e-2
   , optMinGainLim  = 1e-1
+  , optMinResources = 2.0
   , optMinScore    = 1e-9
   , optNoHeaders   = False
   , optNoSimulation = False
@@ -213,6 +238,7 @@ defaultOptions  = Options
   , optShowNodes   = Nothing
   , optShowVer     = False
   , optStdSpec     = Nothing
+  , optTargetResources = 2.0
   , optTestCount   = Nothing
   , optTieredSpec  = Nothing
   , optReplay      = Nothing
@@ -277,6 +303,22 @@ oDiskMoves =
    \ thus allowing only the 'cheap' failover/migrate operations",
    OptComplNone)
 
+oMonD :: OptType
+oMonD =
+  (Option "" ["mond"]
+   (OptArg (\ f opts -> do
+              flag <- parseYesNo True f
+              return $ opts { optMonD = flag }) "CHOICE")
+   "pass either 'yes' or 'no' to query all monDs",
+   optComplYesNo)
+
+oMonDDataFile :: OptType
+oMonDDataFile =
+  (Option "" ["mond-data"]
+   (ReqArg (\ f opts -> Ok opts { optMonDFile = Just f }) "FILE")
+   "Import data provided by MonDs from the given FILE",
+   OptComplFile)
+
 oDiskTemplate :: OptType
 oDiskTemplate =
   (Option "" ["disk-template"]
@@ -320,6 +362,27 @@ oDynuFile =
    "Import dynamic utilisation data from the given FILE",
    OptComplFile)
 
+oIgnoreDyn :: OptType
+oIgnoreDyn =
+  (Option "" ["ignore-dynu"]
+   (NoArg (\ opts -> Ok opts {optIgnoreDynu = True}))
+   "Ignore any dynamic utilisation information",
+   OptComplNone)
+
+oIndependentGroups :: OptType
+oIndependentGroups =
+  (Option "" ["independent-groups"]
+   (NoArg (\ opts -> Ok opts {optIndependentGroups = True}))
+   "Consider groups independently",
+   OptComplNone)
+
+oAcceptExisting :: OptType
+oAcceptExisting =
+  (Option "" ["accept-existing-errors"]
+   (NoArg (\ opts -> Ok opts {optAcceptExisting = True}))
+   "Accept existing N+1 violations; just don't add new ones",
+   OptComplNone)
+
 oEvacMode :: OptType
 oEvacMode =
   (Option "E" ["evac-mode"]
@@ -328,6 +391,14 @@ oEvacMode =
    \ instances away from offline and drained nodes",
    OptComplNone)
 
+oRestrictedMigrate :: OptType
+oRestrictedMigrate =
+  (Option "" ["restricted-migration"]
+   (NoArg (\opts -> Ok opts { optRestrictedMigrate =  True }))
+   "disallow replace-primary moves (aka frf-moves); in evacuation mode, this\
+   \ will ensure that the only migrations are off the drained nodes",
+   OptComplNone)
+
 oExInst :: OptType
 oExInst =
   (Option "" ["exclude-instances"]
@@ -405,7 +476,7 @@ genOLuxiSocket defSocket =
    OptComplFile)
 
 oLuxiSocket :: IO OptType
-oLuxiSocket = liftM genOLuxiSocket Path.defaultLuxiSocket
+oLuxiSocket = liftM genOLuxiSocket Path.defaultMasterSocket
 
 oMachineReadable :: OptType
 oMachineReadable =
@@ -464,6 +535,15 @@ oMinGainLim =
    "minimum cluster score for which we start checking the min-gain",
    OptComplFloat)
 
+oMinResources :: OptType
+oMinResources =
+  (Option "" ["minimal-resources"]
+   (reqWithConversion (tryRead "minimal resources")
+    (\d opts -> Ok opts { optMinResources = d}) "FACTOR")
+   "minimal resources to be present on each in multiples of\ 
+   \ the standard allocation for not onlining standby nodes",
+   OptComplFloat)
+
 oMinScore :: OptType
 oMinScore =
   (Option "e" ["min-score"]
@@ -606,6 +686,15 @@ oStdSpec =
    "enable standard specs allocation, given as 'disk,ram,cpu'",
    OptComplString)
 
+oTargetResources :: OptType
+oTargetResources =
+  (Option "" ["target-resources"]
+   (reqWithConversion (tryRead "target resources")
+    (\d opts -> Ok opts { optTargetResources = d}) "FACTOR")
+   "target resources to be left on each node after squeezing in\
+   \ multiples of the standard allocation",
+   OptComplFloat)
+
 oTieredSpec :: OptType
 oTieredSpec =
   (Option "" ["tiered-alloc"]
@@ -665,6 +754,22 @@ shTemplate =
          \  fi\n\
          \}\n\n"
 
+-- | Optionally show or save a list of commands
+maybeSaveCommands :: String -- ^ Informal description
+                  -> Options
+                  -> String -- ^ commands
+                  -> IO ()
+maybeSaveCommands msg opts cmds =
+  case optShowCmds opts of
+    Nothing -> return ()
+    Just "-" -> do
+      putStrLn ""
+      putStrLn msg
+      putStr . unlines .  map ("  " ++) . filter (/= "  check") . lines $ cmds
+    Just out_path -> do
+      writeFile out_path (shTemplate ++ cmds)
+      printf "The commands have been written to file '%s'\n" out_path
+
 -- | Optionally print the node list.
 maybePrintNodes :: Maybe [String]       -- ^ The field list
                 -> String               -- ^ Informational message
