@@ -46,6 +46,7 @@ module Ganeti.HTools.Cluster
   , printSolutionLine
   , formatCmds
   , involvedNodes
+  , getMoves
   , splitJobs
   -- * Display functions
   , printNodes
@@ -94,7 +95,7 @@ import Ganeti.HTools.Types
 import Ganeti.Compat
 import qualified Ganeti.OpCodes as OpCodes
 import Ganeti.Utils
-import Ganeti.Types (mkNonEmpty)
+import Ganeti.Types (EvacMode(..), mkNonEmpty)
 
 -- * Types
 
@@ -318,70 +319,107 @@ computeAllocationDelta cini cfin =
                        }
   in (rini, rfin, runa)
 
+-- | The names and weights of the individual elements in the CV list, together
+-- with their statistical accumulation function and a bit to decide whether it
+-- is a statistics for online nodes.
+detailedCVInfoExt :: [((Double, String), ([Double] -> Statistics, Bool))]
+detailedCVInfoExt = [ ((1,  "free_mem_cv"), (getStdDevStatistics, True))
+                    , ((1,  "free_disk_cv"), (getStdDevStatistics, True))
+                    , ((1,  "n1_cnt"), (getSumStatistics, True))
+                    , ((1,  "reserved_mem_cv"), (getStdDevStatistics, True))
+                    , ((4,  "offline_all_cnt"), (getSumStatistics, False))
+                    , ((16, "offline_pri_cnt"), (getSumStatistics, False))
+                    , ((1,  "vcpu_ratio_cv"), (getStdDevStatistics, True))
+                    , ((1,  "cpu_load_cv"), (getStdDevStatistics, True))
+                    , ((1,  "mem_load_cv"), (getStdDevStatistics, True))
+                    , ((1,  "disk_load_cv"), (getStdDevStatistics, True))
+                    , ((1,  "net_load_cv"), (getStdDevStatistics, True))
+                    , ((2,  "pri_tags_score"), (getSumStatistics, True))
+                    , ((1,  "spindles_cv"), (getStdDevStatistics, True))
+                    ]
+
 -- | The names and weights of the individual elements in the CV list.
 detailedCVInfo :: [(Double, String)]
-detailedCVInfo = [ (1,  "free_mem_cv")
-                 , (1,  "free_disk_cv")
-                 , (1,  "n1_cnt")
-                 , (1,  "reserved_mem_cv")
-                 , (4,  "offline_all_cnt")
-                 , (16, "offline_pri_cnt")
-                 , (1,  "vcpu_ratio_cv")
-                 , (1,  "cpu_load_cv")
-                 , (1,  "mem_load_cv")
-                 , (1,  "disk_load_cv")
-                 , (1,  "net_load_cv")
-                 , (2,  "pri_tags_score")
-                 , (1,  "spindles_cv")
-                 ]
+detailedCVInfo = map fst detailedCVInfoExt
 
 -- | Holds the weights used by 'compCVNodes' for each metric.
 detailedCVWeights :: [Double]
 detailedCVWeights = map fst detailedCVInfo
 
--- | Compute the mem and disk covariance.
-compDetailedCV :: [Node.Node] -> [Double]
-compDetailedCV all_nodes =
+-- | The aggregation functions for the weights
+detailedCVAggregation :: [([Double] -> Statistics, Bool)]
+detailedCVAggregation = map snd detailedCVInfoExt
+
+-- | The bit vector describing which parts of the statistics are
+-- for online nodes.
+detailedCVOnlineStatus :: [Bool]
+detailedCVOnlineStatus = map snd detailedCVAggregation
+
+-- | Compute statistical measures of a single node.
+compDetailedCVNode :: Node.Node -> [Double]
+compDetailedCVNode node =
+  let mem = Node.pMem node
+      dsk = Node.pDsk node
+      n1 = fromIntegral
+           $ if Node.failN1 node
+               then length (Node.sList node) + length (Node.pList node)
+               else 0
+      res = Node.pRem node
+      ipri = fromIntegral . length $ Node.pList node
+      isec = fromIntegral . length $ Node.sList node
+      ioff = ipri + isec
+      cpu = Node.pCpu node
+      DynUtil c1 m1 d1 nn1 = Node.utilLoad node
+      DynUtil c2 m2 d2 nn2 = Node.utilPool node
+      (c_load, m_load, d_load, n_load) = (c1/c2, m1/m2, d1/d2, nn1/nn2)
+      pri_tags = fromIntegral $ Node.conflictingPrimaries node
+      spindles = Node.instSpindles node / Node.hiSpindles node
+  in [ mem, dsk, n1, res, ioff, ipri, cpu
+     , c_load, m_load, d_load, n_load
+     , pri_tags, spindles
+     ]
+
+-- | Compute the statistics of a cluster.
+compClusterStatistics :: [Node.Node] -> [Statistics]
+compClusterStatistics all_nodes =
   let (offline, nodes) = partition Node.offline all_nodes
-      mem_l = map Node.pMem nodes
-      dsk_l = map Node.pDsk nodes
-      -- metric: memory covariance
-      mem_cv = stdDev mem_l
-      -- metric: disk covariance
-      dsk_cv = stdDev dsk_l
-      -- metric: count of instances living on N1 failing nodes
-      n1_score = fromIntegral . sum . map (\n -> length (Node.sList n) +
-                                                 length (Node.pList n)) .
-                 filter Node.failN1 $ nodes :: Double
-      res_l = map Node.pRem nodes
-      -- metric: reserved memory covariance
-      res_cv = stdDev res_l
-      -- offline instances metrics
-      offline_ipri = sum . map (length . Node.pList) $ offline
-      offline_isec = sum . map (length . Node.sList) $ offline
-      -- metric: count of instances on offline nodes
-      off_score = fromIntegral (offline_ipri + offline_isec)::Double
-      -- metric: count of primary instances on offline nodes (this
-      -- helps with evacuation/failover of primary instances on
-      -- 2-node clusters with one node offline)
-      off_pri_score = fromIntegral offline_ipri::Double
-      cpu_l = map Node.pCpu nodes
-      -- metric: covariance of vcpu/pcpu ratio
-      cpu_cv = stdDev cpu_l
-      -- metrics: covariance of cpu, memory, disk and network load
-      (c_load, m_load, d_load, n_load) =
-        unzip4 $ map (\n ->
-                      let DynUtil c1 m1 d1 n1 = Node.utilLoad n
-                          DynUtil c2 m2 d2 n2 = Node.utilPool n
-                      in (c1/c2, m1/m2, d1/d2, n1/n2)) nodes
-      -- metric: conflicting instance count
-      pri_tags_inst = sum $ map Node.conflictingPrimaries nodes
-      pri_tags_score = fromIntegral pri_tags_inst::Double
-      -- metric: spindles %
-      spindles_cv = map (\n -> Node.instSpindles n / Node.hiSpindles n) nodes
-  in [ mem_cv, dsk_cv, n1_score, res_cv, off_score, off_pri_score, cpu_cv
-     , stdDev c_load, stdDev m_load , stdDev d_load, stdDev n_load
-     , pri_tags_score, stdDev spindles_cv ]
+      offline_values = transpose (map compDetailedCVNode offline)
+                       ++ repeat []
+      -- transpose of an empty list is empty and not k times the empty list, as
+      -- would be the transpose of a 0 x k matrix
+      online_values = transpose $ map compDetailedCVNode nodes
+      aggregate (f, True) (onNodes, _) = f onNodes
+      aggregate (f, False) (_, offNodes) = f offNodes
+  in zipWith aggregate detailedCVAggregation
+       $ zip online_values offline_values
+
+-- | Update a cluster statistics by replacing the contribution of one
+-- node by that of another.
+updateClusterStatistics :: [Statistics]
+                           -> (Node.Node, Node.Node) -> [Statistics]
+updateClusterStatistics stats (old, new) =
+  let update = zip (compDetailedCVNode old) (compDetailedCVNode new)
+      online = not $ Node.offline old
+      updateStat forOnline stat upd = if forOnline == online
+                                        then updateStatistics stat upd
+                                        else stat
+  in zipWith3 updateStat detailedCVOnlineStatus stats update
+
+-- | Update a cluster statistics twice.
+updateClusterStatisticsTwice :: [Statistics]
+                                -> (Node.Node, Node.Node)
+                                -> (Node.Node, Node.Node)
+                                -> [Statistics]
+updateClusterStatisticsTwice s a =
+  updateClusterStatistics (updateClusterStatistics s a)
+
+-- | Compute cluster statistics
+compDetailedCV :: [Node.Node] -> [Double]
+compDetailedCV = map getStatisticValue . compClusterStatistics
+
+-- | Compute the cluster score from its statistics
+compCVfromStats :: [Statistics] -> Double
+compCVfromStats = sum . zipWith (*) detailedCVWeights . map getStatisticValue
 
 -- | Compute the /total/ variance.
 compCVNodes :: [Node.Node] -> Double
@@ -510,9 +548,10 @@ allocateOnSingle nl inst new_pdx =
     return (new_nl, new_inst, [new_p], new_score)
 
 -- | Tries to allocate an instance on a given pair of nodes.
-allocateOnPair :: Node.List -> Instance.Instance -> Ndx -> Ndx
+allocateOnPair :: [Statistics]
+               -> Node.List -> Instance.Instance -> Ndx -> Ndx
                -> OpResult Node.AllocElement
-allocateOnPair nl inst new_pdx new_sdx =
+allocateOnPair stats nl inst new_pdx new_sdx =
   let tgt_p = Container.find new_pdx nl
       tgt_s = Container.find new_sdx nl
   in do
@@ -522,7 +561,9 @@ allocateOnPair nl inst new_pdx new_sdx =
     new_s <- Node.addSec tgt_s inst new_pdx
     let new_inst = Instance.setBoth inst new_pdx new_sdx
         new_nl = Container.addTwo new_pdx new_p new_sdx new_s nl
-    return (new_nl, new_inst, [new_p, new_s], compCV new_nl)
+        new_stats = updateClusterStatisticsTwice stats
+                      (tgt_p, new_p) (tgt_s, new_s)
+    return (new_nl, new_inst, [new_p, new_s], compCVfromStats new_stats)
 
 -- | Tries to perform an instance move and returns the best table
 -- between the original one and the new one.
@@ -550,27 +591,39 @@ checkSingleStep ini_tbl target cur_tbl move =
 possibleMoves :: MirrorType -- ^ The mirroring type of the instance
               -> Bool       -- ^ Whether the secondary node is a valid new node
               -> Bool       -- ^ Whether we can change the primary node
+              -> (Bool, Bool) -- ^ Whether migration is restricted and whether
+                              -- the instance primary is offline
               -> Ndx        -- ^ Target node candidate
               -> [IMove]    -- ^ List of valid result moves
 
-possibleMoves MirrorNone _ _ _ = []
+possibleMoves MirrorNone _ _ _ _ = []
 
-possibleMoves MirrorExternal _ False _ = []
+possibleMoves MirrorExternal _ False _  _ = []
 
-possibleMoves MirrorExternal _ True tdx =
+possibleMoves MirrorExternal _ True _ tdx =
   [ FailoverToAny tdx ]
 
-possibleMoves MirrorInternal _ False tdx =
+possibleMoves MirrorInternal _ False _ tdx =
   [ ReplaceSecondary tdx ]
 
-possibleMoves MirrorInternal True True tdx =
+possibleMoves MirrorInternal _ _ (True, False) tdx =
+  [ ReplaceSecondary tdx
+  ]
+
+possibleMoves MirrorInternal True True (False, _) tdx =
   [ ReplaceSecondary tdx
   , ReplaceAndFailover tdx
   , ReplacePrimary tdx
   , FailoverAndReplace tdx
   ]
 
-possibleMoves MirrorInternal False True tdx =
+possibleMoves MirrorInternal True True (True, True) tdx =
+  [ ReplaceSecondary tdx
+  , ReplaceAndFailover tdx
+  , FailoverAndReplace tdx
+  ]
+
+possibleMoves MirrorInternal False True _ tdx =
   [ ReplaceSecondary tdx
   , ReplaceAndFailover tdx
   ]
@@ -579,10 +632,12 @@ possibleMoves MirrorInternal False True tdx =
 checkInstanceMove :: [Ndx]             -- ^ Allowed target node indices
                   -> Bool              -- ^ Whether disk moves are allowed
                   -> Bool              -- ^ Whether instance moves are allowed
+                  -> Bool              -- ^ Whether migration is restricted
                   -> Table             -- ^ Original table
                   -> Instance.Instance -- ^ Instance to move
                   -> Table             -- ^ Best new table for this instance
-checkInstanceMove nodes_idx disk_moves inst_moves ini_tbl target =
+checkInstanceMove nodes_idx disk_moves inst_moves rest_mig
+                  ini_tbl@(Table nl _ _ _) target =
   let opdx = Instance.pNode target
       osdx = Instance.sNode target
       bad_nodes = [opdx, osdx]
@@ -593,9 +648,13 @@ checkInstanceMove nodes_idx disk_moves inst_moves ini_tbl target =
                        -- if drbd and allowed to failover
                        then checkSingleStep ini_tbl target ini_tbl Failover
                        else ini_tbl
+      primary_drained = Node.offline
+                        . flip Container.find nl
+                        $ Instance.pNode target
       all_moves =
         if disk_moves
-          then concatMap (possibleMoves mir_type use_secondary inst_moves)
+          then concatMap (possibleMoves mir_type use_secondary inst_moves
+                          (rest_mig, primary_drained))
                nodes
           else []
     in
@@ -606,10 +665,11 @@ checkInstanceMove nodes_idx disk_moves inst_moves ini_tbl target =
 checkMove :: [Ndx]               -- ^ Allowed target node indices
           -> Bool                -- ^ Whether disk moves are allowed
           -> Bool                -- ^ Whether instance moves are allowed
+          -> Bool                -- ^ Whether migration is restricted
           -> Table               -- ^ The current solution
           -> [Instance.Instance] -- ^ List of instances still to move
           -> Table               -- ^ The new solution
-checkMove nodes_idx disk_moves inst_moves ini_tbl victims =
+checkMove nodes_idx disk_moves inst_moves rest_mig ini_tbl victims =
   let Table _ _ _ ini_plc = ini_tbl
       -- we're using rwhnf from the Control.Parallel.Strategies
       -- package; we don't need to use rnf as that would force too
@@ -617,7 +677,7 @@ checkMove nodes_idx disk_moves inst_moves ini_tbl victims =
       -- multi-threaded case the weak head normal form is enough to
       -- spark the evaluation
       tables = parMap rwhnf (checkInstanceMove nodes_idx disk_moves
-                             inst_moves ini_tbl)
+                             inst_moves rest_mig ini_tbl)
                victims
       -- iterate over all instances, computing the best move
       best_tbl = foldl' compareTables ini_tbl tables
@@ -641,10 +701,11 @@ tryBalance :: Table       -- ^ The starting table
            -> Bool        -- ^ Allow disk moves
            -> Bool        -- ^ Allow instance moves
            -> Bool        -- ^ Only evacuate moves
+           -> Bool        -- ^ Restrict migration
            -> Score       -- ^ Min gain threshold
            -> Score       -- ^ Min gain
            -> Maybe Table -- ^ The resulting table and commands
-tryBalance ini_tbl disk_moves inst_moves evac_mode mg_limit min_gain =
+tryBalance ini_tbl disk_moves inst_moves evac_mode rest_mig mg_limit min_gain =
     let Table ini_nl ini_il ini_cv _ = ini_tbl
         all_inst = Container.elems ini_il
         all_nodes = Container.elems ini_nl
@@ -657,7 +718,8 @@ tryBalance ini_tbl disk_moves inst_moves evac_mode mg_limit min_gain =
         reloc_inst = filter (\i -> Instance.movable i &&
                                    Instance.autoBalance i) all_inst'
         node_idx = map Node.idx online_nodes
-        fin_tbl = checkMove node_idx disk_moves inst_moves ini_tbl reloc_inst
+        fin_tbl = checkMove node_idx disk_moves inst_moves rest_mig
+                            ini_tbl reloc_inst
         (Table _ _ fin_cv _) = fin_tbl
     in
       if fin_cv < ini_cv && (ini_cv > mg_limit || ini_cv - fin_cv >= min_gain)
@@ -772,10 +834,11 @@ tryAlloc :: (Monad m) =>
          -> m AllocSolution   -- ^ Possible solution list
 tryAlloc _  _ _    (Right []) = fail "Not enough online nodes"
 tryAlloc nl _ inst (Right ok_pairs) =
-  let psols = parMap rwhnf (\(p, ss) ->
+  let cstat = compClusterStatistics $ Container.elems nl
+      psols = parMap rwhnf (\(p, ss) ->
                               foldl' (\cstate ->
                                         concatAllocs cstate .
-                                        allocateOnPair nl inst p)
+                                        allocateOnPair cstat nl inst p)
                               emptyAllocSolution ss) ok_pairs
       sols = foldl' sumAllocs emptyAllocSolution psols
   in return $ annotateSolution sols
@@ -981,6 +1044,12 @@ nodeEvacInstance nl il mode inst@(Instance.Instance
                    failOnSecondaryChange mode dt >>
                    evacOneNodeOnly nl il inst gdx avail_nodes
 
+nodeEvacInstance nl il mode inst@(Instance.Instance
+                                  {Instance.diskTemplate = dt@DTGluster})
+                 gdx avail_nodes =
+                   failOnSecondaryChange mode dt >>
+                   evacOneNodeOnly nl il inst gdx avail_nodes
+
 nodeEvacInstance nl il ChangePrimary
                  inst@(Instance.Instance {Instance.diskTemplate = DTDrbd8})
                  _ _ =
@@ -1328,14 +1397,20 @@ tieredAlloc nl il limit newinst allocnodes ixes cstats =
                                           . flip (tryAlloc nl' il') allocnodes)
                        newinst
           bigSteps = filter isJust . map suffShrink . reverse $ sortedErrs
+          progress (Ok (_, _, _, newil', _)) (Ok (_, _, _, newil, _)) =
+            length newil' > length newil
+          progress _ _ = False
       in if stop then newsol else
-          case bigSteps of
-            Just newinst':_ -> tieredAlloc nl' il' newlimit
-                               newinst' allocnodes ixes' cstats'
-            _ -> case Instance.shrinkByType newinst . last $ sortedErrs of
-                   Bad _ -> newsol
-                   Ok newinst' -> tieredAlloc nl' il' newlimit
-                                  newinst' allocnodes ixes' cstats'
+           let newsol' = case Instance.shrinkByType newinst . last
+                                $ sortedErrs of
+                 Bad _ -> newsol
+                 Ok newinst' -> tieredAlloc nl' il' newlimit
+                                newinst' allocnodes ixes' cstats'
+           in if progress newsol' newsol then newsol' else
+                case bigSteps of
+                  Just newinst':_ -> tieredAlloc nl' il' newlimit
+                                     newinst' allocnodes ixes' cstats'
+                  _ -> newsol
 
 -- * Formatting functions
 
@@ -1410,8 +1485,23 @@ involvedNodes :: Instance.List -- ^ Instance list, used for retrieving
 involvedNodes il plc =
   let (i, np, ns, _, _) = plc
       inst = Container.find i il
-  in nub $ [np, ns] ++ Instance.allNodes inst
+  in nub . filter (>= 0) $ [np, ns] ++ Instance.allNodes inst
 
+-- | From two adjacent cluster tables get the list of moves that transitions
+-- from to the other
+getMoves :: (Table, Table) -> [MoveJob]
+getMoves (Table _ initial_il _ initial_plc, Table final_nl _ _ final_plc) =
+  let
+    plctoMoves (plc@(idx, p, s, mv, _)) =
+      let inst = Container.find idx initial_il
+          inst_name = Instance.name inst
+          affected = involvedNodes initial_il plc
+          np = Node.alias $ Container.find p final_nl
+          ns = Node.alias $ Container.find s final_nl
+          (_, cmds) = computeMoves inst inst_name mv np ns
+      in (affected, idx, mv, cmds)
+  in map plctoMoves . reverse . drop (length initial_plc) $ reverse final_plc
+             
 -- | Inner function for splitJobs, that either appends the next job to
 -- the current jobset, or starts a new jobset.
 mergeJobs :: ([JobSet], [Ndx]) -> MoveJob -> ([JobSet], [Ndx])
