@@ -268,12 +268,12 @@ commitChange client instData = do
   when (isJust arData) $ do
     let tag = arTag $ fromJust arData
     putStrLn (">>> Adding the following tag to " ++ iname ++ ":\n" ++ show tag)
-    execJobsWaitOk' [OpTagsSet (TagInstance iname) [tag]]
+    execJobsWaitOk' [OpTagsSet TagKindInstance [tag] (Just iname)]
 
   unless (null rmTags) $ do
     putStr (">>> Removing the following tags from " ++ iname ++ ":\n" ++
             unlines (map show rmTags))
-    execJobsWaitOk' [OpTagsDel (TagInstance iname) rmTags]
+    execJobsWaitOk' [OpTagsDel TagKindInstance rmTags (Just iname)]
 
   return instData { tagsToRemove = [] }
 
@@ -423,6 +423,7 @@ doRepair client delay instData (rtype, opcodes) =
                             , opDelayOnNodes = []
                             , opDelayOnNodeUuids = Nothing
                             , opDelayRepeat = fromJust $ mkNonNegative 0
+                            , opDelayNoLocks = False
                             } : opcodes
               else
                 opcodes
@@ -453,7 +454,7 @@ main opts args = do
   unless (null args) $
     exitErr "this program doesn't take any arguments."
 
-  luxiDef <- Path.defaultLuxiSocket
+  luxiDef <- Path.defaultMasterSocket
   let master = fromMaybe luxiDef $ optLuxi opts
       opts' = opts { optLuxi = Just master }
 
@@ -463,7 +464,7 @@ main opts args = do
   iniData <- exitIfBad "when parsing auto-repair tags" iniDataRes
 
   -- First step: check all pending repairs, see if they are completed.
-  iniData' <- bracket (L.getClient master) L.closeClient $
+  iniData' <- bracket (L.getLuxiClient master) L.closeClient $
               forM iniData . processPending
 
   -- Second step: detect any problems.
@@ -476,7 +477,7 @@ main opts args = do
                             ArHealthy _ -> doRepair c jobDelay i
                             _           -> const (return i)
 
-  repairDone <- bracket (L.getClient master) L.closeClient $
+  repairDone <- bracket (L.getLuxiClient master) L.closeClient $
                 forM (zip iniData' repairs) . maybeRepair
 
   -- Print some stats and exit.
