@@ -36,18 +36,22 @@ import Control.Monad
 import Control.Monad.IO.Class
 import Data.ByteString.Char8 hiding (map, filter, find)
 import Data.List
+import qualified Data.Map as Map
 import Snap.Core
 import Snap.Http.Server
 import qualified Text.JSON as J
+import Control.Concurrent
 
 import qualified Ganeti.BasicTypes as BT
 import Ganeti.Daemon
+import qualified Ganeti.DataCollectors.CPUload as CPUload
 import qualified Ganeti.DataCollectors.Diskstats as Diskstats
 import qualified Ganeti.DataCollectors.Drbd as Drbd
 import qualified Ganeti.DataCollectors.InstStatus as InstStatus
 import qualified Ganeti.DataCollectors.Lv as Lv
 import Ganeti.DataCollectors.Types
 import qualified Ganeti.Constants as C
+import Ganeti.Runtime
 
 -- * Types and constants definitions
 
@@ -59,7 +63,11 @@ type PrepResult = Config Snap ()
 
 -- | Version of the latest supported http API.
 latestAPIVersion :: Int
-latestAPIVersion = 1
+latestAPIVersion = C.mondLatestApiVersion
+
+-- | A report of a data collector might be stateful or stateless.
+data Report = StatelessR (IO DCReport)
+            | StatefulR (Maybe CollectorData -> IO DCReport)
 
 -- | Type describing a data collector basic information
 data DataCollector = DataCollector
@@ -68,28 +76,35 @@ data DataCollector = DataCollector
                                   --   of the collector
   , dKind     :: DCKind           -- ^ Kind (performance or status reporting) of
                                   --   the data collector
-  , dReport   :: IO DCReport      -- ^ Report produced by the collector
+  , dReport   :: Report           -- ^ Report produced by the collector
+  , dUpdate   :: Maybe (Maybe CollectorData -> IO CollectorData)
+                                  -- ^ Update operation for stateful collectors.
   }
 
+
 -- | The list of available builtin data collectors.
 collectors :: [DataCollector]
 collectors =
   [ DataCollector Diskstats.dcName Diskstats.dcCategory Diskstats.dcKind
-      Diskstats.dcReport
-  , DataCollector Drbd.dcName Drbd.dcCategory Drbd.dcKind Drbd.dcReport
+      (StatelessR Diskstats.dcReport) Nothing
+  , DataCollector Drbd.dcName Drbd.dcCategory Drbd.dcKind
+      (StatelessR Drbd.dcReport) Nothing
   , DataCollector InstStatus.dcName InstStatus.dcCategory InstStatus.dcKind
-      InstStatus.dcReport
-  , DataCollector Lv.dcName Lv.dcCategory Lv.dcKind Lv.dcReport
+      (StatelessR InstStatus.dcReport) Nothing
+  , DataCollector Lv.dcName Lv.dcCategory Lv.dcKind
+      (StatelessR Lv.dcReport) Nothing
+  , DataCollector CPUload.dcName CPUload.dcCategory CPUload.dcKind
+      (StatefulR CPUload.dcReport) (Just CPUload.dcUpdate)
   ]
 
 -- * Configuration handling
 
 -- | The default configuration for the HTTP server.
-defaultHttpConf :: Config Snap ()
-defaultHttpConf =
-  setAccessLog (ConfigFileLog C.daemonsExtraLogfilesGanetiMondAccess) .
+defaultHttpConf :: FilePath -> FilePath -> Config Snap ()
+defaultHttpConf accessLog errorLog =
+  setAccessLog (ConfigFileLog accessLog) .
   setCompression False .
-  setErrorLog (ConfigFileLog C.daemonsExtraLogfilesGanetiMondError) $
+  setErrorLog (ConfigFileLog errorLog) $
   setVerbose False
   emptyConfig
 
@@ -101,10 +116,14 @@ checkMain _ = return $ Right ()
 
 -- | Prepare function for monitoring agent.
 prepMain :: PrepFn CheckResult PrepResult
-prepMain opts _ =
-  return $
-    setPort (maybe C.defaultMondPort fromIntegral (optPort opts))
-      defaultHttpConf
+prepMain opts _ = do
+  accessLog <- daemonsExtraLogFile GanetiMond AccessLog
+  errorLog <- daemonsExtraLogFile GanetiMond ErrorLog
+  return .
+    setPort
+      (maybe C.defaultMondPort fromIntegral (optPort opts)) .
+    maybe id (setBind . pack) (optBindAddress opts)
+    $ defaultHttpConf accessLog errorLog
 
 -- * Query answers
 
@@ -113,13 +132,13 @@ versionQ :: Snap ()
 versionQ = writeBS . pack $ J.encode [latestAPIVersion]
 
 -- | Version 1 of the monitoring HTTP API.
-version1Api :: Snap ()
-version1Api =
+version1Api :: MVar CollectorMap -> Snap ()
+version1Api mvar =
   let returnNull = writeBS . pack $ J.encode J.JSNull :: Snap ()
   in ifTop returnNull <|>
      route
        [ ("list", listHandler)
-       , ("report", reportHandler)
+       , ("report", reportHandler mvar)
        ]
 
 -- | Get the JSON representation of a data collector to be used in the collector
@@ -138,20 +157,36 @@ listHandler =
   dir "collectors" . writeBS . pack . J.encode $ map dcListItem collectors
 
 -- | Handler for returning data collector reports.
-reportHandler :: Snap ()
-reportHandler =
+reportHandler :: MVar CollectorMap -> Snap ()
+reportHandler mvar =
   route
-    [ ("all", allReports)
-    , (":category/:collector", oneReport)
+    [ ("all", allReports mvar)
+    , (":category/:collector", oneReport mvar)
     ] <|>
   errorReport
 
 -- | Return the report of all the available collectors.
-allReports :: Snap ()
-allReports = do
-  reports <- mapM (liftIO . dReport) collectors
+allReports :: MVar CollectorMap -> Snap ()
+allReports mvar = do
+  reports <- mapM (liftIO . getReport mvar) collectors
   writeBS . pack . J.encode $ reports
 
+-- | Takes the CollectorMap and a DataCollector and returns the report for this
+-- collector.
+getReport :: MVar CollectorMap -> DataCollector -> IO DCReport
+getReport mvar collector =
+  case dReport collector of
+    StatelessR r -> r
+    StatefulR r -> do
+      colData <- getColData (dName collector) mvar
+      r colData
+
+-- | Returns the data for the corresponding collector.
+getColData :: String -> MVar CollectorMap -> IO (Maybe CollectorData)
+getColData name mvar = do
+  m <- readMVar mvar
+  return $ Map.lookup name m
+
 -- | Returns a category given its name.
 -- If "collector" is given as the name, the collector has no category, and
 -- Nothing will be returned.
@@ -173,11 +208,11 @@ error404 = do
   modifyResponse $ setResponseStatus 404 "Not found"
   writeBS "Resource not found"
 
--- | Return the report of one collector
-oneReport :: Snap ()
-oneReport = do
-  categoryName <- fmap (maybe mzero unpack) $ getParam "category"
-  collectorName <- fmap (maybe mzero unpack) $ getParam "collector"
+-- | Return the report of one collector.
+oneReport :: MVar CollectorMap -> Snap ()
+oneReport mvar = do
+  categoryName <- maybe mzero unpack <$> getParam "category"
+  collectorName <- maybe mzero unpack <$> getParam "collector"
   category <-
     case catFromName categoryName of
       BT.Ok cat -> return cat
@@ -188,17 +223,45 @@ oneReport = do
         filter (\c -> category == dCategory c) collectors of
       Just col -> return col
       Nothing -> fail "Unable to find the requested collector"
-  report <- liftIO $ dReport collector
-  writeBS . pack . J.encode $ report
+  dcr <- liftIO $ getReport mvar collector
+  writeBS . pack . J.encode $ dcr
 
 -- | The function implementing the HTTP API of the monitoring agent.
-monitoringApi :: Snap ()
-monitoringApi =
+monitoringApi :: MVar CollectorMap -> Snap ()
+monitoringApi mvar =
   ifTop versionQ <|>
-  dir "1" version1Api <|>
+  dir "1" (version1Api mvar) <|>
   error404
 
+-- | The function collecting data for each data collector providing a dcUpdate
+-- function.
+collect :: CollectorMap -> DataCollector -> IO CollectorMap
+collect m collector =
+  case dUpdate collector of
+    Nothing -> return m
+    Just update -> do
+      let name = dName collector
+          existing = Map.lookup name m
+      new_data <- update existing
+      return $ Map.insert name new_data m
+
+-- | Invokes collect for each data collector.
+collection :: CollectorMap -> IO CollectorMap
+collection m = foldM collect m collectors
+
+-- | The thread responsible for the periodical collection of data for each data
+-- data collector.
+collectord :: MVar CollectorMap -> IO ()
+collectord mvar = do
+  m <- takeMVar mvar
+  m' <- collection m
+  putMVar mvar m'
+  threadDelay $ 10^(6 :: Int) * C.mondTimeInterval
+  collectord mvar
+
 -- | Main function.
 main :: MainFn CheckResult PrepResult
-main _ _ httpConf =
-  httpServe httpConf $ method GET monitoringApi
+main _ _ httpConf = do
+  mvar <- newMVar Map.empty
+  _ <- forkIO $ collectord mvar
+  httpServe httpConf . method GET $ monitoringApi mvar
