@@ -29,9 +29,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 -}
 
 module Ganeti.Objects
-  ( VType(..)
-  , vTypeFromRaw
-  , HvParams
+  ( HvParams
   , OsParams
   , PartialNicParams(..)
   , FilledNicParams(..)
@@ -39,8 +37,6 @@ module Ganeti.Objects
   , allNicParamFields
   , PartialNic(..)
   , FileDriver(..)
-  , BlockDriver(..)
-  , DiskMode(..)
   , DiskLogicalId(..)
   , Disk(..)
   , includesLogicalId
@@ -49,18 +45,14 @@ module Ganeti.Objects
   , FilledBeParams(..)
   , fillBeParams
   , allBeParamFields
-  , AdminState(..)
-  , adminStateFromRaw
   , Instance(..)
   , toDictInstance
+  , getDiskSizeRequirements
   , PartialNDParams(..)
   , FilledNDParams(..)
   , fillNDParams
   , allNDParamFields
   , Node(..)
-  , NodeRole(..)
-  , nodeRoleToRaw
-  , roleDescription
   , AllocPolicy(..)
   , FilledISpecParams(..)
   , PartialISpecParams(..)
@@ -93,6 +85,7 @@ module Ganeti.Objects
   , Ip4Network(..)
   , readIp4Address
   , nextIp4Address
+  , IAllocatorParams
   ) where
 
 import Control.Applicative
@@ -104,11 +97,13 @@ import Data.Word
 import Text.JSON (showJSON, readJSON, JSON, JSValue(..), fromJSString)
 import qualified Text.JSON as J
 
+import qualified AutoConf
 import qualified Ganeti.Constants as C
+import qualified Ganeti.ConstantUtils as ConstantUtils
 import Ganeti.JSON
 import Ganeti.Types
 import Ganeti.THH
-import Ganeti.Utils (sepSplit, tryRead)
+import Ganeti.Utils (sepSplit, tryRead, parseUnitAssumeBinary)
 
 -- * Generic definitions
 
@@ -119,16 +114,6 @@ fillDict defaults custom skip_keys =
   let updated = Map.union custom defaults
   in foldl' (flip Map.delete) updated skip_keys
 
--- | The VTYPES, a mini-type system in Python.
-$(declareSADT "VType"
-  [ ("VTypeString",      'C.vtypeString)
-  , ("VTypeMaybeString", 'C.vtypeMaybeString)
-  , ("VTypeBool",        'C.vtypeBool)
-  , ("VTypeSize",        'C.vtypeSize)
-  , ("VTypeInt",         'C.vtypeInt)
-  ])
-$(makeJSONInstance ''VType)
-
 -- | The hypervisor parameter type. This is currently a simple map,
 -- without type checking on key/value pairs.
 type HvParams = Container JSValue
@@ -155,25 +140,6 @@ class SerialNoObject a where
 class TagsObject a where
   tagsOf :: a -> Set.Set String
 
--- * Node role object
-
-$(declareSADT "NodeRole"
-  [ ("NROffline",   'C.nrOffline)
-  , ("NRDrained",   'C.nrDrained)
-  , ("NRRegular",   'C.nrRegular)
-  , ("NRCandidate", 'C.nrMcandidate)
-  , ("NRMaster",    'C.nrMaster)
-  ])
-$(makeJSONInstance ''NodeRole)
-
--- | The description of the node role.
-roleDescription :: NodeRole -> String
-roleDescription NROffline   = "offline"
-roleDescription NRDrained   = "drained"
-roleDescription NRRegular   = "regular"
-roleDescription NRCandidate = "master candidate"
-roleDescription NRMaster    = "master"
-
 -- * Network definitions
 
 -- ** Ipv4 types
@@ -281,6 +247,7 @@ instance TimeStampObject Network where
 $(buildParam "Nic" "nicp"
   [ simpleField "mode" [t| NICMode |]
   , simpleField "link" [t| String  |]
+  , simpleField "vlan" [t| String |]
   ])
 
 $(buildObject "PartialNic" "nic" $
@@ -296,18 +263,6 @@ instance UuidObject PartialNic where
 
 -- * Disk definitions
 
-$(declareSADT "DiskMode"
-  [ ("DiskRdOnly", 'C.diskRdonly)
-  , ("DiskRdWr",   'C.diskRdwr)
-  ])
-$(makeJSONInstance ''DiskMode)
-
--- | The persistent block driver type. Currently only one type is allowed.
-$(declareSADT "BlockDriver"
-  [ ("BlockDrvManual", 'C.blockdevDriverManual)
-  ])
-$(makeJSONInstance ''BlockDriver)
-
 -- | Constant for the dev_type key entry in the disk config.
 devType :: String
 devType = "dev_type"
@@ -398,6 +353,13 @@ decodeDLId obj lid = do
           path'   <- readJSON path
           return $ LIDSharedFile driver' path'
         _ -> fail "Can't read logical_id for shared file type"
+    DTGluster ->
+      case lid of
+        JSArray [driver, path] -> do
+          driver' <- readJSON driver
+          path'   <- readJSON path
+          return $ LIDSharedFile driver' path'
+        _ -> fail "Can't read logical_id for shared file type"
     DTBlock ->
       case lid of
         JSArray [driver, path] -> do
@@ -428,7 +390,6 @@ decodeDLId obj lid = do
 -- code currently can't build it.
 data Disk = Disk
   { diskLogicalId  :: DiskLogicalId
---  , diskPhysicalId :: String
   , diskChildren   :: [Disk]
   , diskIvName     :: String
   , diskSize       :: Int
@@ -441,7 +402,6 @@ data Disk = Disk
 $(buildObjectSerialisation "Disk" $
   [ customField 'decodeDLId 'encodeFullDLId ["dev_type"] $
       simpleField "logical_id"    [t| DiskLogicalId   |]
---  , simpleField "physical_id" [t| String   |]
   , defaultField  [| [] |] $ simpleField "children" [t| [Disk] |]
   , defaultField [| "" |] $ simpleField "iv_name" [t| String |]
   , simpleField "size" [t| Int |]
@@ -465,36 +425,33 @@ includesLogicalId vg_name lv_name disk =
       any (includesLogicalId vg_name lv_name) $ diskChildren disk
     _ -> False
 
-
 -- * Instance definitions
 
-$(declareSADT "AdminState"
-  [ ("AdminOffline", 'C.adminstOffline)
-  , ("AdminDown",    'C.adminstDown)
-  , ("AdminUp",      'C.adminstUp)
-  ])
-$(makeJSONInstance ''AdminState)
-
 $(buildParam "Be" "bep"
-  [ simpleField "minmem"       [t| Int  |]
-  , simpleField "maxmem"       [t| Int  |]
-  , simpleField "vcpus"        [t| Int  |]
-  , simpleField "auto_balance" [t| Bool |]
+  [ specialNumericalField 'parseUnitAssumeBinary
+      $ simpleField "minmem"      [t| Int  |]
+  , specialNumericalField 'parseUnitAssumeBinary
+      $ simpleField "maxmem"      [t| Int  |]
+  , simpleField "vcpus"           [t| Int  |]
+  , simpleField "auto_balance"    [t| Bool |]
+  , simpleField "always_failover" [t| Bool |]
+  , simpleField "spindle_use"     [t| Int  |]
   ])
 
 $(buildObject "Instance" "inst" $
-  [ simpleField "name"           [t| String             |]
-  , simpleField "primary_node"   [t| String             |]
-  , simpleField "os"             [t| String             |]
-  , simpleField "hypervisor"     [t| Hypervisor         |]
-  , simpleField "hvparams"       [t| HvParams           |]
-  , simpleField "beparams"       [t| PartialBeParams    |]
-  , simpleField "osparams"       [t| OsParams           |]
-  , simpleField "admin_state"    [t| AdminState         |]
-  , simpleField "nics"           [t| [PartialNic]       |]
-  , simpleField "disks"          [t| [Disk]             |]
-  , simpleField "disk_template"  [t| DiskTemplate       |]
-  , simpleField "disks_active"   [t| Bool               |]
+  [ simpleField "name"               [t| String             |]
+  , simpleField "primary_node"       [t| String             |]
+  , simpleField "os"                 [t| String             |]
+  , simpleField "hypervisor"         [t| Hypervisor         |]
+  , simpleField "hvparams"           [t| HvParams           |]
+  , simpleField "beparams"           [t| PartialBeParams    |]
+  , simpleField "osparams"           [t| OsParams           |]
+  , simpleField "admin_state"        [t| AdminState         |]
+  , simpleField "admin_state_source" [t| AdminStateSource   |]
+  , simpleField "nics"               [t| [PartialNic]       |]
+  , simpleField "disks"              [t| [Disk]             |]
+  , simpleField "disk_template"      [t| DiskTemplate       |]
+  , simpleField "disks_active"       [t| Bool               |]
   , optionalField $ simpleField "network_port" [t| Int  |]
   ]
   ++ timeStampFields
@@ -515,15 +472,28 @@ instance SerialNoObject Instance where
 instance TagsObject Instance where
   tagsOf = instTags
 
+-- | Retrieves the real disk size requirements for all the disks of the
+-- instance. This includes the metadata etc. and is different from the values
+-- visible to the instance.
+getDiskSizeRequirements :: Instance -> Int
+getDiskSizeRequirements inst =
+  sum . map
+    (\disk -> case instDiskTemplate inst of
+                DTDrbd8    -> diskSize disk + C.drbdMetaSize
+                DTDiskless -> 0
+                DTBlock    -> 0
+                _          -> diskSize disk )
+    $ instDisks inst
+
 -- * IPolicy definitions
 
 $(buildParam "ISpec" "ispec"
-  [ simpleField C.ispecMemSize     [t| Int |]
-  , simpleField C.ispecDiskSize    [t| Int |]
-  , simpleField C.ispecDiskCount   [t| Int |]
-  , simpleField C.ispecCpuCount    [t| Int |]
-  , simpleField C.ispecNicCount    [t| Int |]
-  , simpleField C.ispecSpindleUse  [t| Int |]
+  [ simpleField ConstantUtils.ispecMemSize     [t| Int |]
+  , simpleField ConstantUtils.ispecDiskSize    [t| Int |]
+  , simpleField ConstantUtils.ispecDiskCount   [t| Int |]
+  , simpleField ConstantUtils.ispecCpuCount    [t| Int |]
+  , simpleField ConstantUtils.ispecNicCount    [t| Int |]
+  , simpleField ConstantUtils.ispecSpindleUse  [t| Int |]
   ])
 
 $(buildObject "MinMaxISpecs" "mmis"
@@ -534,23 +504,23 @@ $(buildObject "MinMaxISpecs" "mmis"
 -- | Custom partial ipolicy. This is not built via buildParam since it
 -- has a special 2-level inheritance mode.
 $(buildObject "PartialIPolicy" "ipolicy"
-  [ optionalField . renameField "MinMaxISpecsP"
-                    $ simpleField C.ispecsMinmax   [t| [MinMaxISpecs] |]
-  , optionalField . renameField "StdSpecP"
-                    $ simpleField "std"            [t| PartialISpecParams |]
-  , optionalField . renameField "SpindleRatioP"
-                    $ simpleField "spindle-ratio"  [t| Double |]
-  , optionalField . renameField "VcpuRatioP"
-                    $ simpleField "vcpu-ratio"     [t| Double |]
-  , optionalField . renameField "DiskTemplatesP"
-                    $ simpleField "disk-templates" [t| [DiskTemplate] |]
+  [ optionalField . renameField "MinMaxISpecsP" $
+    simpleField ConstantUtils.ispecsMinmax [t| [MinMaxISpecs] |]
+  , optionalField . renameField "StdSpecP" $
+    simpleField "std" [t| PartialISpecParams |]
+  , optionalField . renameField "SpindleRatioP" $
+    simpleField "spindle-ratio" [t| Double |]
+  , optionalField . renameField "VcpuRatioP" $
+    simpleField "vcpu-ratio" [t| Double |]
+  , optionalField . renameField "DiskTemplatesP" $
+    simpleField "disk-templates" [t| [DiskTemplate] |]
   ])
 
 -- | Custom filled ipolicy. This is not built via buildParam since it
 -- has a special 2-level inheritance mode.
 $(buildObject "FilledIPolicy" "ipolicy"
-  [ renameField "MinMaxISpecs"
-    $ simpleField C.ispecsMinmax [t| [MinMaxISpecs] |]
+  [ renameField "MinMaxISpecs" $
+    simpleField ConstantUtils.ispecsMinmax [t| [MinMaxISpecs] |]
   , renameField "StdSpec" $ simpleField "std" [t| FilledISpecParams |]
   , simpleField "spindle-ratio"  [t| Double |]
   , simpleField "vcpu-ratio"     [t| Double |]
@@ -584,6 +554,10 @@ $(buildParam "ND" "ndp"
   [ simpleField "oob_program"   [t| String |]
   , simpleField "spindle_count" [t| Int    |]
   , simpleField "exclusive_storage" [t| Bool |]
+  , simpleField "ovs"           [t| Bool |]
+  , simpleField "ovs_name"       [t| String |]
+  , simpleField "ovs_link"       [t| String |]
+  , simpleField "ssh_port"      [t| Int |]
   ])
 
 $(buildObject "Node" "node" $
@@ -654,8 +628,8 @@ instance TagsObject NodeGroup where
 
 -- | IP family type
 $(declareIADT "IpFamily"
-  [ ("IpFamilyV4", 'C.ip4Family)
-  , ("IpFamilyV6", 'C.ip6Family)
+  [ ("IpFamilyV4", 'AutoConf.pyAfInet4)
+  , ("IpFamilyV6", 'AutoConf.pyAfInet6)
   ])
 $(makeJSONInstance ''IpFamily)
 
@@ -683,47 +657,58 @@ type ClusterNicParams = Container FilledNicParams
 -- | Cluster UID Pool, list (low, high) UID ranges.
 type UidPool = [(Int, Int)]
 
+-- | The iallocator parameters type.
+type IAllocatorParams = Container JSValue
+
+-- | The master candidate client certificate digests
+type CandidateCertificates = Container String
+
 -- * Cluster definitions
 $(buildObject "Cluster" "cluster" $
-  [ simpleField "rsahostkeypub"           [t| String           |]
+  [ simpleField "rsahostkeypub"             [t| String           |]
   , optionalField $
-    simpleField "dsahostkeypub"           [t| String           |]
-  , simpleField "highest_used_port"       [t| Int              |]
-  , simpleField "tcpudp_port_pool"        [t| [Int]            |]
-  , simpleField "mac_prefix"              [t| String           |]
+    simpleField "dsahostkeypub"             [t| String           |]
+  , simpleField "highest_used_port"         [t| Int              |]
+  , simpleField "tcpudp_port_pool"          [t| [Int]            |]
+  , simpleField "mac_prefix"                [t| String           |]
   , optionalField $
-    simpleField "volume_group_name"       [t| String           |]
-  , simpleField "reserved_lvs"            [t| [String]         |]
+    simpleField "volume_group_name"         [t| String           |]
+  , simpleField "reserved_lvs"              [t| [String]         |]
   , optionalField $
-    simpleField "drbd_usermode_helper"    [t| String           |]
-  , simpleField "master_node"             [t| String           |]
-  , simpleField "master_ip"               [t| String           |]
-  , simpleField "master_netdev"           [t| String           |]
-  , simpleField "master_netmask"          [t| Int              |]
-  , simpleField "use_external_mip_script" [t| Bool             |]
-  , simpleField "cluster_name"            [t| String           |]
-  , simpleField "file_storage_dir"        [t| String           |]
-  , simpleField "shared_file_storage_dir" [t| String           |]
-  , simpleField "enabled_hypervisors"     [t| [Hypervisor]     |]
-  , simpleField "hvparams"                [t| ClusterHvParams  |]
-  , simpleField "os_hvp"                  [t| OsHvParams       |]
-  , simpleField "beparams"                [t| ClusterBeParams  |]
-  , simpleField "osparams"                [t| ClusterOsParams  |]
-  , simpleField "nicparams"               [t| ClusterNicParams |]
-  , simpleField "ndparams"                [t| FilledNDParams   |]
-  , simpleField "diskparams"              [t| DiskParams       |]
-  , simpleField "candidate_pool_size"     [t| Int              |]
-  , simpleField "modify_etc_hosts"        [t| Bool             |]
-  , simpleField "modify_ssh_setup"        [t| Bool             |]
-  , simpleField "maintain_node_health"    [t| Bool             |]
-  , simpleField "uid_pool"                [t| UidPool          |]
-  , simpleField "default_iallocator"      [t| String           |]
-  , simpleField "hidden_os"               [t| [String]         |]
-  , simpleField "blacklisted_os"          [t| [String]         |]
-  , simpleField "primary_ip_family"       [t| IpFamily         |]
-  , simpleField "prealloc_wipe_disks"     [t| Bool             |]
-  , simpleField "ipolicy"                 [t| FilledIPolicy    |]
-  , simpleField "enabled_disk_templates"  [t| [DiskTemplate]   |]
+    simpleField "drbd_usermode_helper"      [t| String           |]
+  , simpleField "master_node"               [t| String           |]
+  , simpleField "master_ip"                 [t| String           |]
+  , simpleField "master_netdev"             [t| String           |]
+  , simpleField "master_netmask"            [t| Int              |]
+  , simpleField "use_external_mip_script"   [t| Bool             |]
+  , simpleField "cluster_name"              [t| String           |]
+  , simpleField "file_storage_dir"          [t| String           |]
+  , simpleField "shared_file_storage_dir"   [t| String           |]
+  , simpleField "gluster_storage_dir"       [t| String           |]
+  , simpleField "enabled_hypervisors"       [t| [Hypervisor]     |]
+  , simpleField "hvparams"                  [t| ClusterHvParams  |]
+  , simpleField "os_hvp"                    [t| OsHvParams       |]
+  , simpleField "beparams"                  [t| ClusterBeParams  |]
+  , simpleField "osparams"                  [t| ClusterOsParams  |]
+  , simpleField "nicparams"                 [t| ClusterNicParams |]
+  , simpleField "ndparams"                  [t| FilledNDParams   |]
+  , simpleField "diskparams"                [t| DiskParams       |]
+  , simpleField "candidate_pool_size"       [t| Int              |]
+  , simpleField "modify_etc_hosts"          [t| Bool             |]
+  , simpleField "modify_ssh_setup"          [t| Bool             |]
+  , simpleField "maintain_node_health"      [t| Bool             |]
+  , simpleField "uid_pool"                  [t| UidPool          |]
+  , simpleField "default_iallocator"        [t| String           |]
+  , simpleField "default_iallocator_params" [t| IAllocatorParams |]
+  , simpleField "hidden_os"                 [t| [String]         |]
+  , simpleField "blacklisted_os"            [t| [String]         |]
+  , simpleField "primary_ip_family"         [t| IpFamily         |]
+  , simpleField "prealloc_wipe_disks"       [t| Bool             |]
+  , simpleField "ipolicy"                   [t| FilledIPolicy    |]
+  , simpleField "enabled_disk_templates"    [t| [DiskTemplate]   |]
+  , simpleField "candidate_certs"           [t| CandidateCertificates |]
+  , simpleField "max_running_jobs"          [t| Int              |]
+  , simpleField "enabled_user_shutdown"     [t| Bool             |]
  ]
  ++ timeStampFields
  ++ uuidFields
