@@ -1,4 +1,5 @@
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE ExistentialQuantification, TemplateHaskell #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 
 {-| Implementation of the opcodes.
 
@@ -26,137 +27,181 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 -}
 
 module Ganeti.OpCodes
-  ( OpCode(..)
-  , TagObject(..)
-  , tagObjectFrom
-  , encodeTagObject
-  , decodeTagObject
+  ( pyClasses
+  , OpCode(..)
   , ReplaceDisksMode(..)
   , DiskIndex
   , mkDiskIndex
   , unDiskIndex
   , opID
+  , opReasonSrcID
   , allOpIDs
   , allOpFields
   , opSummary
   , CommonOpParams(..)
   , defOpParams
   , MetaOpCode(..)
+  , resolveDependencies
   , wrapOpCode
   , setOpComment
   , setOpPriority
   ) where
 
-import Data.Maybe (fromMaybe)
-import Text.JSON (readJSON, JSON, JSValue, makeObj)
+import Data.List (intercalate)
+import Data.Map (Map)
 import qualified Text.JSON
+import Text.JSON (readJSON, JSObject, JSON, JSValue(..), makeObj, fromJSObject)
 
-import Ganeti.THH
-
+import qualified Ganeti.Constants as C
+import qualified Ganeti.Hs2Py.OpDoc as OpDoc
 import Ganeti.OpParams
-import Ganeti.Types (OpSubmitPriority(..), fromNonEmpty)
+import Ganeti.PyValue ()
 import Ganeti.Query.Language (queryTypeOpToRaw)
+import Ganeti.THH
+import Ganeti.Types
+
+instance PyValue DiskIndex where
+  showValue = showValue . unDiskIndex
+
+instance PyValue IDiskParams where
+  showValue _ = error "OpCodes.showValue(IDiskParams): unhandled case"
+
+instance PyValue RecreateDisksInfo where
+  showValue RecreateDisksAll = "[]"
+  showValue (RecreateDisksIndices is) = showValue is
+  showValue (RecreateDisksParams is) = showValue is
+
+instance PyValue a => PyValue (SetParamsMods a) where
+  showValue SetParamsEmpty = "[]"
+  showValue _ = error "OpCodes.showValue(SetParamsMods): unhandled case"
+
+instance PyValue a => PyValue (NonNegative a) where
+  showValue = showValue . fromNonNegative
+
+instance PyValue a => PyValue (NonEmpty a) where
+  showValue = showValue . fromNonEmpty
+
+-- FIXME: should use the 'toRaw' function instead of being harcoded or
+-- perhaps use something similar to the NonNegative type instead of
+-- using the declareSADT
+instance PyValue ExportMode where
+  showValue ExportModeLocal = show C.exportModeLocal
+  showValue ExportModeRemote = show C.exportModeLocal
+
+instance PyValue CVErrorCode where
+  showValue = cVErrorCodeToRaw
+
+instance PyValue VerifyOptionalChecks where
+  showValue = verifyOptionalChecksToRaw
+
+instance PyValue INicParams where
+  showValue = error "instance PyValue INicParams: not implemented"
+
+instance PyValue a => PyValue (JSObject a) where
+  showValue obj =
+    "{" ++ intercalate ", " (map showPair (fromJSObject obj)) ++ "}"
+    where showPair (k, v) = show k ++ ":" ++ showValue v
+
+instance PyValue JSValue where
+  showValue (JSObject obj) = showValue obj
+  showValue x = show x
+
+type JobIdListOnly = Map String [(Bool, Either String JobId)]
+
+type InstanceMultiAllocResponse =
+  ([(Bool, Either String JobId)], NonEmptyString)
+
+type QueryFieldDef =
+  (NonEmptyString, NonEmptyString, TagKind, NonEmptyString)
+
+type QueryResponse =
+  ([QueryFieldDef], [[(QueryResultCode, JSValue)]])
+
+type QueryFieldsResponse = [QueryFieldDef]
 
 -- | OpCode representation.
 --
 -- We only implement a subset of Ganeti opcodes: those which are actually used
 -- in the htools codebase.
 $(genOpCode "OpCode"
-  [ ("OpTestDelay",
-     [ pDelayDuration
-     , pDelayOnMaster
-     , pDelayOnNodes
-     , pDelayOnNodeUuids
-     , pDelayRepeat
-     ])
-  , ("OpInstanceReplaceDisks",
-     [ pInstanceName
-     , pInstanceUuid
-     , pEarlyRelease
-     , pIgnoreIpolicy
-     , pReplaceDisksMode
-     , pReplaceDisksList
-     , pRemoteNode
-     , pRemoteNodeUuid
-     , pIallocator
-     ])
-  , ("OpInstanceFailover",
-     [ pInstanceName
-     , pInstanceUuid
-     , pShutdownTimeout
-     , pIgnoreConsistency
-     , pMigrationTargetNode
-     , pMigrationTargetNodeUuid
-     , pIgnoreIpolicy
-     , pIallocator
-     , pMigrationCleanup
-     ])
-  , ("OpInstanceMigrate",
-     [ pInstanceName
-     , pInstanceUuid
-     , pMigrationMode
-     , pMigrationLive
-     , pMigrationTargetNode
-     , pMigrationTargetNodeUuid
-     , pAllowRuntimeChgs
-     , pIgnoreIpolicy
-     , pMigrationCleanup
-     , pIallocator
-     , pAllowFailover
-     ])
-  , ("OpTagsGet",
-     [ pTagsObject
-     , pUseLocking
-     ])
-  , ("OpTagsSearch",
-     [ pTagSearchPattern ])
-  , ("OpTagsSet",
-     [ pTagsObject
-     , pTagsList
-     ])
-  , ("OpTagsDel",
-     [ pTagsObject
-     , pTagsList
-     ])
-  , ("OpClusterPostInit", [])
-  , ("OpClusterDestroy", [])
-  , ("OpClusterQuery", [])
+  [ ("OpClusterPostInit",
+     [t| Bool |],
+     OpDoc.opClusterPostInit,
+     [],
+     [])
+  , ("OpClusterDestroy",
+     [t| NonEmptyString |],
+     OpDoc.opClusterDestroy,
+     [],
+     [])
+  , ("OpClusterQuery",
+     [t| JSObject JSValue |],
+     OpDoc.opClusterQuery,
+     [],
+     [])
   , ("OpClusterVerify",
+     [t| JobIdListOnly |],
+     OpDoc.opClusterVerify,
      [ pDebugSimulateErrors
      , pErrorCodes
      , pSkipChecks
      , pIgnoreErrors
      , pVerbose
      , pOptGroupName
-     ])
+     ],
+     [])
   , ("OpClusterVerifyConfig",
+     [t| Bool |],
+     OpDoc.opClusterVerifyConfig,
      [ pDebugSimulateErrors
      , pErrorCodes
      , pIgnoreErrors
      , pVerbose
-     ])
+     ],
+     [])
   , ("OpClusterVerifyGroup",
+     [t| Bool |],
+     OpDoc.opClusterVerifyGroup,
      [ pGroupName
      , pDebugSimulateErrors
      , pErrorCodes
      , pSkipChecks
      , pIgnoreErrors
      , pVerbose
-     ])
-  , ("OpClusterVerifyDisks", [])
+     ],
+     "group_name")
+  , ("OpClusterVerifyDisks",
+     [t| JobIdListOnly |],
+     OpDoc.opClusterVerifyDisks,
+     [],
+     [])
   , ("OpGroupVerifyDisks",
+     [t| (Map String String, [String], Map String [[String]]) |],
+     OpDoc.opGroupVerifyDisks,
      [ pGroupName
-     ])
+     ],
+     "group_name")
   , ("OpClusterRepairDiskSizes",
+     [t| [(NonEmptyString, NonNegative Int, NonEmptyString, NonNegative Int)]|],
+     OpDoc.opClusterRepairDiskSizes,
      [ pInstances
-     ])
+     ],
+     [])
   , ("OpClusterConfigQuery",
+     [t| [JSValue] |],
+     OpDoc.opClusterConfigQuery,
      [ pOutputFields
-     ])
+     ],
+     [])
   , ("OpClusterRename",
+      [t| NonEmptyString |],
+      OpDoc.opClusterRename,
      [ pName
-     ])
+     ],
+     "name")
   , ("OpClusterSetParams",
+     [t| () |],
+     OpDoc.opClusterSetParams,
      [ pForce
      , pHvState
      , pDiskState
@@ -168,16 +213,18 @@ $(genOpCode "OpCode"
      , pClusterOsParams
      , pDiskParams
      , pCandidatePoolSize
+     , pMaxRunningJobs
      , pUidPool
      , pAddUids
      , pRemoveUids
      , pMaintainNodeHealth
      , pPreallocWipeDisks
      , pNicParams
-     , pNdParams
-     , pIpolicy
+     , withDoc "Cluster-wide node parameter defaults" pNdParams
+     , withDoc "Cluster-wide ipolicy specs" pIpolicy
      , pDrbdHelper
      , pDefaultIAllocator
+     , pDefaultIAllocatorParams
      , pMasterNetdev
      , pMasterNetmask
      , pReservedLvs
@@ -186,35 +233,82 @@ $(genOpCode "OpCode"
      , pUseExternalMipScript
      , pEnabledDiskTemplates
      , pModifyEtcHosts
-     , pGlobalFileStorageDir
-     , pGlobalSharedFileStorageDir
-     ])
-  , ("OpClusterRedistConf", [])
-  , ("OpClusterActivateMasterIp", [])
-  , ("OpClusterDeactivateMasterIp", [])
+     , pClusterFileStorageDir
+     , pClusterSharedFileStorageDir
+     , pClusterGlusterStorageDir
+     , pEnabledUserShutdown
+     ],
+     [])
+  , ("OpClusterRedistConf",
+     [t| () |],
+     OpDoc.opClusterRedistConf,
+     [],
+     [])
+  , ("OpClusterActivateMasterIp",
+     [t| () |],
+     OpDoc.opClusterActivateMasterIp,
+     [],
+     [])
+  , ("OpClusterDeactivateMasterIp",
+     [t| () |],
+     OpDoc.opClusterDeactivateMasterIp,
+     [],
+     [])
+  , ("OpClusterRenewCrypto",
+     [t| () |],
+     OpDoc.opClusterRenewCrypto,
+     [],
+     [])
   , ("OpQuery",
+     [t| QueryResponse |],
+     OpDoc.opQuery,
      [ pQueryWhat
      , pUseLocking
      , pQueryFields
      , pQueryFilter
-     ])
+     ],
+     "what")
   , ("OpQueryFields",
+     [t| QueryFieldsResponse |],
+     OpDoc.opQueryFields,
      [ pQueryWhat
-     , pQueryFields
-     ])
+     , pQueryFieldsFields
+     ],
+     "what")
   , ("OpOobCommand",
+     [t| [[(QueryResultCode, JSValue)]] |],
+     OpDoc.opOobCommand,
      [ pNodeNames
-     , pNodeUuids
+     , withDoc "List of node UUIDs to run the OOB command against" pNodeUuids
      , pOobCommand
      , pOobTimeout
      , pIgnoreStatus
      , pPowerDelay
-     ])
+     ],
+     [])
+  , ("OpRestrictedCommand",
+     [t| [(Bool, String)] |],
+     OpDoc.opRestrictedCommand,
+     [ pUseLocking
+     , withDoc
+       "Nodes on which the command should be run (at least one)"
+       pRequiredNodes
+     , withDoc
+       "Node UUIDs on which the command should be run (at least one)"
+       pRequiredNodeUuids
+     , pRestrictedCommand
+     ],
+     [])
   , ("OpNodeRemove",
+     [t| () |],
+      OpDoc.opNodeRemove,
      [ pNodeName
      , pNodeUuid
-     ])
+     ],
+     "node_name")
   , ("OpNodeAdd",
+     [t| () |],
+      OpDoc.opNodeAdd,
      [ pNodeName
      , pHvState
      , pDiskState
@@ -225,40 +319,56 @@ $(genOpCode "OpCode"
      , pMasterCapable
      , pVmCapable
      , pNdParams
-    ])
-  , ("OpNodeQuery", dOldQuery)
+     ],
+     "node_name")
   , ("OpNodeQueryvols",
+     [t| [JSValue] |],
+     OpDoc.opNodeQueryvols,
      [ pOutputFields
-     , pNodes
-     ])
+     , withDoc "Empty list to query all nodes, node names otherwise" pNodes
+     ],
+     [])
   , ("OpNodeQueryStorage",
+     [t| [[JSValue]] |],
+     OpDoc.opNodeQueryStorage,
      [ pOutputFields
-     , pStorageType
-     , pNodes
+     , pStorageTypeOptional
+     , withDoc
+       "Empty list to query all, list of names to query otherwise"
+       pNodes
      , pStorageName
-     ])
+     ],
+     [])
   , ("OpNodeModifyStorage",
+     [t| () |],
+     OpDoc.opNodeModifyStorage,
      [ pNodeName
      , pNodeUuid
      , pStorageType
      , pStorageName
      , pStorageChanges
-     ])
+     ],
+     "node_name")
   , ("OpRepairNodeStorage",
+      [t| () |],
+      OpDoc.opRepairNodeStorage,
      [ pNodeName
      , pNodeUuid
      , pStorageType
      , pStorageName
      , pIgnoreConsistency
-     ])
+     ],
+     "node_name")
   , ("OpNodeSetParams",
+     [t| [(NonEmptyString, JSValue)] |],
+     OpDoc.opNodeSetParams,
      [ pNodeName
      , pNodeUuid
      , pForce
      , pHvState
      , pDiskState
      , pMasterCandidate
-     , pOffline
+     , withDoc "Whether to mark the node offline" pOffline
      , pDrained
      , pAutoPromote
      , pMasterCapable
@@ -266,13 +376,19 @@ $(genOpCode "OpCode"
      , pSecondaryIp
      , pNdParams
      , pPowered
-     ])
+     ],
+     "node_name")
   , ("OpNodePowercycle",
+     [t| Maybe NonEmptyString |],
+     OpDoc.opNodePowercycle,
      [ pNodeName
      , pNodeUuid
      , pForce
-     ])
+     ],
+     "node_name")
   , ("OpNodeMigrate",
+     [t| JobIdListOnly |],
+     OpDoc.opNodeMigrate,
      [ pNodeName
      , pNodeUuid
      , pMigrationMode
@@ -282,8 +398,11 @@ $(genOpCode "OpCode"
      , pAllowRuntimeChgs
      , pIgnoreIpolicy
      , pIallocator
-     ])
+     ],
+     "node_name")
   , ("OpNodeEvacuate",
+     [t| JobIdListOnly |],
+     OpDoc.opNodeEvacuate,
      [ pEarlyRelease
      , pNodeName
      , pNodeUuid
@@ -291,16 +410,20 @@ $(genOpCode "OpCode"
      , pRemoteNodeUuid
      , pIallocator
      , pEvacMode
-     ])
+     ],
+     "node_name")
   , ("OpInstanceCreate",
+     [t| [NonEmptyString] |],
+     OpDoc.opInstanceCreate,
      [ pInstanceName
      , pForceVariant
      , pWaitForSync
      , pNameCheck
      , pIgnoreIpolicy
+     , pOpportunisticLocking
      , pInstBeParams
      , pInstDisks
-     , pDiskTemplate
+     , pOptDiskTemplate
      , pFileDriver
      , pFileStorageDir
      , pInstHvParams
@@ -325,36 +448,51 @@ $(genOpCode "OpCode"
      , pSrcNode
      , pSrcNodeUuid
      , pSrcPath
+     , pBackupCompress
      , pStartInstance
-     , pOpportunisticLocking
      , pInstTags
-     ])
+     ],
+     "instance_name")
   , ("OpInstanceMultiAlloc",
-     [ pIallocator
+     [t| InstanceMultiAllocResponse |],
+     OpDoc.opInstanceMultiAlloc,
+     [ pOpportunisticLocking
+     , pIallocator
      , pMultiAllocInstances
-     , pOpportunisticLocking
-     ])
+     ],
+     [])
   , ("OpInstanceReinstall",
+     [t| () |],
+     OpDoc.opInstanceReinstall,
      [ pInstanceName
      , pInstanceUuid
      , pForceVariant
      , pInstOs
      , pTempOsParams
-     ])
+     ],
+     "instance_name")
   , ("OpInstanceRemove",
+     [t| () |],
+     OpDoc.opInstanceRemove,
      [ pInstanceName
      , pInstanceUuid
      , pShutdownTimeout
      , pIgnoreFailures
-     ])
+     ],
+     "instance_name")
   , ("OpInstanceRename",
+     [t| NonEmptyString |],
+     OpDoc.opInstanceRename,
      [ pInstanceName
      , pInstanceUuid
-     , pNewName
+     , withDoc "New instance name" pNewName
      , pNameCheck
      , pIpCheck
-     ])
+     ],
+     [])
   , ("OpInstanceStartup",
+     [t| () |],
+     OpDoc.opInstanceStartup,
      [ pInstanceName
      , pInstanceUuid
      , pForce
@@ -363,61 +501,135 @@ $(genOpCode "OpCode"
      , pTempBeParams
      , pNoRemember
      , pStartupPaused
-     ])
+       -- timeout to cleanup a user down instance
+     , pShutdownTimeout
+     ],
+     "instance_name")
   , ("OpInstanceShutdown",
+     [t| () |],
+     OpDoc.opInstanceShutdown,
      [ pInstanceName
      , pInstanceUuid
      , pForce
      , pIgnoreOfflineNodes
      , pShutdownTimeout'
      , pNoRemember
-     ])
+     , pAdminStateSource
+     ],
+     "instance_name")
   , ("OpInstanceReboot",
+     [t| () |],
+     OpDoc.opInstanceReboot,
      [ pInstanceName
      , pInstanceUuid
      , pShutdownTimeout
      , pIgnoreSecondaries
      , pRebootType
-     ])
+     ],
+     "instance_name")
+  , ("OpInstanceReplaceDisks",
+     [t| () |],
+     OpDoc.opInstanceReplaceDisks,
+     [ pInstanceName
+     , pInstanceUuid
+     , pEarlyRelease
+     , pIgnoreIpolicy
+     , pReplaceDisksMode
+     , pReplaceDisksList
+     , pRemoteNode
+     , pRemoteNodeUuid
+     , pIallocator
+     ],
+     "instance_name")
+  , ("OpInstanceFailover",
+     [t| () |],
+     OpDoc.opInstanceFailover,
+     [ pInstanceName
+     , pInstanceUuid
+     , pShutdownTimeout
+     , pIgnoreConsistency
+     , pMigrationTargetNode
+     , pMigrationTargetNodeUuid
+     , pIgnoreIpolicy
+     , pMigrationCleanup
+     , pIallocator
+     ],
+     "instance_name")
+  , ("OpInstanceMigrate",
+     [t| () |],
+     OpDoc.opInstanceMigrate,
+     [ pInstanceName
+     , pInstanceUuid
+     , pMigrationMode
+     , pMigrationLive
+     , pMigrationTargetNode
+     , pMigrationTargetNodeUuid
+     , pAllowRuntimeChgs
+     , pIgnoreIpolicy
+     , pMigrationCleanup
+     , pIallocator
+     , pAllowFailover
+     ],
+     "instance_name")
   , ("OpInstanceMove",
+     [t| () |],
+     OpDoc.opInstanceMove,
      [ pInstanceName
      , pInstanceUuid
      , pShutdownTimeout
      , pIgnoreIpolicy
      , pMoveTargetNode
      , pMoveTargetNodeUuid
+     , pMoveCompress
      , pIgnoreConsistency
-     ])
+     ],
+     "instance_name")
   , ("OpInstanceConsole",
+     [t| JSObject JSValue |],
+     OpDoc.opInstanceConsole,
      [ pInstanceName
      , pInstanceUuid
-     ])
+     ],
+     "instance_name")
   , ("OpInstanceActivateDisks",
+     [t| [(NonEmptyString, NonEmptyString, NonEmptyString)] |],
+     OpDoc.opInstanceActivateDisks,
      [ pInstanceName
      , pInstanceUuid
      , pIgnoreDiskSize
      , pWaitForSyncFalse
-     ])
+     ],
+     "instance_name")
   , ("OpInstanceDeactivateDisks",
+     [t| () |],
+     OpDoc.opInstanceDeactivateDisks,
      [ pInstanceName
      , pInstanceUuid
      , pForce
-     ])
+     ],
+     "instance_name")
   , ("OpInstanceRecreateDisks",
+     [t| () |],
+     OpDoc.opInstanceRecreateDisks,
      [ pInstanceName
      , pInstanceUuid
      , pRecreateDisksInfo
-     , pNodes
-     , pNodeUuids
+     , withDoc "New instance nodes, if relocation is desired" pNodes
+     , withDoc "New instance node UUIDs, if relocation is desired" pNodeUuids
      , pIallocator
-     ])
-  , ("OpInstanceQuery", dOldQuery)
+     ],
+     "instance_name")
   , ("OpInstanceQueryData",
+     [t| JSObject (JSObject JSValue) |],
+     OpDoc.opInstanceQueryData,
      [ pUseLocking
      , pInstances
      , pStatic
-     ])
+     ],
+     [])
   , ("OpInstanceSetParams",
+      [t| [(NonEmptyString, JSValue)] |],
+      OpDoc.opInstanceSetParams,
      [ pInstanceName
      , pInstanceUuid
      , pForce
@@ -431,99 +643,187 @@ $(genOpCode "OpCode"
      , pOptDiskTemplate
      , pPrimaryNode
      , pPrimaryNodeUuid
-     , pRemoteNode
-     , pRemoteNodeUuid
+     , withDoc "Secondary node (used when changing disk template)" pRemoteNode
+     , withDoc
+       "Secondary node UUID (used when changing disk template)"
+       pRemoteNodeUuid
      , pOsNameChange
      , pInstOsParams
      , pWaitForSync
-     , pOffline
+     , withDoc "Whether to mark the instance as offline" pOffline
      , pIpConflictsCheck
-     ])
+     , pHotplug
+     , pHotplugIfPossible
+     ],
+     "instance_name")
   , ("OpInstanceGrowDisk",
+     [t| () |],
+     OpDoc.opInstanceGrowDisk,
      [ pInstanceName
      , pInstanceUuid
      , pWaitForSync
      , pDiskIndex
      , pDiskChgAmount
      , pDiskChgAbsolute
-     ])
+     ],
+     "instance_name")
   , ("OpInstanceChangeGroup",
+     [t| JobIdListOnly |],
+     OpDoc.opInstanceChangeGroup,
      [ pInstanceName
      , pInstanceUuid
      , pEarlyRelease
      , pIallocator
      , pTargetGroups
-     ])
+     ],
+     "instance_name")
   , ("OpGroupAdd",
+     [t| () |],
+     OpDoc.opGroupAdd,
      [ pGroupName
      , pNodeGroupAllocPolicy
      , pGroupNodeParams
      , pDiskParams
      , pHvState
      , pDiskState
-     , pIpolicy
-     ])
+     , withDoc "Group-wide ipolicy specs" pIpolicy
+     ],
+     "group_name")
   , ("OpGroupAssignNodes",
+     [t| () |],
+     OpDoc.opGroupAssignNodes,
      [ pGroupName
      , pForce
-     , pRequiredNodes
-     , pRequiredNodeUuids
-     ])
-  , ("OpGroupQuery", dOldQueryNoLocking)
+     , withDoc "List of nodes to assign" pRequiredNodes
+     , withDoc "List of node UUIDs to assign" pRequiredNodeUuids
+     ],
+     "group_name")
   , ("OpGroupSetParams",
+     [t| [(NonEmptyString, JSValue)] |],
+     OpDoc.opGroupSetParams,
      [ pGroupName
      , pNodeGroupAllocPolicy
      , pGroupNodeParams
      , pDiskParams
      , pHvState
      , pDiskState
-     , pIpolicy
-     ])
+     , withDoc "Group-wide ipolicy specs" pIpolicy
+     ],
+     "group_name")
   , ("OpGroupRemove",
-     [ pGroupName ])
-  , ("OpGroupRename",
+     [t| () |],
+     OpDoc.opGroupRemove,
      [ pGroupName
-     , pNewName
-     ])
+     ],
+     "group_name")
+  , ("OpGroupRename",
+     [t| NonEmptyString |],
+     OpDoc.opGroupRename,
+     [ pGroupName
+     , withDoc "New group name" pNewName
+     ],
+     [])
   , ("OpGroupEvacuate",
+     [t| JobIdListOnly |],
+     OpDoc.opGroupEvacuate,
      [ pGroupName
      , pEarlyRelease
      , pIallocator
      , pTargetGroups
-     ])
+     , pSequential
+     , pForceFailover
+     ],
+     "group_name")
   , ("OpOsDiagnose",
+     [t| [[JSValue]] |],
+     OpDoc.opOsDiagnose,
      [ pOutputFields
-     , pNames ])
+     , withDoc "Which operating systems to diagnose" pNames
+     ],
+     [])
   , ("OpExtStorageDiagnose",
+     [t| [[JSValue]] |],
+     OpDoc.opExtStorageDiagnose,
      [ pOutputFields
-     , pNames ])
-  , ("OpBackupQuery",
-     [ pUseLocking
-     , pNodes
-     ])
+     , withDoc "Which ExtStorage Provider to diagnose" pNames
+     ],
+     [])
   , ("OpBackupPrepare",
+     [t| Maybe (JSObject JSValue) |],
+     OpDoc.opBackupPrepare,
      [ pInstanceName
      , pInstanceUuid
      , pExportMode
-     ])
+     ],
+     "instance_name")
   , ("OpBackupExport",
+     [t| (Bool, [Bool]) |],
+     OpDoc.opBackupExport,
      [ pInstanceName
      , pInstanceUuid
+     , pBackupCompress
      , pShutdownTimeout
      , pExportTargetNode
      , pExportTargetNodeUuid
      , pShutdownInstance
      , pRemoveInstance
      , pIgnoreRemoveFailures
-     , pExportMode
+     , defaultField [| ExportModeLocal |] pExportMode
      , pX509KeyName
      , pX509DestCA
-     ])
+     ],
+     "instance_name")
   , ("OpBackupRemove",
+     [t| () |],
+     OpDoc.opBackupRemove,
      [ pInstanceName
      , pInstanceUuid
-     ])
+     ],
+     "instance_name")
+  , ("OpTagsGet",
+     [t| [NonEmptyString] |],
+     OpDoc.opTagsGet,
+     [ pTagsObject
+     , pUseLocking
+     , withDoc "Name of object to retrieve tags from" pTagsName
+     ],
+     "name")
+  , ("OpTagsSearch",
+     [t| [(NonEmptyString, NonEmptyString)] |],
+     OpDoc.opTagsSearch,
+     [ pTagSearchPattern
+     ],
+     "pattern")
+  , ("OpTagsSet",
+     [t| () |],
+     OpDoc.opTagsSet,
+     [ pTagsObject
+     , pTagsList
+     , withDoc "Name of object where tag(s) should be added" pTagsName
+     ],
+     [])
+  , ("OpTagsDel",
+     [t| () |],
+     OpDoc.opTagsDel,
+     [ pTagsObject
+     , pTagsList
+     , withDoc "Name of object where tag(s) should be deleted" pTagsName
+     ],
+     [])
+  , ("OpTestDelay",
+     [t| () |],
+     OpDoc.opTestDelay,
+     [ pDelayDuration
+     , pDelayOnMaster
+     , pDelayOnNodes
+     , pDelayOnNodeUuids
+     , pDelayRepeat
+     , pDelayNoLocks
+     ],
+     "duration")
   , ("OpTestAllocator",
+     [t| String |],
+     OpDoc.opTestAllocator,
      [ pIAllocatorDirection
      , pIAllocatorMode
      , pIAllocatorReqName
@@ -541,20 +841,29 @@ $(genOpCode "OpCode"
      , pTargetGroups
      , pIAllocatorSpindleUse
      , pIAllocatorCount
-     ])
+     ],
+     "iallocator")
   , ("OpTestJqueue",
+     [t| Bool |],
+     OpDoc.opTestJqueue,
      [ pJQueueNotifyWaitLock
      , pJQueueNotifyExec
      , pJQueueLogMessages
      , pJQueueFail
-     ])
+     ],
+     [])
   , ("OpTestDummy",
+     [t| () |],
+     OpDoc.opTestDummy,
      [ pTestDummyResult
      , pTestDummyMessages
      , pTestDummyFail
      , pTestDummySubmitJobs
-     ])
+     ],
+     [])
   , ("OpNetworkAdd",
+     [t| () |],
+     OpDoc.opNetworkAdd,
      [ pNetworkName
      , pNetworkAddress4
      , pNetworkGateway4
@@ -563,39 +872,46 @@ $(genOpCode "OpCode"
      , pNetworkMacPrefix
      , pNetworkAddRsvdIps
      , pIpConflictsCheck
-     , pInstTags
-     ])
+     , withDoc "Network tags" pInstTags
+     ],
+     "network_name")
   , ("OpNetworkRemove",
+     [t| () |],
+     OpDoc.opNetworkRemove,
      [ pNetworkName
      , pForce
-     ])
+     ],
+     "network_name")
   , ("OpNetworkSetParams",
+     [t| () |],
+     OpDoc.opNetworkSetParams,
      [ pNetworkName
      , pNetworkGateway4
      , pNetworkAddress6
      , pNetworkGateway6
      , pNetworkMacPrefix
-     , pNetworkAddRsvdIps
+     , withDoc "Which external IP addresses to reserve" pNetworkAddRsvdIps
      , pNetworkRemoveRsvdIps
-     ])
+     ],
+     "network_name")
   , ("OpNetworkConnect",
+     [t| () |],
+     OpDoc.opNetworkConnect,
      [ pGroupName
      , pNetworkName
      , pNetworkMode
      , pNetworkLink
+     , pNetworkVlan
      , pIpConflictsCheck
-     ])
+     ],
+     "network_name")
   , ("OpNetworkDisconnect",
+     [t| () |],
+     OpDoc.opNetworkDisconnect,
      [ pGroupName
      , pNetworkName
-     ])
-  , ("OpNetworkQuery", dOldQuery)
-  , ("OpRestrictedCommand",
-     [ pUseLocking
-     , pRequiredNodes
-     , pRequiredNodeUuids
-     , pRestrictedCommand
-     ])
+     ],
+     "network_name")
   ])
 
 -- | Returns the OP_ID for a given opcode value.
@@ -604,6 +920,10 @@ $(genOpID ''OpCode "opID")
 -- | A list of all defined/supported opcode IDs.
 $(genAllOpIDs ''OpCode "allOpIDs")
 
+-- | Convert the opcode name to lowercase with underscores and strip
+-- the @Op@ prefix.
+$(genOpLowerStrip (C.opcodeReasonSrcOpcode ++ ":") ''OpCode "opReasonSrcID")
+
 instance JSON OpCode where
   readJSON = loadOpCode
   showJSON = saveOpCode
@@ -650,8 +970,7 @@ opSummaryVal OpGroupEvacuate { opGroupName = s } = Just (fromNonEmpty s)
 opSummaryVal OpBackupPrepare { opInstanceName = s } = Just s
 opSummaryVal OpBackupExport { opInstanceName = s } = Just s
 opSummaryVal OpBackupRemove { opInstanceName = s } = Just s
-opSummaryVal OpTagsGet { opKind = k } =
-  Just . fromMaybe "None" $ tagNameOf k
+opSummaryVal OpTagsGet { opKind = s } = Just (show s)
 opSummaryVal OpTagsSearch { opTagSearchPattern = s } = Just (fromNonEmpty s)
 opSummaryVal OpTestDelay { opDelayDuration = d } = Just (show d)
 opSummaryVal OpTestAllocator { opIallocator = s } =
@@ -693,11 +1012,24 @@ defOpParams =
                  , opReason     = []
                  }
 
+-- | Resolve relative dependencies to absolute ones, given the job ID.
+resolveDependsCommon :: (Monad m) => CommonOpParams -> JobId -> m CommonOpParams
+resolveDependsCommon p@(CommonOpParams { opDepends = Just deps}) jid = do
+  deps' <- mapM (`absoluteJobDependency` jid) deps
+  return p { opDepends = Just deps' }
+resolveDependsCommon p _ = return p
+
 -- | The top-level opcode type.
 data MetaOpCode = MetaOpCode { metaParams :: CommonOpParams
                              , metaOpCode :: OpCode
                              } deriving (Show, Eq)
 
+-- | Resolve relative dependencies to absolute ones, given the job Id.
+resolveDependencies :: (Monad m) => MetaOpCode -> JobId -> m MetaOpCode
+resolveDependencies mopc jid = do
+  mpar <- resolveDependsCommon (metaParams mopc) jid
+  return (mopc { metaParams = mpar })
+
 -- | JSON serialisation for 'MetaOpCode'.
 showMeta :: MetaOpCode -> JSValue
 showMeta (MetaOpCode params op) =
