@@ -24,11 +24,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 -}
 
 module Ganeti.Query.Common
-  ( rsNoData
+  ( NoDataRuntime(..)
+  , rsNoData
   , rsUnavail
   , rsNormal
   , rsMaybeNoData
   , rsMaybeUnavail
+  , rsErrorNoData
   , rsUnknown
   , missingRuntime
   , rpcErrorToStatus
@@ -37,20 +39,32 @@ module Ganeti.Query.Common
   , serialFields
   , tagsFields
   , dictFieldGetter
-  , buildQFTLookup
   , buildNdParamField
+  , buildBeParamField
+  , buildHvParamField
+  , getDefaultHypervisorSpec
+  , getHvParamsFromCluster
+  , aliasFields
   ) where
 
+import Control.Monad (guard)
 import qualified Data.Map as Map
 import Data.Maybe (fromMaybe)
 import Text.JSON (JSON, showJSON)
 
+import Ganeti.BasicTypes
 import qualified Ganeti.Constants as C
 import Ganeti.Config
+import Ganeti.Errors
+import Ganeti.JSON
 import Ganeti.Objects
 import Ganeti.Rpc
 import Ganeti.Query.Language
 import Ganeti.Query.Types
+import Ganeti.Types
+
+-- | The runtime used by queries which retrieve no live data.
+data NoDataRuntime = NoDataRuntime
 
 -- * Generic functions
 
@@ -85,6 +99,14 @@ rsNormal a = ResultEntry RSNormal $ Just (showJSON a)
 rsMaybeNoData :: (JSON a) => Maybe a -> ResultEntry
 rsMaybeNoData = maybe rsNoData rsNormal
 
+-- | Helper to declare a result from a 'ErrorResult' (an error happened
+-- while retrieving the data from a config, or there was no data).
+-- This function should be used if an error signals there was no data.
+rsErrorNoData :: (JSON a) => ErrorResult a -> ResultEntry
+rsErrorNoData res = case res of
+  Ok  x -> rsNormal x
+  Bad _ -> rsNoData
+
 -- | Helper to declare a result from a 'Maybe'. This version returns
 -- a 'RSUnavail' in case of 'Nothing'. It should be used for optional
 -- fields that are not set. For cases where 'Nothing' means that there
@@ -146,20 +168,13 @@ tagsFields =
 dictFieldGetter :: (DictObject a) => String -> Maybe a -> ResultEntry
 dictFieldGetter k = maybe rsNoData (rsMaybeNoData . lookup k . toDict)
 
--- | Build an optimised lookup map from a Python _PARAMETER_TYPES
--- association list.
-buildQFTLookup :: [(String, String)] -> Map.Map String FieldType
-buildQFTLookup =
-  Map.fromList .
-  map (\(k, v) -> (k, maybe QFTOther vTypeToQFT (vTypeFromRaw v)))
-
 -- | Ndparams optimised lookup map.
 ndParamTypes :: Map.Map String FieldType
-ndParamTypes = buildQFTLookup C.ndsParameterTypes
+ndParamTypes = Map.map vTypeToQFT C.ndsParameterTypes
 
 -- | Ndparams title map.
 ndParamTitles :: Map.Map String FieldTitle
-ndParamTitles = Map.fromList C.ndsParameterTitles
+ndParamTitles = C.ndsParameterTitles
 
 -- | Ndparam getter builder: given a field, it returns a FieldConfig
 -- getter, that is a function that takes the config and the object and
@@ -172,10 +187,66 @@ ndParamGetter field config =
 
 -- | Builds the ndparam fields for an object.
 buildNdParamField :: (NdParamObject a) => String -> FieldData a b
-buildNdParamField field =
-  let full_name = "ndp/" ++ field
-      title = fromMaybe field $ field `Map.lookup` ndParamTitles
-      qft = fromMaybe QFTOther $ field `Map.lookup` ndParamTypes
-      desc = "The \"" ++ field ++ "\" node parameter"
-  in (FieldDefinition full_name title qft desc,
-      FieldConfig (ndParamGetter field), QffNormal)
+buildNdParamField =
+  buildParamField "ndp" "node" ndParamTitles ndParamTypes ndParamGetter
+
+-- | Beparams optimised lookup map.
+beParamTypes :: Map.Map String FieldType
+beParamTypes = Map.map vTypeToQFT C.besParameterTypes
+
+-- | Builds the beparam fields for an object.
+buildBeParamField :: (String -> ConfigData -> a -> ResultEntry)
+                  -> String
+                  -> FieldData a b
+buildBeParamField =
+  buildParamField "be" "backend" C.besParameterTitles beParamTypes
+
+-- | Hvparams optimised lookup map.
+hvParamTypes :: Map.Map String FieldType
+hvParamTypes = Map.map vTypeToQFT C.hvsParameterTypes
+
+-- | Builds the beparam fields for an object.
+buildHvParamField :: (String -> ConfigData -> a -> ResultEntry)
+                  -> String
+                  -> FieldData a b
+buildHvParamField =
+  buildParamField "hv" "hypervisor" C.hvsParameterTitles hvParamTypes
+
+-- | Builds a param field for a certain getter class
+buildParamField :: String -- ^ Prefix
+                -> String -- ^ Parameter group name
+                -> Map.Map String String -- ^ Parameter title map
+                -> Map.Map String FieldType -- ^ Parameter type map
+                -> (String -> ConfigData -> a -> ResultEntry)
+                -> String -- ^ The parameter name
+                -> FieldData a b
+buildParamField prefix paramGroupName titleMap typeMap getter field =
+  let full_name = prefix ++ "/" ++ field
+      title = fromMaybe full_name $ field `Map.lookup` titleMap
+      qft = fromMaybe QFTOther $ field `Map.lookup` typeMap
+      desc = "The \"" ++ field ++ "\" " ++ paramGroupName ++ " parameter"
+  in ( FieldDefinition full_name title qft desc
+     , FieldConfig (getter field), QffNormal
+     )
+
+-- | Looks up the default hypervisor and its hvparams
+getDefaultHypervisorSpec :: ConfigData -> (Hypervisor, HvParams)
+getDefaultHypervisorSpec cfg = (hv, getHvParamsFromCluster cfg hv)
+  where hv = getDefaultHypervisor cfg
+
+-- | Looks up the cluster's hvparams of the given hypervisor
+getHvParamsFromCluster :: ConfigData -> Hypervisor -> HvParams
+getHvParamsFromCluster cfg hv =
+  fromMaybe (GenericContainer Map.empty) .
+    Map.lookup (hypervisorToRaw hv) .
+      fromContainer . clusterHvparams $ configCluster cfg
+
+-- | Given an alias list and a field list, copies field definitions under a
+-- new field name. Aliases should be tested - see the test module
+-- 'Test.Ganeti.Query.Aliases'!
+aliasFields :: [(FieldName, FieldName)] -> FieldList a b -> FieldList a b
+aliasFields aliases fieldList = fieldList ++ do
+  alias <- aliases
+  (FieldDefinition name d1 d2 d3, v1, v2) <- fieldList
+  guard (snd alias == name)
+  return (FieldDefinition (fst alias) d1 d2 d3, v1, v2)
