@@ -1,4 +1,4 @@
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE ParallelListComp, TemplateHaskell #-}
 
 {-| TemplateHaskell helper for Ganeti Haskell code.
 
@@ -10,7 +10,7 @@ needs in this module (except the one for unittests).
 
 {-
 
-Copyright (C) 2011, 2012 Google Inc.
+Copyright (C) 2011, 2012, 2013 Google Inc.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,17 +30,28 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 -}
 
 module Ganeti.THH ( declareSADT
+                  , declareLADT
+                  , declareILADT
                   , declareIADT
                   , makeJSONInstance
+                  , deCamelCase
                   , genOpID
+                  , genOpLowerStrip
                   , genAllConstr
                   , genAllOpIDs
+                  , PyValue(..)
+                  , PyValueEx(..)
+                  , OpCodeField(..)
+                  , OpCodeDescriptor(..)
                   , genOpCode
                   , genStrOfOp
                   , genStrOfKey
                   , genLuxiOp
-                  , Field
+                  , Field (..)
                   , simpleField
+                  , andRestArguments
+                  , specialNumericalField
+                  , withDoc
                   , defaultField
                   , optionalField
                   , optionalNullSerField
@@ -59,10 +70,12 @@ module Ganeti.THH ( declareSADT
                   , excErrMsg
                   ) where
 
-import Control.Monad (liftM)
+import Control.Applicative
+import Control.Monad
 import Data.Char
 import Data.List
-import Data.Maybe (fromMaybe)
+import Data.Maybe
+import qualified Data.Map as M
 import qualified Data.Set as Set
 import Language.Haskell.TH
 
@@ -70,6 +83,9 @@ import qualified Text.JSON as JSON
 import Text.JSON.Pretty (pp_value)
 
 import Ganeti.JSON
+import Ganeti.PyValue
+import Ganeti.THH.PyType
+
 
 -- * Exported types
 
@@ -83,6 +99,8 @@ data OptionalType
   = NotOptional           -- ^ Field is not optional
   | OptionalOmitNull      -- ^ Field is optional, null is not serialised
   | OptionalSerializeNull -- ^ Field is optional, null is serialised
+  | AndRestArguments      -- ^ Special field capturing all the remaining fields
+                          -- as plain JSON values
   deriving (Show, Eq)
 
 -- | Serialised field data type.
@@ -94,6 +112,7 @@ data Field = Field { fieldName        :: String
                    , fieldDefault     :: Maybe (Q Exp)
                    , fieldConstr      :: Maybe String
                    , fieldIsOptional  :: OptionalType
+                   , fieldDoc         :: String
                    }
 
 -- | Generates a simple field.
@@ -107,8 +126,27 @@ simpleField fname ftype =
         , fieldDefault     = Nothing
         , fieldConstr      = Nothing
         , fieldIsOptional  = NotOptional
+        , fieldDoc         = ""
         }
 
+-- | Generate an AndRestArguments catch-all field.
+andRestArguments :: String -> Field
+andRestArguments fname =
+  Field { fieldName        = fname
+        , fieldType        = [t| M.Map String JSON.JSValue |]
+        , fieldRead        = Nothing
+        , fieldShow        = Nothing
+        , fieldExtraKeys   = []
+        , fieldDefault     = Nothing
+        , fieldConstr      = Nothing
+        , fieldIsOptional  = AndRestArguments
+        , fieldDoc         = ""
+        }
+
+withDoc :: String -> Field -> Field
+withDoc doc field =
+  field { fieldDoc = doc }
+
 -- | Sets the renamed constructor field.
 renameField :: String -> Field -> Field
 renameField constrName field = field { fieldConstr = Just constrName }
@@ -127,6 +165,32 @@ optionalField field = field { fieldIsOptional = OptionalOmitNull }
 optionalNullSerField :: Field -> Field
 optionalNullSerField field = field { fieldIsOptional = OptionalSerializeNull }
 
+-- | Wrapper around a special parse function, suitable as field-parsing
+-- function.
+numericalReadFn :: JSON.JSON a => (String -> JSON.Result a)
+                   -> [(String, JSON.JSValue)] -> JSON.JSValue -> JSON.Result a
+numericalReadFn _ _ v@(JSON.JSRational _ _) = JSON.readJSON v
+numericalReadFn f _ (JSON.JSString x) = f $ JSON.fromJSString x
+numericalReadFn _ _ _ = JSON.Error "A numerical field has to be a number or\ 
+                                   \ a string."
+
+-- | Wrapper to lift a read function to optional values
+makeReadOptional :: ([(String, JSON.JSValue)] -> JSON.JSValue -> JSON.Result a)
+                    -> [(String, JSON.JSValue)]
+                    -> Maybe JSON.JSValue -> JSON.Result (Maybe a)
+makeReadOptional _ _ Nothing = JSON.Ok Nothing
+makeReadOptional f o (Just x) = fmap Just $ f o x
+
+-- | Sets the read function to also accept string parsable by the given
+-- function.
+specialNumericalField :: Name -> Field -> Field
+specialNumericalField f field =
+  if (fieldIsOptional field == NotOptional)
+     then field { fieldRead = Just (appE (varE 'numericalReadFn) (varE f)) }
+     else field { fieldRead = Just (appE (varE 'makeReadOptional)
+                                         (appE (varE 'numericalReadFn)
+                                               (varE f))) }
+
 -- | Sets custom functions on a field.
 customField :: Name      -- ^ The name of the read function
             -> Name      -- ^ The name of the show function
@@ -157,8 +221,8 @@ fieldVariable f =
 -- | Compute the actual field type (taking into account possible
 -- optional status).
 actualFieldType :: Field -> Q Type
-actualFieldType f | fieldIsOptional f /= NotOptional = [t| Maybe $t |]
-                  | otherwise = t
+actualFieldType f | fieldIsOptional f `elem` [NotOptional, AndRestArguments] = t
+                  | otherwise =  [t| Maybe $t |]
                   where t = fieldType f
 
 -- | Checks that a given field is not optional (for object types or
@@ -222,8 +286,11 @@ type SimpleConstructor = (String, [SimpleField])
 -- | A definition for ADTs with simple fields.
 type SimpleObject = [SimpleConstructor]
 
--- | A type alias for a constructor of a regular object.
-type Constructor = (String, [Field])
+-- | A type alias for an opcode constructor of a regular object.
+type OpCodeConstructor = (String, Q Type, String, [Field], String)
+
+-- | A type alias for a Luxi constructor of a regular object.
+type LuxiConstructor = (String, [Field])
 
 -- * Helper functions
 
@@ -344,7 +411,7 @@ genToRaw traw fname tname constructors = do
 --               | s == \"value2\" = Cons2
 --               | otherwise = fail /.../
 -- @
-genFromRaw :: Name -> Name -> Name -> [(String, Name)] -> Q [Dec]
+genFromRaw :: Name -> Name -> Name -> [(String, Either String Name)] -> Q [Dec]
 genFromRaw traw fname tname constructors = do
   -- signature of form (Monad m) => String -> m $name
   sigt <- [t| (Monad m) => $(conT traw) -> m $(conT tname) |]
@@ -353,7 +420,7 @@ genFromRaw traw fname tname constructors = do
       varpe = varE varp
   clauses <- mapM (\(c, v) -> do
                      -- the clause match condition
-                     g <- normalG [| $varpe == $(varE v) |]
+                     g <- normalG [| $varpe == $(reprE v) |]
                      -- the clause result
                      r <- [| return $(conE (mkName c)) |]
                      return (g, r)) constructors
@@ -383,21 +450,38 @@ genFromRaw traw fname tname constructors = do
 --
 -- Note that this is basically just a custom show\/read instance,
 -- nothing else.
-declareADT :: Name -> String -> [(String, Name)] -> Q [Dec]
-declareADT traw sname cons = do
+declareADT
+  :: (a -> Either String Name) -> Name -> String -> [(String, a)] -> Q [Dec]
+declareADT fn traw sname cons = do
   let name = mkName sname
       ddecl = strADTDecl name (map fst cons)
       -- process cons in the format expected by genToRaw
-      cons' = map (\(a, b) -> (a, Right b)) cons
+      cons' = map (\(a, b) -> (a, fn b)) cons
   toraw <- genToRaw traw (toRawName sname) name cons'
-  fromraw <- genFromRaw traw (fromRawName sname) name cons
+  fromraw <- genFromRaw traw (fromRawName sname) name cons'
   return $ ddecl:toraw ++ fromraw
 
+declareLADT :: Name -> String -> [(String, String)] -> Q [Dec]
+declareLADT = declareADT Left
+
+declareILADT :: String -> [(String, Int)] -> Q [Dec]
+declareILADT sname cons = do
+  consNames <- sequence [ newName ('_':n) | (n, _) <- cons ]
+  consFns <- concat <$> sequence
+             [ do sig <- sigD n [t| Int |]
+                  let expr = litE (IntegerL (toInteger i))
+                  fn <- funD n [clause [] (normalB expr) []]
+                  return [sig, fn]
+             | n <- consNames
+             | (_, i) <- cons ]
+  let cons' = [ (n, n') | (n, _) <- cons | n' <- consNames ]
+  (consFns ++) <$> declareADT Right ''Int sname cons'
+
 declareIADT :: String -> [(String, Name)] -> Q [Dec]
-declareIADT = declareADT ''Int
+declareIADT = declareADT Right ''Int
 
 declareSADT :: String -> [(String, Name)] -> Q [Dec]
-declareSADT = declareADT ''String
+declareSADT = declareADT Right ''String
 
 -- | Creates the showJSON member of a JSON instance declaration.
 --
@@ -485,15 +569,27 @@ reifyConsNames name = do
 --
 -- This builds a custom list of name\/string pairs and then uses
 -- 'genToRaw' to actually generate the function.
-genConstrToStr :: (String -> String) -> Name -> String -> Q [Dec]
+genConstrToStr :: (String -> Q String) -> Name -> String -> Q [Dec]
 genConstrToStr trans_fun name fname = do
   cnames <- reifyConsNames name
-  let svalues = map (Left . trans_fun) cnames
+  svalues <- mapM (liftM Left . trans_fun) cnames
   genToRaw ''String (mkName fname) name $ zip cnames svalues
 
 -- | Constructor-to-string for OpCode.
 genOpID :: Name -> String -> Q [Dec]
-genOpID = genConstrToStr deCamelCase
+genOpID = genConstrToStr (return . deCamelCase)
+
+-- | Strips @Op@ from the constructor name, converts to lower-case
+-- and adds a given prefix.
+genOpLowerStrip :: String -> Name -> String -> Q [Dec]
+genOpLowerStrip prefix =
+    genConstrToStr (liftM ((prefix ++) . map toLower . deCamelCase)
+                    . stripPrefixM "Op")
+  where
+    stripPrefixM :: String -> String -> Q String
+    stripPrefixM pfx s = maybe (fail $ s ++ " doesn't start with " ++ pfx)
+                               return
+                         $ stripPrefix pfx s
 
 -- | Builds a list with all defined constructor names for a type.
 --
@@ -520,42 +616,124 @@ genAllOpIDs = genAllConstr deCamelCase
 -- | OpCode parameter (field) type.
 type OpParam = (String, Q Type, Q Exp)
 
+-- * Python code generation
+
+data OpCodeField = OpCodeField { ocfName :: String
+                               , ocfType :: PyType
+                               , ocfDefl :: Maybe PyValueEx
+                               , ocfDoc  :: String
+                               }
+
+-- | Transfers opcode data between the opcode description (through
+-- @genOpCode@) and the Python code generation functions.
+data OpCodeDescriptor = OpCodeDescriptor { ocdName   :: String
+                                         , ocdType   :: PyType
+                                         , ocdDoc    :: String
+                                         , ocdFields :: [OpCodeField]
+                                         , ocdDescr  :: String
+                                         }
+
+-- | Optionally encapsulates default values in @PyValueEx@.
+--
+-- @maybeApp exp typ@ returns a quoted expression that encapsulates
+-- the default value @exp@ of an opcode parameter cast to @typ@ in a
+-- @PyValueEx@, if @exp@ is @Just@.  Otherwise, it returns a quoted
+-- expression with @Nothing@.
+maybeApp :: Maybe (Q Exp) -> Q Type -> Q Exp
+maybeApp Nothing _ =
+  [| Nothing |]
+
+maybeApp (Just expr) typ =
+  [| Just ($(conE (mkName "PyValueEx")) ($expr :: $typ)) |]
+
+-- | Generates a Python type according to whether the field is
+-- optional.
+--
+-- The type of created expression is PyType.
+genPyType' :: OptionalType -> Q Type -> Q PyType
+genPyType' opt typ = typ >>= pyOptionalType (opt /= NotOptional)
+
+-- | Generates Python types from opcode parameters.
+genPyType :: Field -> Q PyType
+genPyType f = genPyType' (fieldIsOptional f) (fieldType f)
+
+-- | Generates Python default values from opcode parameters.
+genPyDefault :: Field -> Q Exp
+genPyDefault f = maybeApp (fieldDefault f) (fieldType f)
+
+pyField :: Field -> Q Exp
+pyField f = genPyType f >>= \t ->
+            [| OpCodeField $(stringE (fieldName f))
+                           t
+                           $(genPyDefault f)
+                           $(stringE (fieldDoc f)) |]
+
+-- | Generates a Haskell function call to "showPyClass" with the
+-- necessary information on how to build the Python class string.
+pyClass :: OpCodeConstructor -> Q Exp
+pyClass (consName, consType, consDoc, consFields, consDscField) =
+  do let pyClassVar = varNameE "showPyClass"
+         consName' = stringE consName
+     consType' <- genPyType' NotOptional consType
+     let consDoc' = stringE consDoc
+     [| OpCodeDescriptor $consName'
+                         consType'
+                         $consDoc'
+                         $(listE $ map pyField consFields)
+                         consDscField |]
+
+-- | Generates a function called "pyClasses" that holds the list of
+-- all the opcode descriptors necessary for generating the Python
+-- opcodes.
+pyClasses :: [OpCodeConstructor] -> Q [Dec]
+pyClasses cons =
+  do let name = mkName "pyClasses"
+         sig = SigD name (AppT ListT (ConT ''OpCodeDescriptor))
+     fn <- FunD name <$> (:[]) <$> declClause cons
+     return [sig, fn]
+  where declClause c =
+          clause [] (normalB (ListE <$> mapM pyClass c)) []
+
+-- | Converts from an opcode constructor to a Luxi constructor.
+opcodeConsToLuxiCons :: (a, b, c, d, e) -> (a, d)
+opcodeConsToLuxiCons (x, _, _, y, _) = (x, y)
+
 -- | Generates the OpCode data type.
 --
 -- This takes an opcode logical definition, and builds both the
 -- datatype and the JSON serialisation out of it. We can't use a
 -- generic serialisation since we need to be compatible with Ganeti's
 -- own, so we have a few quirks to work around.
-genOpCode :: String        -- ^ Type name to use
-          -> [Constructor] -- ^ Constructor name and parameters
+genOpCode :: String              -- ^ Type name to use
+          -> [OpCodeConstructor] -- ^ Constructor name and parameters
           -> Q [Dec]
 genOpCode name cons = do
   let tname = mkName name
-  decl_d <- mapM (\(cname, fields) -> do
+  decl_d <- mapM (\(cname, _, _, fields, _) -> do
                     -- we only need the type of the field, without Q
                     fields' <- mapM (fieldTypeInfo "op") fields
                     return $ RecC (mkName cname) fields')
             cons
   let declD = DataD [] tname [] decl_d [''Show, ''Eq]
-
   let (allfsig, allffn) = genAllOpFields "allOpFields" cons
   save_decs <- genSaveOpCode tname "saveOpCode" "toDictOpCode"
-               cons (uncurry saveConstructor) True
+               (map opcodeConsToLuxiCons cons) saveConstructor True
   (loadsig, loadfn) <- genLoadOpCode cons
-  return $ [declD, allfsig, allffn, loadsig, loadfn] ++ save_decs
+  pyDecls <- pyClasses cons
+  return $ [declD, allfsig, allffn, loadsig, loadfn] ++ save_decs ++ pyDecls
 
 -- | Generates the function pattern returning the list of fields for a
 -- given constructor.
-genOpConsFields :: Constructor -> Clause
-genOpConsFields (cname, fields) =
+genOpConsFields :: OpCodeConstructor -> Clause
+genOpConsFields (cname, _, _, fields, _) =
   let op_id = deCamelCase cname
       fvals = map (LitE . StringL) . sort . nub $
               concatMap (\f -> fieldName f:fieldExtraKeys f) fields
   in Clause [LitP (StringL op_id)] (NormalB $ ListE fvals) []
 
 -- | Generates a list of all fields of an opcode constructor.
-genAllOpFields  :: String        -- ^ Function name
-                -> [Constructor] -- ^ Object definition
+genAllOpFields  :: String              -- ^ Function name
+                -> [OpCodeConstructor] -- ^ Object definition
                 -> (Dec, Dec)
 genAllOpFields sname opdefs =
   let cclauses = map genOpConsFields opdefs
@@ -569,11 +747,9 @@ genAllOpFields sname opdefs =
 -- This matches the opcode with variables named the same as the
 -- constructor fields (just so that the spliced in code looks nicer),
 -- and passes those name plus the parameter definition to 'saveObjectField'.
-saveConstructor :: String    -- ^ The constructor name
-                -> [Field]   -- ^ The parameter definitions for this
-                             -- constructor
-                -> Q Clause  -- ^ Resulting clause
-saveConstructor sname fields = do
+saveConstructor :: LuxiConstructor -- ^ The constructor
+                -> Q Clause        -- ^ Resulting clause
+saveConstructor (sname, fields) = do
   let cname = mkName sname
   fnames <- mapM (newName . fieldVariable) fields
   let pat = conP cname (map varP fnames)
@@ -590,14 +766,14 @@ saveConstructor sname fields = do
 --
 -- This builds a per-constructor match clause that contains the
 -- respective constructor-serialisation code.
-genSaveOpCode :: Name                      -- ^ Object ype
-              -> String                    -- ^ To 'JSValue' function name
-              -> String                    -- ^ To 'JSObject' function name
-              -> [Constructor]             -- ^ Object definition
-              -> (Constructor -> Q Clause) -- ^ Constructor save fn
-              -> Bool                      -- ^ Whether to generate
-                                           -- obj or just a
-                                           -- list\/tuple of values
+genSaveOpCode :: Name                          -- ^ Object ype
+              -> String                        -- ^ To 'JSValue' function name
+              -> String                        -- ^ To 'JSObject' function name
+              -> [LuxiConstructor]             -- ^ Object definition
+              -> (LuxiConstructor -> Q Clause) -- ^ Constructor save fn
+              -> Bool                          -- ^ Whether to generate
+                                               -- obj or just a
+                                               -- list\/tuple of values
               -> Q [Dec]
 genSaveOpCode tname jvalstr tdstr opdefs fn gen_object = do
   tdclauses <- mapM fn opdefs
@@ -615,17 +791,17 @@ genSaveOpCode tname jvalstr tdstr opdefs fn gen_object = do
          , ValD (VarP jvalname) (NormalB jvalclause) []]
 
 -- | Generates load code for a single constructor of the opcode data type.
-loadConstructor :: String -> [Field] -> Q Exp
-loadConstructor sname fields = do
+loadConstructor :: OpCodeConstructor -> Q Exp
+loadConstructor (sname, _, _, fields, _) = do
   let name = mkName sname
-  fbinds <- mapM loadObjectField fields
+  fbinds <- mapM (loadObjectField fields) fields
   let (fnames, fstmts) = unzip fbinds
   let cval = foldl (\accu fn -> AppE accu (VarE fn)) (ConE name) fnames
       fstmts' = fstmts ++ [NoBindS (AppE (VarE 'return) cval)]
   return $ DoE fstmts'
 
 -- | Generates the loadOpCode function.
-genLoadOpCode :: [Constructor] -> Q (Dec, Dec)
+genLoadOpCode :: [OpCodeConstructor] -> Q (Dec, Dec)
 genLoadOpCode opdefs = do
   let fname = mkName "loadOpCode"
       arg1 = mkName "v"
@@ -635,10 +811,10 @@ genLoadOpCode opdefs = do
                                  (JSON.readJSON $(varE arg1)) |]
   st2 <- bindS (varP opid) [| $fromObjE $(varE objname) $(stringE "OP_ID") |]
   -- the match results (per-constructor blocks)
-  mexps <- mapM (uncurry loadConstructor) opdefs
+  mexps <- mapM loadConstructor opdefs
   fails <- [| fail $ "Unknown opcode " ++ $(varE opid) |]
-  let mpats = map (\(me, c) ->
-                       let mp = LitP . StringL . deCamelCase . fst $ c
+  let mpats = map (\(me, (consName, _, _, _, _)) ->
+                       let mp = LitP . StringL . deCamelCase $ consName
                        in Match mp (NormalB me) []
                   ) $ zip mexps opdefs
       defmatch = Match WildP (NormalB fails) []
@@ -651,11 +827,11 @@ genLoadOpCode opdefs = do
 
 -- | Constructor-to-string for LuxiOp.
 genStrOfOp :: Name -> String -> Q [Dec]
-genStrOfOp = genConstrToStr id
+genStrOfOp = genConstrToStr return
 
 -- | Constructor-to-string for MsgKeys.
 genStrOfKey :: Name -> String -> Q [Dec]
-genStrOfKey = genConstrToStr ensureLower
+genStrOfKey = genConstrToStr (return . ensureLower)
 
 -- | Generates the LuxiOp data type.
 --
@@ -670,7 +846,7 @@ genStrOfKey = genConstrToStr ensureLower
 --
 -- * type
 --
-genLuxiOp :: String -> [Constructor] -> Q [Dec]
+genLuxiOp :: String -> [LuxiConstructor] -> Q [Dec]
 genLuxiOp name cons = do
   let tname = mkName name
   decl_d <- mapM (\(cname, fields) -> do
@@ -688,7 +864,7 @@ genLuxiOp name cons = do
   return $ declD:save_decs ++ req_defs
 
 -- | Generates the \"save\" clause for entire LuxiOp constructor.
-saveLuxiConstructor :: Constructor -> Q Clause
+saveLuxiConstructor :: LuxiConstructor -> Q Clause
 saveLuxiConstructor (sname, fields) = do
   let cname = mkName sname
   fnames <- mapM (newName . fieldVariable) fields
@@ -721,7 +897,7 @@ buildObjectSerialisation :: String -> [Field] -> Q [Dec]
 buildObjectSerialisation sname fields = do
   let name = mkName sname
   savedecls <- genSaveObject saveObjectField sname fields
-  (loadsig, loadfn) <- genLoadObject loadObjectField sname fields
+  (loadsig, loadfn) <- genLoadObject (loadObjectField fields) sname fields
   shjson <- objectShowJSON sname
   rdjson <- objectReadJSON sname
   let instdecl = InstanceD [] (AppT (ConT ''JSON.JSON) (ConT name))
@@ -776,6 +952,7 @@ saveObjectField fvar field =
         Just fn -> [| let (actual, extra) = $fn $fvarE
                       in ($nameE, JSON.showJSON actual):extra
                     |]
+    AndRestArguments -> [| M.toList $(varE fvar) |]
   where nameE = stringE (fieldName field)
         fvarE = varE fvar
 
@@ -810,24 +987,30 @@ genLoadObject load_fn sname fields = do
             FunD funname [Clause [VarP arg1] (NormalB (DoE fstmts')) []])
 
 -- | Generates code for loading an object's field.
-loadObjectField :: Field -> Q (Name, Stmt)
-loadObjectField field = do
+loadObjectField :: [Field] -> Field -> Q (Name, Stmt)
+loadObjectField allFields field = do
   let name = fieldVariable field
+      names = map fieldVariable allFields
+      otherNames = listE . map stringE $ names \\ [name]
   fvar <- newName name
   -- these are used in all patterns below
   let objvar = varNameE "o"
       objfield = stringE (fieldName field)
       loadexp =
-        if fieldIsOptional field /= NotOptional
-          -- we treat both optional types the same, since
-          -- 'maybeFromObj' can deal with both missing and null values
-          -- appropriately (the same)
-          then [| $(varE 'maybeFromObj) $objvar $objfield |]
-          else case fieldDefault field of
+        case fieldIsOptional field of
+          NotOptional ->
+            case fieldDefault field of
                  Just defv ->
                    [| $(varE 'fromObjWithDefault) $objvar
                       $objfield $defv |]
                  Nothing -> [| $fromObjE $objvar $objfield |]
+          AndRestArguments -> [| return . M.fromList
+                                   $ filter (not . (`elem` $otherNames) . fst)
+                                            $objvar |]
+          _ -> [| $(varE 'maybeFromObj) $objvar $objfield |]
+          -- we treat both optional types the same, since
+          -- 'maybeFromObj' can deal with both missing and null values
+          -- appropriately (the same)
   bexp <- loadFn field loadexp objvar
 
   return (fvar, BindS (VarP fvar) bexp)
@@ -934,7 +1117,9 @@ loadPParamField field = do
   let objvar = varNameE "o"
       objfield = stringE name
       loadexp = [| $(varE 'maybeFromObj) $objvar $objfield |]
-  bexp <- loadFn field loadexp objvar
+      field' = field {fieldRead=fmap (appE (varE 'makeReadOptional))
+                                  $ fieldRead field}
+  bexp <- loadFn field' loadexp objvar
   return (fvar, BindS (VarP fvar) bexp)
 
 -- | Builds a simple declaration of type @n_x = fromMaybe f_x p_x@.
