@@ -101,6 +101,8 @@ $(genArbitrary ''PartialBeParams)
 
 $(genArbitrary ''AdminState)
 
+$(genArbitrary ''AdminStateSource)
+
 $(genArbitrary ''PartialNicParams)
 
 $(genArbitrary ''PartialNic)
@@ -125,6 +127,8 @@ instance Arbitrary Instance where
       <*> pure (GenericContainer Map.empty)
       -- admin_state
       <*> arbitrary
+      -- admin_state_source
+      <*> arbitrary
       -- nics
       <*> arbitrary
       -- disks
@@ -166,7 +170,7 @@ enhanceInstWithNets inst nets = do
   uuid <- arbitrary
   -- generate some more networks than the given ones
   num_more_nets <- choose (0,3)
-  more_nets <- vectorOf num_more_nets genName
+  more_nets <- vectorOf num_more_nets genUUID
   let genNic net = PartialNic mac ip nicparams net name uuid
       partial_nics = map (genNic . Just)
                          (List.nub (nets ++ more_nets))
@@ -228,6 +232,9 @@ instance Arbitrary ClusterBeParams where
 instance Arbitrary TagSet where
   arbitrary = Set.fromList <$> genTags
 
+instance Arbitrary IAllocatorParams where
+  arbitrary = return $ GenericContainer Map.empty
+
 $(genArbitrary ''Cluster)
 
 instance Arbitrary Network where
