@@ -83,6 +83,7 @@ case_UsersGroups = do
               \         constants.NODED_USER,\n\
               \         constants.RAPI_USER,\n\
               \         constants.CONFD_USER,\n\
+              \         constants.KVMD_USER,\n\
               \         constants.LUXID_USER,\n\
               \         constants.MOND_USER,\n\
               \        ]\n\
@@ -90,6 +91,7 @@ case_UsersGroups = do
               \          constants.NODED_GROUP,\n\
               \          constants.RAPI_GROUP,\n\
               \          constants.CONFD_GROUP,\n\
+              \          constants.KVMD_GROUP,\n\
               \          constants.LUXID_GROUP,\n\
               \          constants.MOND_GROUP,\n\
               \          constants.DAEMONS_GROUP,\n\
