@@ -30,10 +30,12 @@ import Test.Framework
 import System.Environment (getArgs)
 import System.Log.Logger
 
+import Test.AutoConf
 import Test.Ganeti.TestImports ()
 import Test.Ganeti.Attoparsec
 import Test.Ganeti.BasicTypes
 import Test.Ganeti.Common
+import Test.Ganeti.Constants
 import Test.Ganeti.Confd.Utils
 import Test.Ganeti.Confd.Types
 import Test.Ganeti.Daemon
@@ -53,11 +55,14 @@ import Test.Ganeti.Hypervisor.Xen.XmParser
 import Test.Ganeti.JSON
 import Test.Ganeti.Jobs
 import Test.Ganeti.JQueue
+import Test.Ganeti.Kvmd
 import Test.Ganeti.Luxi
 import Test.Ganeti.Network
 import Test.Ganeti.Objects
 import Test.Ganeti.OpCodes
+import Test.Ganeti.Query.Aliases
 import Test.Ganeti.Query.Filter
+import Test.Ganeti.Query.Instance
 import Test.Ganeti.Query.Language
 import Test.Ganeti.Query.Network
 import Test.Ganeti.Query.Query
@@ -87,9 +92,11 @@ defOpts = TestOptions
 -- | All our defined tests.
 allTests :: [Test]
 allTests =
-  [ testBasicTypes
+  [ testAutoConf
+  , testBasicTypes
   , testAttoparsec
   , testCommon
+  , testConstants
   , testConfd_Types
   , testConfd_Utils
   , testDaemon
@@ -112,11 +119,14 @@ allTests =
   , testJSON
   , testJobs
   , testJQueue
+  , testKvmd
   , testLuxi
   , testNetwork
   , testObjects
   , testOpCodes
+  , testQuery_Aliases
   , testQuery_Filter
+  , testQuery_Instance
   , testQuery_Language
   , testQuery_Network
   , testQuery_Query
