@@ -24,11 +24,13 @@
 import unittest
 import optparse
 
+from ganeti import errors
 from ganeti.client import gnt_cluster
 from ganeti import utils
 from ganeti import compat
 from ganeti import constants
 
+import mock
 import testutils
 
 
@@ -180,7 +182,7 @@ class TestEpo(unittest.TestCase):
     return self._OFF_EXITCODE
 
   def _Test(self, *args, **kwargs):
-    defaults = dict(cl=NotImplemented, _on_fn=NotImplemented,
+    defaults = dict(qcl=NotImplemented, _on_fn=NotImplemented,
                     _off_fn=NotImplemented,
                     _stdout_fn=lambda *args: None,
                     _stderr_fn=lambda *args: None)
@@ -214,7 +216,7 @@ class TestEpo(unittest.TestCase):
 
       off_fn = compat.partial(self._Off, ["node1.example.com"])
 
-      result = self._Test(opts, [], cl=client, _off_fn=off_fn,
+      result = self._Test(opts, [], qcl=client, _off_fn=off_fn,
                           _confirm_fn=confirm_fn)
       if force or confirm_result:
         self.assertEqual(result, self._OFF_EXITCODE)
@@ -244,7 +246,7 @@ class TestEpo(unittest.TestCase):
         self.assertFalse(inst_map)
         return self._ON_EXITCODE
 
-      result = self._Test(opts, [], cl=client, _on_fn=_On,
+      result = self._Test(opts, [], qcl=client, _on_fn=_On,
                           _confirm_fn=self._ConfirmForce)
       self.assertEqual(result, self._ON_EXITCODE)
 
@@ -254,9 +256,103 @@ class TestEpo(unittest.TestCase):
     client = _ClientForEpo(NotImplemented, [
       ("node1.example.com", True, [], [], True, False),
       ])
-    result = self._Test(opts, [], cl=client, _confirm_fn=self._ConfirmForce)
+    result = self._Test(opts, [], qcl=client, _confirm_fn=self._ConfirmForce)
     self.assertEqual(result, constants.EXIT_FAILURE)
 
 
+class DrbdHelperTestCase(unittest.TestCase):
+
+  def setUp(self):
+    unittest.TestCase.setUp(self)
+    self.enabled_disk_templates = []
+
+  def enableDrbd(self):
+    self.enabled_disk_templates = [constants.DT_DRBD8]
+
+  def disableDrbd(self):
+    self.enabled_disk_templates = [constants.DT_DISKLESS]
+
+
+class InitDrbdHelper(DrbdHelperTestCase):
+
+  def testNoDrbdNoHelper(self):
+    opts = mock.Mock()
+    opts.drbd_helper = None
+    self.disableDrbd()
+    helper = gnt_cluster._InitDrbdHelper(opts, self.enabled_disk_templates)
+    self.assertEquals(None, helper)
+
+  def testNoDrbdHelper(self):
+    opts = mock.Mock()
+    self.disableDrbd()
+    opts.drbd_helper = "/bin/true"
+    helper = gnt_cluster._InitDrbdHelper(opts, self.enabled_disk_templates)
+    self.assertEquals(opts.drbd_helper, helper)
+
+  def testDrbdHelperNone(self):
+    opts = mock.Mock()
+    self.enableDrbd()
+    opts.drbd_helper = None
+    helper = gnt_cluster._InitDrbdHelper(opts, self.enabled_disk_templates)
+    self.assertEquals(constants.DEFAULT_DRBD_HELPER, helper)
+
+  def testDrbdHelperEmpty(self):
+    opts = mock.Mock()
+    self.enableDrbd()
+    opts.drbd_helper = ''
+    self.assertRaises(errors.OpPrereqError, gnt_cluster._InitDrbdHelper, opts,
+        self.enabled_disk_templates)
+
+  def testDrbdHelper(self):
+    opts = mock.Mock()
+    self.enableDrbd()
+    opts.drbd_helper = "/bin/true"
+    helper = gnt_cluster._InitDrbdHelper(opts, self.enabled_disk_templates)
+    self.assertEquals(opts.drbd_helper, helper)
+
+
+class GetDrbdHelper(DrbdHelperTestCase):
+
+  def testNoDrbdNoHelper(self):
+    opts = mock.Mock()
+    self.disableDrbd()
+    opts.drbd_helper = None
+    helper = gnt_cluster._GetDrbdHelper(opts, self.enabled_disk_templates)
+    self.assertEquals(None, helper)
+
+  def testNoTemplateInfoNoHelper(self):
+    opts = mock.Mock()
+    opts.drbd_helper = None
+    helper = gnt_cluster._GetDrbdHelper(opts, None)
+    self.assertEquals(None, helper)
+
+  def testNoTemplateInfoHelper(self):
+    opts = mock.Mock()
+    opts.drbd_helper = "/bin/true"
+    helper = gnt_cluster._GetDrbdHelper(opts, None)
+    self.assertEquals(opts.drbd_helper, helper)
+
+  def testNoDrbdHelper(self):
+    opts = mock.Mock()
+    self.disableDrbd()
+    opts.drbd_helper = "/bin/true"
+    helper = gnt_cluster._GetDrbdHelper(opts, None)
+    self.assertEquals(opts.drbd_helper, helper)
+
+  def testDrbdNoHelper(self):
+    opts = mock.Mock()
+    self.enableDrbd()
+    opts.drbd_helper = None
+    helper = gnt_cluster._GetDrbdHelper(opts, self.enabled_disk_templates)
+    self.assertEquals(None, helper)
+
+  def testDrbdHelper(self):
+    opts = mock.Mock()
+    self.enableDrbd()
+    opts.drbd_helper = "/bin/true"
+    helper = gnt_cluster._GetDrbdHelper(opts, self.enabled_disk_templates)
+    self.assertEquals(opts.drbd_helper, helper)
+
+
 if __name__ == "__main__":
   testutils.GanetiTestProgram()
