@@ -242,6 +242,10 @@ class TestConfigRunner(unittest.TestCase):
         constants.ND_OOB_PROGRAM: "/bin/node-oob",
         constants.ND_SPINDLE_COUNT: 1,
         constants.ND_EXCLUSIVE_STORAGE: False,
+        constants.ND_OVS: True,
+        constants.ND_OVS_NAME: "openvswitch",
+        constants.ND_OVS_LINK: "eth1",
+        constants.ND_SSH_PORT: 22,
         }
 
     cfg = self._get_object()
@@ -253,15 +257,23 @@ class TestConfigRunner(unittest.TestCase):
   def testGetNdParamsInheritance(self):
     node_ndparams = {
       constants.ND_OOB_PROGRAM: "/bin/node-oob",
+      constants.ND_OVS_LINK: "eth3"
       }
     group_ndparams = {
       constants.ND_SPINDLE_COUNT: 10,
+      constants.ND_OVS: True,
+      constants.ND_OVS_NAME: "openvswitch",
+      constants.ND_SSH_PORT: 222,
       }
     expected_ndparams = {
       constants.ND_OOB_PROGRAM: "/bin/node-oob",
       constants.ND_SPINDLE_COUNT: 10,
       constants.ND_EXCLUSIVE_STORAGE:
         constants.NDC_DEFAULTS[constants.ND_EXCLUSIVE_STORAGE],
+      constants.ND_OVS: True,
+      constants.ND_OVS_NAME: "openvswitch",
+      constants.ND_OVS_LINK: "eth3",
+      constants.ND_SSH_PORT: 222,
       }
     cfg = self._get_object()
     node = cfg.GetNodeInfo(cfg.GetNodeList()[0])
