@@ -27,6 +27,7 @@ import unittest
 import socket
 import os
 import struct
+import re
 
 from ganeti import serializer
 from ganeti import constants
@@ -199,21 +200,24 @@ class TestQmp(testutils.GanetiTestCase):
 
 
 class TestConsole(unittest.TestCase):
-  def _Test(self, instance, node, hvparams):
-    cons = hv_kvm.KVMHypervisor.GetInstanceConsole(instance, node, hvparams, {})
-    self.assertTrue(cons.Validate())
+  def _Test(self, instance, node, group, hvparams):
+    cons = hv_kvm.KVMHypervisor.GetInstanceConsole(instance, node, group,
+                                                   hvparams, {})
+    self.assertEqual(cons.Validate(), None)
     return cons
 
   def testSerial(self):
     instance = objects.Instance(name="kvm.example.com",
                                 primary_node="node6017-uuid")
-    node = objects.Node(name="node6017", uuid="node6017-uuid")
+    node = objects.Node(name="node6017", uuid="node6017-uuid",
+                        ndparams={})
+    group = objects.NodeGroup(name="group6134", ndparams={})
     hvparams = {
       constants.HV_SERIAL_CONSOLE: True,
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, node, hvparams)
+    cons = self._Test(instance, node, group, hvparams)
     self.assertEqual(cons.kind, constants.CONS_SSH)
     self.assertEqual(cons.host, node.name)
     self.assertEqual(cons.command[0], pathutils.KVM_CONSOLE_WRAPPER)
@@ -223,13 +227,15 @@ class TestConsole(unittest.TestCase):
     instance = objects.Instance(name="kvm.example.com",
                                 primary_node="node7235-uuid",
                                 network_port=constants.VNC_BASE_PORT + 10)
-    node = objects.Node(name="node7235", uuid="node7235-uuid")
+    node = objects.Node(name="node7235", uuid="node7235-uuid",
+                        ndparams={})
+    group = objects.NodeGroup(name="group3632", ndparams={})
     hvparams = {
       constants.HV_SERIAL_CONSOLE: False,
       constants.HV_VNC_BIND_ADDRESS: "192.0.2.1",
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, node, hvparams)
+    cons = self._Test(instance, node, group, hvparams)
     self.assertEqual(cons.kind, constants.CONS_VNC)
     self.assertEqual(cons.host, "192.0.2.1")
     self.assertEqual(cons.port, constants.VNC_BASE_PORT + 10)
@@ -239,13 +245,15 @@ class TestConsole(unittest.TestCase):
     instance = objects.Instance(name="kvm.example.com",
                                 primary_node="node7235",
                                 network_port=11000)
-    node = objects.Node(name="node7235", uuid="node7235-uuid")
+    node = objects.Node(name="node7235", uuid="node7235-uuid",
+                        ndparams={})
+    group = objects.NodeGroup(name="group0132", ndparams={})
     hvparams = {
       constants.HV_SERIAL_CONSOLE: False,
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: "192.0.2.1",
       }
-    cons = self._Test(instance, node, hvparams)
+    cons = self._Test(instance, node, group, hvparams)
     self.assertEqual(cons.kind, constants.CONS_SPICE)
     self.assertEqual(cons.host, "192.0.2.1")
     self.assertEqual(cons.port, 11000)
@@ -254,13 +262,15 @@ class TestConsole(unittest.TestCase):
     instance = objects.Instance(name="kvm.example.com",
                                 primary_node="node24325",
                                 network_port=0)
-    node = objects.Node(name="node24325", uuid="node24325-uuid")
+    node = objects.Node(name="node24325", uuid="node24325-uuid",
+                        ndparams={})
+    group = objects.NodeGroup(name="group9184", ndparams={})
     hvparams = {
       constants.HV_SERIAL_CONSOLE: False,
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, node, hvparams)
+    cons = self._Test(instance, node, group, hvparams)
     self.assertEqual(cons.kind, constants.CONS_MESSAGE)
 
 
@@ -384,5 +394,52 @@ class TestProbeTapVnetHdr(unittest.TestCase):
     self.assertFalse(hv_kvm._ProbeTapVnetHdr(fd, _features_fn=lambda _: None))
 
 
+class TestGenerateDeviceKVMId(unittest.TestCase):
+  def test(self):
+    device = objects.NIC()
+    target = constants.HOTPLUG_TARGET_NIC
+    fn = hv_kvm._GenerateDeviceKVMId
+    self.assertRaises(errors.HotplugError, fn, target, device)
+
+    device.pci = 5
+    device.uuid = "003fc157-66a8-4e6d-8b7e-ec4f69751396"
+    self.assertTrue(re.match("hotnic-003fc157-pci-5", fn(target, device)))
+
+
+class TestGetRuntimeInfo(unittest.TestCase):
+  @classmethod
+  def _GetRuntime(cls):
+    data = testutils.ReadTestData("kvm_runtime.json")
+    return hv_kvm._AnalyzeSerializedRuntime(data)
+
+  def _fail(self, target, device, runtime):
+    device.uuid = "aaaaaaaa-66a8-4e6d-8b7e-ec4f69751396"
+    self.assertRaises(errors.HotplugError,
+                      hv_kvm._GetExistingDeviceInfo,
+                      target, device, runtime)
+
+  def testNIC(self):
+    device = objects.NIC()
+    target = constants.HOTPLUG_TARGET_NIC
+    runtime = self._GetRuntime()
+
+    self._fail(target, device, runtime)
+
+    device.uuid = "003fc157-66a8-4e6d-8b7e-ec4f69751396"
+    devinfo = hv_kvm._GetExistingDeviceInfo(target, device, runtime)
+    self.assertTrue(devinfo.pci==6)
+
+  def testDisk(self):
+    device = objects.Disk()
+    target = constants.HOTPLUG_TARGET_DISK
+    runtime = self._GetRuntime()
+
+    self._fail(target, device, runtime)
+
+    device.uuid = "9f5c5bd4-6f60-480b-acdc-9bb1a4b7df79"
+    (devinfo, _, __) = hv_kvm._GetExistingDeviceInfo(target, device, runtime)
+    self.assertTrue(devinfo.pci==5)
+
+
 if __name__ == "__main__":
   testutils.GanetiTestProgram()
