@@ -37,6 +37,7 @@ from ganeti import utils
 from ganeti import errors
 from ganeti import compat
 
+from ganeti.hypervisor import hv_base
 from ganeti.hypervisor import hv_xen
 
 import testutils
@@ -52,9 +53,11 @@ class TestConsole(unittest.TestCase):
     for cls in [hv_xen.XenPvmHypervisor(), hv_xen.XenHvmHypervisor()]:
       instance = objects.Instance(name="xen.example.com",
                                   primary_node="node24828-uuid")
-      node = objects.Node(name="node24828", uuid="node24828-uuid")
-      cons = cls.GetInstanceConsole(instance, node, hvparams, {})
-      self.assertTrue(cons.Validate())
+      node = objects.Node(name="node24828", uuid="node24828-uuid",
+                          ndparams={})
+      group = objects.NodeGroup(name="group52341", ndparams={})
+      cons = cls.GetInstanceConsole(instance, node, group, hvparams, {})
+      self.assertEqual(cons.Validate(), None)
       self.assertEqual(cons.kind, constants.CONS_SSH)
       self.assertEqual(cons.host, node.name)
       self.assertEqual(cons.command[-1], instance.name)
@@ -137,7 +140,7 @@ class TestParseInstanceList(testutils.GanetiTestCase):
     self.assertEqual(result[0][3], 1)
 
     # State
-    self.assertEqual(result[0][4], "r-----")
+    self.assertEqual(result[0][4], hv_base.HvInstanceState.RUNNING)
 
     # Time
     self.assertAlmostEqual(result[0][5], 121152.6)
@@ -166,7 +169,7 @@ class TestGetInstanceList(testutils.GanetiTestCase):
   def testTimeout(self):
     fn = testutils.CallCounter(self._Fail)
     try:
-      hv_xen._GetInstanceList(fn, False, _timeout=0.1)
+      hv_xen._GetRunningInstanceList(fn, False, _timeout=0.1)
     except errors.HypervisorError, err:
       self.assertTrue("timeout exceeded" in str(err))
     else:
@@ -184,7 +187,7 @@ class TestGetInstanceList(testutils.GanetiTestCase):
 
     fn = testutils.CallCounter(compat.partial(self._Success, data))
 
-    result = hv_xen._GetInstanceList(fn, True, _timeout=0.1)
+    result = hv_xen._GetRunningInstanceList(fn, True, _timeout=0.1)
 
     self.assertEqual(len(result), 4)
 
@@ -269,7 +272,9 @@ class TestGetConfigFileDiskData(unittest.TestCase):
 
   def testManyDisks(self):
     for offset in [0, 1, 10]:
-      disks = [(objects.Disk(dev_type=constants.DT_PLAIN), "/tmp/disk/%s" % idx)
+      disks = [(objects.Disk(dev_type=constants.DT_PLAIN),
+               "/tmp/disk/%s" % idx,
+               NotImplemented)
                for idx in range(len(hv_xen._DISK_LETTERS) + offset)]
 
       if offset == 0:
@@ -289,9 +294,11 @@ class TestGetConfigFileDiskData(unittest.TestCase):
   def testTwoLvDisksWithMode(self):
     disks = [
       (objects.Disk(dev_type=constants.DT_PLAIN, mode=constants.DISK_RDWR),
-       "/tmp/diskFirst"),
+       "/tmp/diskFirst",
+       NotImplemented),
       (objects.Disk(dev_type=constants.DT_PLAIN, mode=constants.DISK_RDONLY),
-       "/tmp/diskLast"),
+       "/tmp/diskLast",
+       NotImplemented),
       ]
 
     result = hv_xen._GetConfigFileDiskData(disks, "hd")
@@ -303,20 +310,25 @@ class TestGetConfigFileDiskData(unittest.TestCase):
   def testFileDisks(self):
     disks = [
       (objects.Disk(dev_type=constants.DT_FILE, mode=constants.DISK_RDWR,
-                    physical_id=[constants.FD_LOOP]),
-       "/tmp/diskFirst"),
+                    logical_id=[constants.FD_LOOP]),
+       "/tmp/diskFirst",
+       NotImplemented),
       (objects.Disk(dev_type=constants.DT_FILE, mode=constants.DISK_RDONLY,
-                    physical_id=[constants.FD_BLKTAP]),
-       "/tmp/diskTwo"),
+                    logical_id=[constants.FD_BLKTAP]),
+       "/tmp/diskTwo",
+       NotImplemented),
       (objects.Disk(dev_type=constants.DT_FILE, mode=constants.DISK_RDWR,
-                    physical_id=[constants.FD_LOOP]),
-       "/tmp/diskThree"),
+                    logical_id=[constants.FD_LOOP]),
+       "/tmp/diskThree",
+       NotImplemented),
       (objects.Disk(dev_type=constants.DT_FILE, mode=constants.DISK_RDONLY,
-                    physical_id=[constants.FD_BLKTAP2]),
-       "/tmp/diskFour"),
+                    logical_id=[constants.FD_BLKTAP2]),
+       "/tmp/diskFour",
+       NotImplemented),
       (objects.Disk(dev_type=constants.DT_FILE, mode=constants.DISK_RDWR,
-                    physical_id=[constants.FD_BLKTAP]),
-       "/tmp/diskLast"),
+                    logical_id=[constants.FD_BLKTAP]),
+       "/tmp/diskLast",
+       NotImplemented),
       ]
 
     result = hv_xen._GetConfigFileDiskData(disks, "sd")
@@ -331,8 +343,9 @@ class TestGetConfigFileDiskData(unittest.TestCase):
   def testInvalidFileDisk(self):
     disks = [
       (objects.Disk(dev_type=constants.DT_FILE, mode=constants.DISK_RDWR,
-                    physical_id=["#unknown#"]),
-       "/tmp/diskinvalid"),
+                    logical_id=["#unknown#"]),
+       "/tmp/diskinvalid",
+       NotImplemented),
       ]
 
     self.assertRaises(KeyError, hv_xen._GetConfigFileDiskData, disks, "sd")
@@ -374,7 +387,7 @@ class TestXenHypervisorGetInstanceList(unittest.TestCase):
 
   def testNoHvparams(self):
     expected_xen_cmd = "xm"
-    mock_run_cmd = mock.Mock( return_value=self.RESULT_OK )
+    mock_run_cmd = mock.Mock(return_value=self.RESULT_OK)
     hv = hv_xen.XenHypervisor(_cfgdir=NotImplemented,
                               _run_cmd_fn=mock_run_cmd)
     self.assertRaises(errors.HypervisorError, hv._GetInstanceList, True, None)
@@ -382,7 +395,7 @@ class TestXenHypervisorGetInstanceList(unittest.TestCase):
   def testFromHvparams(self):
     expected_xen_cmd = "xl"
     hvparams = {constants.HV_XEN_CMD: constants.XEN_CMD_XL}
-    mock_run_cmd = mock.Mock( return_value=self.RESULT_OK )
+    mock_run_cmd = mock.Mock(return_value=self.RESULT_OK)
     hv = hv_xen.XenHypervisor(_cfgdir=NotImplemented,
                               _run_cmd_fn=mock_run_cmd)
     hv._GetInstanceList(True, hvparams)
@@ -396,7 +409,7 @@ class TestXenHypervisorListInstances(unittest.TestCase):
 
   def testNoHvparams(self):
     expected_xen_cmd = "xm"
-    mock_run_cmd = mock.Mock( return_value=self.RESULT_OK )
+    mock_run_cmd = mock.Mock(return_value=self.RESULT_OK)
     hv = hv_xen.XenHypervisor(_cfgdir=NotImplemented,
                               _run_cmd_fn=mock_run_cmd)
     self.assertRaises(errors.HypervisorError, hv.ListInstances)
@@ -404,7 +417,7 @@ class TestXenHypervisorListInstances(unittest.TestCase):
   def testHvparamsXl(self):
     expected_xen_cmd = "xl"
     hvparams = {constants.HV_XEN_CMD: constants.XEN_CMD_XL}
-    mock_run_cmd = mock.Mock( return_value=self.RESULT_OK )
+    mock_run_cmd = mock.Mock(return_value=self.RESULT_OK)
     hv = hv_xen.XenHypervisor(_cfgdir=NotImplemented,
                               _run_cmd_fn=mock_run_cmd)
     hv.ListInstances(hvparams=hvparams)
@@ -441,7 +454,7 @@ class TestXenHypervisorCheckToolstack(unittest.TestCase):
   def testCheckToolstackXlNotConfigured(self):
     RESULT_FAILED = utils.RunResult(
         1, None, "",
-        "ERROR:  A different toolstack (xm) have been selected!",
+        "ERROR:  A different toolstack (xm) has been selected!",
         "", None, None)
     mock_run_cmd = mock.Mock(return_value=RESULT_FAILED)
     hv = hv_xen.XenHypervisor(_cfgdir=NotImplemented,
@@ -605,7 +618,7 @@ class _TestXenHypervisor(object):
     self.assertEqual(instid, 1)
     self.assertEqual(memory, 1024)
     self.assertEqual(vcpus, 1)
-    self.assertEqual(state, "-b----")
+    self.assertEqual(state, hv_base.HvInstanceState.RUNNING)
     self.assertAlmostEqual(runtime, 167643.2)
 
   def testGetInstanceInfoDom0(self):
@@ -620,7 +633,7 @@ class _TestXenHypervisor(object):
     self.assertEqual(instid, 0)
     self.assertEqual(memory, 1023)
     self.assertEqual(vcpus, 1)
-    self.assertEqual(state, "r-----")
+    self.assertEqual(state, hv_base.HvInstanceState.RUNNING)
     self.assertAlmostEqual(runtime, 154706.1)
 
   def testGetInstanceInfoUnknown(self):
@@ -683,9 +696,11 @@ class _TestXenHypervisor(object):
 
     disks = [
       (objects.Disk(dev_type=constants.DT_PLAIN, mode=constants.DISK_RDWR),
-       utils.PathJoin(self.tmpdir, "disk0")),
+       utils.PathJoin(self.tmpdir, "disk0"),
+       NotImplemented),
       (objects.Disk(dev_type=constants.DT_PLAIN, mode=constants.DISK_RDONLY),
-       utils.PathJoin(self.tmpdir, "disk1")),
+       utils.PathJoin(self.tmpdir, "disk1"),
+       NotImplemented),
       ]
 
     inst = objects.Instance(name="server01.example.com",
@@ -730,7 +745,14 @@ class _TestXenHypervisor(object):
           extra = inst.hvparams[constants.HV_KERNEL_ARGS]
           self.assertTrue(("extra = '%s'" % extra) in lines)
 
-  def _StopInstanceCommand(self, instance_name, force, fail, cmd):
+  def _StopInstanceCommand(self, instance_name, force, fail, full_cmd):
+    # Remove the timeout (and its number of seconds) if it's there
+    if full_cmd[:1][0] == "timeout":
+      cmd = full_cmd[2:]
+    else:
+      cmd = full_cmd
+
+    # Test the actual command
     if (cmd == [self.CMD, "list"]):
       output = "Name  ID  Mem  VCPUs  State  Time(s)\n" \
         "Domain-0  0  1023  1  r-----  142691.0\n" \
@@ -767,7 +789,8 @@ class _TestXenHypervisor(object):
 
         if fail:
           try:
-            hv._StopInstance(name, force, None)
+            hv._StopInstance(name, force, None,
+                             constants.DEFAULT_SHUTDOWN_TIMEOUT)
           except errors.HypervisorError, err:
             self.assertTrue(str(err).startswith("listing instances failed"),
                             msg=str(err))
@@ -777,7 +800,8 @@ class _TestXenHypervisor(object):
                            msg=("Configuration was removed when stopping"
                                 " instance failed"))
         else:
-          hv._StopInstance(name, force, None)
+          hv._StopInstance(name, force, None,
+                           constants.DEFAULT_SHUTDOWN_TIMEOUT)
           self.assertFalse(os.path.exists(cfgfile))
 
   def _MigrateNonRunningInstCmd(self, cmd):
