@@ -174,6 +174,10 @@ class TestClusterObject(unittest.TestCase):
         constants.ND_OOB_PROGRAM: "/bin/group-oob",
         constants.ND_SPINDLE_COUNT: 10,
         constants.ND_EXCLUSIVE_STORAGE: True,
+        constants.ND_OVS: True,
+        constants.ND_OVS_LINK: "eth2",
+        constants.ND_OVS_NAME: "openvswitch",
+        constants.ND_SSH_PORT: 122,
         }
     fake_group = objects.NodeGroup(name="testgroup",
                                    ndparams=group_ndparams)
@@ -185,6 +189,10 @@ class TestClusterObject(unittest.TestCase):
         constants.ND_OOB_PROGRAM: "/bin/node-oob",
         constants.ND_SPINDLE_COUNT: 2,
         constants.ND_EXCLUSIVE_STORAGE: True,
+        constants.ND_OVS: True,
+        constants.ND_OVS_LINK: "eth2",
+        constants.ND_OVS_NAME: "openvswitch",
+        constants.ND_SSH_PORT: 222,
         }
     fake_node = objects.Node(name="test",
                              ndparams=node_ndparams,
@@ -199,6 +207,10 @@ class TestClusterObject(unittest.TestCase):
         constants.ND_OOB_PROGRAM: "/bin/node-oob",
         constants.ND_SPINDLE_COUNT: 5,
         constants.ND_EXCLUSIVE_STORAGE: True,
+        constants.ND_OVS: True,
+        constants.ND_OVS_LINK: "eth2",
+        constants.ND_OVS_NAME: "openvswitch",
+        constants.ND_SSH_PORT: 322,
         }
     fake_node = objects.Node(name="test",
                              ndparams=node_ndparams,
@@ -206,6 +218,7 @@ class TestClusterObject(unittest.TestCase):
     group_ndparams = {
         constants.ND_OOB_PROGRAM: "/bin/group-oob",
         constants.ND_SPINDLE_COUNT: 4,
+        constants.ND_SSH_PORT: 422,
         }
     fake_group = objects.NodeGroup(name="testgroup",
                                    ndparams=group_ndparams)
@@ -347,10 +360,10 @@ class TestInstance(unittest.TestCase):
 
   def testNodesDrbdDisks(self):
     inst = objects.Instance(name="fakeinstdrbd.example.com",
-      primary_node="node10.example.com",
+      primary_node="node20.example.com",
       disks=[
         objects.Disk(dev_type=constants.DT_DRBD8, size=786432,
-          logical_id=("node10.example.com", "node15.example.com",
+          logical_id=("node20.example.com", "node15.example.com",
                       12300, 0, 0, "secret"),
           children=[
             objects.Disk(dev_type=constants.DT_PLAIN, size=786432,
@@ -740,7 +753,7 @@ class TestDisk(unittest.TestCase):
   def testUpgradeConfigDevTypeLegacyUnchanged(self):
     dev_types = [constants.DT_FILE, constants.DT_SHARED_FILE,
                  constants.DT_BLOCK, constants.DT_EXT,
-                 constants.DT_RBD]
+                 constants.DT_RBD, constants.DT_GLUSTER]
     for dev_type in dev_types:
       disk = objects.Disk()
       disk.dev_type = dev_type
