@@ -44,7 +44,7 @@ class TestFillOpcode(unittest.TestCase):
     for static in [None, {}]:
       op = baserlib.FillOpcode(self.OpTest, {}, static)
       self.assertTrue(isinstance(op, self.OpTest))
-      self.assertFalse(hasattr(op, "test"))
+      self.assertTrue(op.test is None)
 
   def testStatic(self):
     op = baserlib.FillOpcode(self.OpTest, {}, {"test": "abc"})
@@ -107,7 +107,7 @@ class TestOpcodeResource(unittest.TestCase):
   @staticmethod
   def _GetMethodAttributes(method):
     attrs = ["%s_OPCODE" % method, "%s_RENAME" % method,
-             "Get%sOpInput" % method.capitalize()]
+             "%s_ALIASES" % method, "Get%sOpInput" % method.capitalize()]
     assert attrs == dict((opattrs[0], list(opattrs[1:]))
                          for opattrs in baserlib.OPCODE_ATTRS)[method]
     return attrs
