@@ -503,8 +503,8 @@ class GanetiRapiClientTests(testutils.GanetiTestCase):
   def testInstancesMultiAlloc(self):
     response = {
       constants.JOB_IDS_KEY: ["23423"],
-      opcodes.OpInstanceMultiAlloc.ALLOCATABLE_KEY: ["foobar"],
-      opcodes.OpInstanceMultiAlloc.FAILED_KEY: ["foobar2"],
+      constants.ALLOCATABLE_KEY: ["foobar"],
+      constants.FAILED_KEY: ["foobar2"],
       }
     self.rapi.AddResponse(serializer.DumpJson(response))
     insts = [self.client.InstanceAllocation("create", "foobar",
@@ -1271,13 +1271,15 @@ class GanetiRapiClientTests(testutils.GanetiTestCase):
     for mnh in [None, False, True]:
       self.rapi.AddResponse("14470")
       self.assertEqual(14470,
-        self.client.ModifyCluster(maintain_node_health=mnh))
+        self.client.ModifyCluster(maintain_node_health=mnh,
+                                  reason="PinkBunniesInvasion"))
       self.assertHandler(rlib2.R_2_cluster_modify)
       self.assertItems([])
       data = serializer.LoadJson(self.rapi.GetLastRequestData())
       self.assertEqual(len(data), 1)
       self.assertEqual(data["maintain_node_health"], mnh)
       self.assertEqual(self.rapi.CountPending(), 0)
+      self.assertQuery("reason", ["PinkBunniesInvasion"])
 
   def testRedistributeConfig(self):
     self.rapi.AddResponse("3364")
