@@ -33,7 +33,7 @@ from ganeti import opcodes
 from ganeti import compat
 from ganeti import http
 from ganeti import query
-from ganeti import luxi
+import ganeti.rpc.errors as rpcerr
 from ganeti import errors
 from ganeti import rapi
 
@@ -107,7 +107,7 @@ class TestConstants(unittest.TestCase):
 class TestClientConnectError(unittest.TestCase):
   @staticmethod
   def _FailingClient(address=None):
-    raise luxi.NoMasterError("test")
+    raise rpcerr.NoMasterError("test")
 
   def test(self):
     resources = [
@@ -261,7 +261,7 @@ class TestNodeEvacuate(unittest.TestCase):
     handler = _CreateHandler(rlib2.R_2_nodes_name_evacuate, ["node92"], {
       "dry-run": ["1"],
       }, {
-      "mode": constants.IALLOCATOR_NEVAC_SEC,
+      "mode": constants.NODE_EVAC_SEC,
       }, clfactory)
     job_id = handler.POST()
 
@@ -272,7 +272,7 @@ class TestNodeEvacuate(unittest.TestCase):
     self.assertEqual(job_id, exp_job_id)
     self.assertTrue(isinstance(op, opcodes.OpNodeEvacuate))
     self.assertEqual(op.node_name, "node92")
-    self.assertEqual(op.mode, constants.IALLOCATOR_NEVAC_SEC)
+    self.assertEqual(op.mode, constants.NODE_EVAC_SEC)
     self.assertTrue(op.dry_run)
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -468,7 +468,7 @@ class TestInstanceActivateDisks(unittest.TestCase):
     self.assertTrue(isinstance(op, opcodes.OpInstanceActivateDisks))
     self.assertEqual(op.instance_name, "xyz")
     self.assertTrue(op.ignore_size)
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
 
@@ -487,8 +487,8 @@ class TestInstanceDeactivateDisks(unittest.TestCase):
     self.assertEqual(job_id, exp_job_id)
     self.assertTrue(isinstance(op, opcodes.OpInstanceDeactivateDisks))
     self.assertEqual(op.instance_name, "inst22357")
-    self.assertFalse(hasattr(op, "dry_run"))
-    self.assertFalse(hasattr(op, "force"))
+    self.assertFalse(op.dry_run)
+    self.assertFalse(op.force)
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
 
@@ -507,7 +507,7 @@ class TestInstanceRecreateDisks(unittest.TestCase):
     self.assertEqual(job_id, exp_job_id)
     self.assertTrue(isinstance(op, opcodes.OpInstanceRecreateDisks))
     self.assertEqual(op.instance_name, "inst22357")
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -527,7 +527,7 @@ class TestInstanceFailover(unittest.TestCase):
     self.assertEqual(job_id, exp_job_id)
     self.assertTrue(isinstance(op, opcodes.OpInstanceFailover))
     self.assertEqual(op.instance_name, "inst12794")
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -552,12 +552,46 @@ class TestInstanceDiskGrow(unittest.TestCase):
     self.assertEqual(op.instance_name, "inst10742")
     self.assertEqual(op.disk, 3)
     self.assertEqual(op.amount, 1024)
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
 
 
+class TestInstanceModify(unittest.TestCase):
+  def testCustomParamRename(self):
+    clfactory = _FakeClientFactory(_FakeClient)
+
+    name = "instant_instance"
+    data = {
+      "custom_beparams": {},
+      "custom_hvparams": {},
+      }
+
+    handler = _CreateHandler(rlib2.R_2_instances_name_modify, [name], {}, data,
+                             clfactory)
+    job_id = handler.PUT()
+
+    cl = clfactory.GetNextClient()
+    self.assertRaises(IndexError, clfactory.GetNextClient)
+
+    (exp_job_id, (op, )) = cl.GetNextSubmittedJob()
+    self.assertEqual(job_id, exp_job_id)
+
+    self.assertTrue(isinstance(op, opcodes.OpInstanceSetParams))
+    self.assertEqual(op.beparams, {})
+    self.assertEqual(op.hvparams, {})
+
+    self.assertRaises(IndexError, cl.GetNextSubmittedJob)
+
+    # Define both
+    data["beparams"] = {}
+    assert "beparams" in data and "custom_beparams" in data
+    handler = _CreateHandler(rlib2.R_2_instances_name_modify, [name], {}, data,
+                             clfactory)
+    self.assertRaises(http.HttpBadRequest, handler.PUT)
+
+
 class TestBackupPrepare(unittest.TestCase):
   def test(self):
     clfactory = _FakeClientFactory(_FakeClient)
@@ -576,7 +610,7 @@ class TestBackupPrepare(unittest.TestCase):
     self.assertTrue(isinstance(op, opcodes.OpBackupPrepare))
     self.assertEqual(op.instance_name, "inst17925")
     self.assertEqual(op.mode, constants.EXPORT_MODE_REMOTE)
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -622,7 +656,7 @@ class TestStorageQuery(unittest.TestCase):
     self.assertEqual(op.nodes, ["node21075"])
     self.assertEqual(op.storage_type, constants.ST_LVM_PV)
     self.assertEqual(op.output_fields, ["name", "other"])
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -630,11 +664,12 @@ class TestStorageQuery(unittest.TestCase):
   def testErrors(self):
     clfactory = _FakeClientFactory(_FakeClient)
 
+    # storage type which does not support space reporting
     queryargs = {
-      "output_fields": "name,other",
+      "storage_type": constants.ST_DISKLESS,
       }
     handler = _CreateHandler(rlib2.R_2_nodes_name_storage,
-                             ["node10538"], queryargs, {}, clfactory)
+                             ["node21273"], queryargs, {}, clfactory)
     self.assertRaises(http.HttpBadRequest, handler.GET)
 
     queryargs = {
@@ -686,7 +721,7 @@ class TestStorageModify(unittest.TestCase):
         self.assertEqual(op.changes, {
           constants.SF_ALLOCATABLE: (allocatable == "1"),
           })
-      self.assertFalse(hasattr(op, "dry_run"))
+      self.assertFalse(op.dry_run)
       self.assertFalse(hasattr(op, "force"))
 
       self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -741,7 +776,7 @@ class TestStorageRepair(unittest.TestCase):
     self.assertEqual(op.node_name, "node19265")
     self.assertEqual(op.storage_type, constants.ST_LVM_PV)
     self.assertEqual(op.name, "pv16611")
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -920,12 +955,12 @@ class TestInstanceCreation(testutils.GanetiTestCase):
                     self.assertFalse("foobar" in opnic)
 
                   if beparams is None:
-                    self.assertFalse(hasattr(op, "beparams"))
+                    self.assertTrue(op.beparams in [None, {}])
                   else:
                     self.assertEqualValues(op.beparams, beparams)
 
                   if hvparams is None:
-                    self.assertFalse(hasattr(op, "hvparams"))
+                    self.assertTrue(op.hvparams in [None, {}])
                   else:
                     self.assertEqualValues(op.hvparams, hvparams)
 
@@ -1011,7 +1046,6 @@ class TestInstanceCreation(testutils.GanetiTestCase):
       "disks": [],
       "nics": [],
       "mode": constants.INSTANCE_CREATE,
-      "disk_template": constants.DT_PLAIN,
       }
 
     for name in reqfields.keys():
@@ -1102,7 +1136,7 @@ class TestBackupExport(unittest.TestCase):
     self.assertEqual(op.remove_instance, True)
     self.assertEqual(op.x509_key_name, ["name", "hash"])
     self.assertEqual(op.destination_x509_ca, "---cert---")
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -1128,10 +1162,10 @@ class TestBackupExport(unittest.TestCase):
     self.assertTrue(isinstance(op, opcodes.OpBackupExport))
     self.assertEqual(op.instance_name, name)
     self.assertEqual(op.target_node, "node2")
-    self.assertFalse(hasattr(op, "mode"))
-    self.assertFalse(hasattr(op, "remove_instance"))
+    self.assertEqual(op.mode, "local")
+    self.assertFalse(op.remove_instance)
     self.assertFalse(hasattr(op, "destination"))
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -1172,7 +1206,7 @@ class TestInstanceMigrate(testutils.GanetiTestCase):
         self.assertEqual(op.instance_name, name)
         self.assertEqual(op.mode, mode)
         self.assertEqual(op.cleanup, cleanup)
-        self.assertFalse(hasattr(op, "dry_run"))
+        self.assertFalse(op.dry_run)
         self.assertFalse(hasattr(op, "force"))
 
         self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -1193,9 +1227,9 @@ class TestInstanceMigrate(testutils.GanetiTestCase):
     self.assertEqual(job_id, exp_job_id)
     self.assertTrue(isinstance(op, opcodes.OpInstanceMigrate))
     self.assertEqual(op.instance_name, name)
-    self.assertFalse(hasattr(op, "mode"))
-    self.assertFalse(hasattr(op, "cleanup"))
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertTrue(op.mode is None)
+    self.assertFalse(op.cleanup)
+    self.assertFalse(op.dry_run)
     self.assertFalse(hasattr(op, "force"))
 
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -1230,7 +1264,7 @@ class TestParseRenameInstanceRequest(testutils.GanetiTestCase):
           self.assertEqual(op.new_name, new_name)
           self.assertEqual(op.ip_check, ip_check)
           self.assertEqual(op.name_check, name_check)
-          self.assertFalse(hasattr(op, "dry_run"))
+          self.assertFalse(op.dry_run)
           self.assertFalse(hasattr(op, "force"))
 
           self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -1257,9 +1291,9 @@ class TestParseRenameInstanceRequest(testutils.GanetiTestCase):
       self.assertTrue(isinstance(op, opcodes.OpInstanceRename))
       self.assertEqual(op.instance_name, name)
       self.assertEqual(op.new_name, new_name)
-      self.assertFalse(hasattr(op, "ip_check"))
-      self.assertFalse(hasattr(op, "name_check"))
-      self.assertFalse(hasattr(op, "dry_run"))
+      self.assertTrue(op.ip_check)
+      self.assertTrue(op.name_check)
+      self.assertFalse(op.dry_run)
       self.assertFalse(hasattr(op, "force"))
 
       self.assertRaises(IndexError, cl.GetNextSubmittedJob)
@@ -1311,10 +1345,10 @@ class TestParseModifyInstanceRequest(unittest.TestCase):
                   self.assertEqual(op.nics, nics)
                   self.assertEqual(op.disks, disks)
                   self.assertEqual(op.disk_template, disk_template)
-                  self.assertFalse(hasattr(op, "remote_node"))
-                  self.assertFalse(hasattr(op, "os_name"))
-                  self.assertFalse(hasattr(op, "force_variant"))
-                  self.assertFalse(hasattr(op, "dry_run"))
+                  self.assertTrue(op.remote_node is None)
+                  self.assertTrue(op.os_name is None)
+                  self.assertFalse(op.force_variant)
+                  self.assertFalse(op.dry_run)
 
                   self.assertRaises(IndexError, cl.GetNextSubmittedJob)
 
@@ -1337,7 +1371,7 @@ class TestParseModifyInstanceRequest(unittest.TestCase):
 
     for i in ["hvparams", "beparams", "osparams", "force", "nics", "disks",
               "disk_template", "remote_node", "os_name", "force_variant"]:
-      self.assertFalse(hasattr(op, i))
+      self.assertTrue(hasattr(op, i))
 
 
 class TestParseInstanceReinstallRequest(testutils.GanetiTestCase):
@@ -1497,8 +1531,8 @@ class TestInstanceReplaceDisks(unittest.TestCase):
     self.assertTrue(isinstance(op, opcodes.OpInstanceReplaceDisks))
     self.assertEqual(op.instance_name, name)
     self.assertEqual(op.mode, constants.REPLACE_DISK_AUTO)
-    self.assertFalse(hasattr(op, "iallocator"))
-    self.assertFalse(hasattr(op, "disks"))
+    self.assertTrue(op.iallocator is None)
+    self.assertEqual(op.disks, [])
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
 
   def testNoDisks(self):
@@ -1552,7 +1586,7 @@ class TestGroupModify(unittest.TestCase):
       self.assertTrue(isinstance(op, opcodes.OpGroupSetParams))
       self.assertEqual(op.group_name, name)
       self.assertEqual(op.alloc_policy, policy)
-      self.assertFalse(hasattr(op, "dry_run"))
+      self.assertFalse(op.dry_run)
       self.assertRaises(IndexError, cl.GetNextSubmittedJob)
 
   def testUnknownPolicy(self):
@@ -1584,10 +1618,44 @@ class TestGroupModify(unittest.TestCase):
 
     self.assertTrue(isinstance(op, opcodes.OpGroupSetParams))
     self.assertEqual(op.group_name, name)
-    self.assertFalse(hasattr(op, "alloc_policy"))
-    self.assertFalse(hasattr(op, "dry_run"))
+    self.assertTrue(op.alloc_policy is None)
+    self.assertFalse(op.dry_run)
     self.assertRaises(IndexError, cl.GetNextSubmittedJob)
 
+  def testCustomParamRename(self):
+    clfactory = _FakeClientFactory(_FakeClient)
+
+    name = "groupie"
+    data = {
+      "custom_diskparams": {},
+      "custom_ipolicy": {},
+      "custom_ndparams": {},
+      }
+
+    handler = _CreateHandler(rlib2.R_2_groups_name_modify, [name], {}, data,
+                             clfactory)
+    job_id = handler.PUT()
+
+    cl = clfactory.GetNextClient()
+    self.assertRaises(IndexError, clfactory.GetNextClient)
+
+    (exp_job_id, (op, )) = cl.GetNextSubmittedJob()
+    self.assertEqual(job_id, exp_job_id)
+
+    self.assertTrue(isinstance(op, opcodes.OpGroupSetParams))
+    self.assertEqual(op.diskparams, {})
+    self.assertEqual(op.ipolicy, {})
+    self.assertEqual(op.ndparams, {})
+
+    self.assertRaises(IndexError, cl.GetNextSubmittedJob)
+
+    # Define both
+    data["diskparams"] = {}
+    assert "diskparams" in data and "custom_diskparams" in data
+    handler = _CreateHandler(rlib2.R_2_groups_name_modify, [name], {}, data,
+                             clfactory)
+    self.assertRaises(http.HttpBadRequest, handler.PUT)
+
 
 class TestGroupAdd(unittest.TestCase):
   def test(self):
@@ -1646,7 +1714,7 @@ class TestGroupAdd(unittest.TestCase):
 
     self.assertTrue(isinstance(op, opcodes.OpGroupAdd))
     self.assertEqual(op.group_name, name)
-    self.assertFalse(hasattr(op, "alloc_policy"))
+    self.assertTrue(op.alloc_policy is None)
     self.assertFalse(op.dry_run)
 
   def testLegacyName(self):
@@ -1670,7 +1738,7 @@ class TestGroupAdd(unittest.TestCase):
 
     self.assertTrue(isinstance(op, opcodes.OpGroupAdd))
     self.assertEqual(op.group_name, name)
-    self.assertFalse(hasattr(op, "alloc_policy"))
+    self.assertTrue(op.alloc_policy is None)
     self.assertTrue(op.dry_run)
 
 
@@ -1694,7 +1762,7 @@ class TestNodeRole(unittest.TestCase):
         self.assertTrue(isinstance(op, opcodes.OpNodeSetParams))
         self.assertEqual(op.node_name, "node-z")
         self.assertFalse(op.force)
-        self.assertFalse(hasattr(op, "dry_run"))
+        self.assertFalse(op.dry_run)
 
         if role == rlib2._NR_REGULAR:
           self.assertFalse(op.drained)
@@ -1746,7 +1814,7 @@ class TestClusterInfo(unittest.TestCase):
 
     def QueryClusterInfo(self):
       assert self.cluster_info is None
-      self.cluster_info = object()
+      self.cluster_info = {}
       return self.cluster_info
 
   def test(self):
@@ -1763,18 +1831,27 @@ class TestInstancesMultiAlloc(unittest.TestCase):
     clfactory = _FakeClientFactory(_FakeClient)
     data = {
       "instances": [{
-        "instance_name": "bar",
+        "name": "bar",
         "mode": "create",
+        "disks": [{"size": 1024}],
+        "disk_template": "plain",
+        "nics": [{}],
         }, {
-        "instance_name": "foo",
+        "name": "foo",
         "mode": "create",
+        "disks": [{"size": 1024}],
+        "disk_template": "drbd",
+        "nics": [{}],
         }],
       }
     handler = _CreateHandler(rlib2.R_2_instances_multi_alloc, [], {}, data,
                              clfactory)
+
     (body, _) = handler.GetPostOpInput()
-    self.assertTrue(compat.all([inst["OP_ID"] == handler.POST_OPCODE.OP_ID
-                                for inst in body["instances"]]))
+
+    self.assertTrue(compat.all(
+      [isinstance(inst, opcodes.OpInstanceCreate) for inst in body["instances"]]
+    ))
 
 
 class TestPermissions(unittest.TestCase):
