@@ -39,9 +39,11 @@ import testutils
 
 KNOWN_UNUSED_LUXI = compat.UniqueFrozenset([
   luxi.REQ_SUBMIT_MANY_JOBS,
+  luxi.REQ_SUBMIT_JOB_TO_DRAINED_QUEUE,
   luxi.REQ_ARCHIVE_JOB,
   luxi.REQ_AUTO_ARCHIVE_JOBS,
   luxi.REQ_CHANGE_JOB_PRIORITY,
+  luxi.REQ_PICKUP_JOB,
   luxi.REQ_QUERY_EXPORTS,
   luxi.REQ_QUERY_CONFIG_VALUES,
   luxi.REQ_QUERY_NETWORKS,
@@ -112,8 +114,6 @@ class TestVerifyOpResult(unittest.TestCase):
   def testNoResultCheck(self):
     vor = rapi.testutils.VerifyOpResult
 
-    assert opcodes.OpTestDummy.OP_RESULT is None
-
     vor(opcodes.OpTestDummy.OP_ID, None)
 
 
