@@ -289,6 +289,8 @@ class TestDRBD8Status(testutils.GanetiTestCase):
       testutils.TestDataFilename("proc_drbd83_sync_krnl2.6.39.txt")
     proc84_data = testutils.TestDataFilename("proc_drbd84.txt")
     proc84_sync_data = testutils.TestDataFilename("proc_drbd84_sync.txt")
+    proc84_emptyfirst_data = \
+      testutils.TestDataFilename("proc_drbd84_emptyfirst.txt")
 
     self.proc80ev_data = \
       testutils.TestDataFilename("proc_drbd80-emptyversion.txt")
@@ -303,6 +305,8 @@ class TestDRBD8Status(testutils.GanetiTestCase):
     self.drbd_info84 = drbd.DRBD8Info.CreateFromFile(filename=proc84_data)
     self.drbd_info84_sync = \
       drbd.DRBD8Info.CreateFromFile(filename=proc84_sync_data)
+    self.drbd_info84_emptyfirst = \
+      drbd.DRBD8Info.CreateFromFile(filename=proc84_emptyfirst_data)
 
   def testIOErrors(self):
     """Test handling of errors while reading the proc file."""
@@ -329,6 +333,7 @@ class TestDRBD8Status(testutils.GanetiTestCase):
     self.failUnless(not self.drbd_info.HasMinorStatus(9))
     self.failUnless(not self.drbd_info83.HasMinorStatus(9))
     self.failUnless(not self.drbd_info80e.HasMinorStatus(3))
+    self.failUnless(not self.drbd_info84_emptyfirst.HasMinorStatus(0))
 
   def testLineNotMatch(self):
     """Test wrong line passed to drbd_info.DRBD8Status"""
@@ -344,7 +349,8 @@ class TestDRBD8Status(testutils.GanetiTestCase):
 
   def testMinor1(self):
     """Test connected, secondary device"""
-    for info in [self.drbd_info, self.drbd_info83, self.drbd_info84]:
+    for info in [self.drbd_info, self.drbd_info83, self.drbd_info84,
+                 self.drbd_info84_emptyfirst]:
       stats = info.GetMinorStatus(1)
       self.failUnless(stats.is_in_use)
       self.failUnless(stats.is_connected and stats.is_secondary and
@@ -353,13 +359,15 @@ class TestDRBD8Status(testutils.GanetiTestCase):
   def testMinor2(self):
     """Test unconfigured device"""
     for info in [self.drbd_info, self.drbd_info83,
-                 self.drbd_info80e, self.drbd_info84]:
+                 self.drbd_info80e, self.drbd_info84,
+                 self.drbd_info84_emptyfirst]:
       stats = info.GetMinorStatus(2)
       self.failIf(stats.is_in_use)
 
   def testMinor4(self):
     """Test WFconn device"""
-    for info in [self.drbd_info, self.drbd_info83, self.drbd_info84]:
+    for info in [self.drbd_info, self.drbd_info83,
+                 self.drbd_info84, self.drbd_info84_emptyfirst]:
       stats = info.GetMinorStatus(4)
       self.failUnless(stats.is_in_use)
       self.failUnless(stats.is_wfconn and stats.is_primary and
@@ -368,7 +376,8 @@ class TestDRBD8Status(testutils.GanetiTestCase):
 
   def testMinor6(self):
     """Test diskless device"""
-    for info in [self.drbd_info, self.drbd_info83, self.drbd_info84]:
+    for info in [self.drbd_info, self.drbd_info83,
+                 self.drbd_info84, self.drbd_info84_emptyfirst]:
       stats = info.GetMinorStatus(6)
       self.failUnless(stats.is_in_use)
       self.failUnless(stats.is_connected and stats.is_secondary and
@@ -419,12 +428,18 @@ class TestDRBD8Construction(testutils.GanetiTestCase):
         filename=testutils.TestDataFilename("proc_drbd84.txt"))
 
     self.test_unique_id = ("hosta.com", 123, "host2.com", 123, 0, "secret")
+    self.test_dyn_params = {
+      constants.DDP_LOCAL_IP: "192.0.2.1",
+      constants.DDP_LOCAL_MINOR: 0,
+      constants.DDP_REMOTE_IP: "192.0.2.2",
+      constants.DDP_REMOTE_MINOR: 0,
+    }
 
   @testutils.patch_object(drbd.DRBD8, "GetProcInfo")
   def testConstructionWith80Data(self, mock_create_from_file):
     mock_create_from_file.return_value = self.proc80_info
 
-    inst = drbd.DRBD8Dev(self.test_unique_id, [], 123, {})
+    inst = drbd.DRBD8Dev(self.test_unique_id, [], 123, {}, self.test_dyn_params)
     self.assertEqual(inst._show_info_cls, drbd_info.DRBD83ShowInfo)
     self.assertTrue(isinstance(inst._cmd_gen, drbd_cmdgen.DRBD83CmdGenerator))
 
@@ -432,7 +447,7 @@ class TestDRBD8Construction(testutils.GanetiTestCase):
   def testConstructionWith83Data(self, mock_create_from_file):
     mock_create_from_file.return_value = self.proc83_info
 
-    inst = drbd.DRBD8Dev(self.test_unique_id, [], 123, {})
+    inst = drbd.DRBD8Dev(self.test_unique_id, [], 123, {}, self.test_dyn_params)
     self.assertEqual(inst._show_info_cls, drbd_info.DRBD83ShowInfo)
     self.assertTrue(isinstance(inst._cmd_gen, drbd_cmdgen.DRBD83CmdGenerator))
 
@@ -440,7 +455,7 @@ class TestDRBD8Construction(testutils.GanetiTestCase):
   def testConstructionWith84Data(self, mock_create_from_file):
     mock_create_from_file.return_value = self.proc84_info
 
-    inst = drbd.DRBD8Dev(self.test_unique_id, [], 123, {})
+    inst = drbd.DRBD8Dev(self.test_unique_id, [], 123, {}, self.test_dyn_params)
     self.assertEqual(inst._show_info_cls, drbd_info.DRBD84ShowInfo)
     self.assertTrue(isinstance(inst._cmd_gen, drbd_cmdgen.DRBD84CmdGenerator))
 
